/*---------------------------------------------------------------------------
File Name:      BasicInit.h
Author:         Alexandre Belloni
Last Modified:  09-SEP-05

Description:    We will gather some basic libraries that are commonly used
                by many files, define some auxiliary variables types,  and
                other "initialization" steps...
-----------------------------------------------------------------------------*/

//---------------------------------------------------------------------------
#ifndef BasicInitH
#define BasicInitH
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//              Definitions
//---------------------------------------------------------------------------


#define NotRelease      0

#define PrintVisual     0

#define TRUE            1
#define FALSE           0
#define TimeMode        0  	//    Value     Implementation
							//      0       no time computation is used.
							//      1       time for each step is computed and used to update RedCostBar


#define memalloc( object , type , length )   object = new type [ length ]

//---------------------------------------------------------------------------
//              Include Files
//---------------------------------------------------------------------------

//#include <values.h>
#include <limits.h>
#include <math.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#if ( TimeMode )
#  if defined(_MSC_VER)
#     include <sys\timeb.h>
#  endif
#endif


//---------------------------------------------------------------------------
//              Types
//---------------------------------------------------------------------------

typedef double          hpReal;                 // "high precision" real numbers
typedef hpReal          *hpVec;                 // vectors
typedef hpVec           *hpMat;                 // matrix
#define hpZero          0.0e0
#define hpInf           1.0e15

typedef double          Real;                   // real numbers
typedef Real            *RealVec;               // vectors of real numbers
typedef RealVec         *RealMat;               // matrix of real numbers

typedef unsigned char   boolean;                // binary variables {0,1}
typedef boolean         *boolVec;               // binary vector
typedef boolVec         *boolMat;               // binary matrix

typedef unsigned int    Index;                  // Index Variables
typedef Index           *IndexVec;              // vector of indices
typedef IndexVec        *IndexMat;              // matrix of indices

typedef long int        IntNumber;              // integer numbers
typedef IntNumber       *IntVec;                // vector of integers
typedef IntVec          *IntMat;                // matrix of integers


#endif

//---------------------------------------------------------------------------
//   End of File BasicInit.h (there should be no code after this comment)
//---------------------------------------------------------------------------

