//---------------------------------------------------------------------------

#ifndef MatrixOperationsH
#define MatrixOperationsH
//---------------------------------------------------------------------------

#include "BasicInit.h"
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

static inline void TriangularSystem ( const  hpVec *L,  register hpVec b,
                                        hpVec x, Index Size );

static inline void TriangularSystemTranspose ( const  hpVec *L,  register hpVec b,
                                        hpVec x, Index Size );

static inline void TriangularSystem ( const  hpVec *L,  register hpVec b,
                                        hpVec x, Index Size, IndexVec Set );

static inline void TriangularSystemTranspose ( const  hpVec *L,  register hpVec b,
                                        hpVec x, Index Size, IndexVec Set );

static inline void GivenRotationPostMultiplication( hpReal c,  hpReal s, hpReal &Mi, hpReal &Mk);

//---------------------------------------------------------------------------
static inline void TriangularSystem (  hpVec *L,  hpVec b,
                                        hpVec x, Index Size )
{
// Solve L x = b for L lower triangular with non-zero elements on the diagonal.

for (register Index i = 0; i < Size ; ){
        register hpReal res  = 0;
        register hpVec  aux  = x;      // aux is a vector equal to the current x
        register hpVec  Lrow = *(L++); // Lrow pointer to L[i]*, ith row of L.
        for (register Index j = i++; j-- ; ) // i++ here instead of "regular" place is slightly faster
                res += (*(Lrow++))  *   (*(aux++));

        *(aux) = ( *(b++) - res  ) / (*Lrow) ; // x[i] = (b[i] - SUM...  )/ L[i][i];
        }
}

//---------------------------------------------------------------------------
static inline void TriangularSystemTranspose (  hpVec *L,  hpVec b,
                                        hpVec x, Index Size )
{
// Solve L^T x = b for L lower triangular with non-zero elements on the diagonal.
register Index i = Size--;
b += Size;
L += Size;
x += Size;

for ( ; i ; ){
        register hpReal res  = 0;
        register hpVec  aux  = x;                       // aux is a vector equal to the current x
        register hpVec  *Lrow = L;                      // Lrow pointer to L[i]*, ith row of L.
        for (register Index j = Size - (--i) ; j-- ; )// --i here instead of "regular" place is slightly faster
                res += (*(Lrow--))[i]  *   (*(aux--));

        *(aux) = ( *(b--) - res  ) / (*Lrow)[i] ;       // x[i] = (b[i] - SUM...  )/ L[i][i];
        }
}

//---------------------------------------------------------------------------
static inline void TriangularSystem ( const  hpVec *L,  register hpVec b,
                                        hpVec x, Index Size, IndexVec Set )
{
// Solve L x = b for L lower triangular with non-zero elements on the diagonal.

for (register Index i = 0; i < Size ; ){
        register hpReal res  = 0;
        register hpVec  aux  = x;      // aux is a vector equal to the current x
        register hpVec  Lrow = *(L++); // Lrow pointer to L[i]*, ith row of L.
        for (register Index j = i++; j-- ; ) // i++ here instead of "regular" place is slightly faster
                res += (*(Lrow++))  *   (*(aux++));

        *(aux) = ( b[*(Set++)] - res  ) / (*Lrow) ; // x[i] = (b[i] - SUM...  )/ L[i][i];
        }
}

//---------------------------------------------------------------------------
static inline void TriangularSystemTranspose ( const  hpVec *L,  register hpVec b,
                                        hpVec x, Index Size, IndexVec Set )
{
// Solve L^T x = b for L lower triangular with non-zero elements on the diagonal.
register Index i = Size--;
Set += Size;
L += Size;
x += Size;

for ( ; i ; ){
        register hpReal res  = 0;
        register hpVec  aux  = x;      // aux is a vector equal to the current x
        register const hpVec  *Lrow = L; // Lrow pointer to L[i]*, ith row of L.
        for (register Index j = Size-- - (--i) ; j-- ; ) // i-- here instead of "regular" place is slightly faster
                res += (*(Lrow--))[i]  *   (*(aux--));

        *(aux) = ( b[*(Set--)] - res  ) / (*Lrow)[i] ; // x[i] = (b[i] - SUM...  )/ L[i][i];
        }
}

//GivenRotationPostMultiplication( Cos,  Sin, r1[ place_i ], r1[ j ]);
//--------------------------------------------------------------------
static inline void GivenRotationPostMultiplication( hpReal c,  hpReal s, hpReal &Mi, hpReal &Mk)
{
register const hpReal tau1 = Mi;
register const hpReal tau2 = Mk;

Mi = c * tau1 - s * tau2;
Mk = s * tau1 + c * tau2;

}

//--------------------------------------------------------------------

static inline void FastGivenRotation (  )
{
}
















#endif
