
#include <math.h>
#include "SVMRBFreal.h"
#include "QPtime.h"


//---------------------------------------------------------------------------

#pragma hdrstop

//---------------------------------------------------------------------------

#pragma argsused
//---------------------------------------------------------------------------


void PrintInstructions();

void SolveSVM_C_real( char *input_file, FILE *out_info, Index type_prob, hpReal prob_par1, hpReal prob_par2,
						Index Bias, Index type_kernel, hpReal kernel_par1, hpReal kernel_par2 );

void SolveSVM_Path_real( char *input_file, FILE *out_info, Index type_type, hpReal prob_par1, hpReal prob_par2,
						Index Bias, Index type_kernel, hpReal kernel_par1, hpReal kernel_par2 );

//---------------------------------------------------------------------------
void SolveSVM_C_real( char *input_file, FILE *out_info, Index type_prob, hpReal prob_par1, hpReal prob_par2,
						Index Bias, Index type_kernel, hpReal kernel_par1, hpReal kernel_par2 )
{
class SVMRBFreal SVMQP = SVMRBFreal( input_file );
switch ( type_kernel ){
		case 1: // linear
				SVMQP.SetKernel ( 1, 0.0e0, 0.0e0, 0.0e0 );
				break;
		case 2: // Gaussian
				if ( kernel_par1 <= 0 ){
						printf("Invalid value of sigma = %lf. It was reset to sigma = 1.\n", kernel_par1);
						kernel_par1 = 1.0e0;
						}
				SVMQP.SetKernel ( 2, kernel_par1, 0.0e0, 0.0e0 );
				break;
		case 3: // Inhomogeneous polynomial
				if ( kernel_par1 <= 0 ){
						printf("Invalid value of poly_d = %lf. It was reset to poly_d = 2.\n", kernel_par1);
						kernel_par1 = 2.0e0;
						}
				if ( kernel_par2 < 0 ){
						printf("Invalid value of poly_c = %lf. It was reset to poly_c = 1.\n", kernel_par1);
						kernel_par2 = 1.0e0;
						}
				SVMQP.SetKernel ( 2, 0.0e0, kernel_par1, kernel_par2 );

				break;
		default:
				break;
		}

SVMQP.SetC    ( prob_par1 );
SVMQP.SetNoBias ( !Bias );
SVMQP.SolveSVM();

SVMQP.PrintCurrentSolutionSparse(out_info);
fflush(out_info);
}


//---------------------------------------------------------------------------
//
// c:\> QP-svmCall  InputDataFile  OutputDataFile  TypeSolver  prob_par1  Par2  Par3
//
//------------------------
void PrintInstructions(){
		printf("Solver PathSVM-QP v1.01\n");
		printf(" Command line: (Note: parameters are read in order)\n");
		printf(" \n \n");
		printf(" PathSVM [what] [input_file] [test_file]  \n");
		printf("\n what: == 1  to solve the QP associated with input_file at the initial value\n             ([test_file] is ignored)\n");
		printf("\n what: == 2  to compute the regularized Path associated with input_file \n             From initial value until final value \n             ([test_file] can be used, see PathParameter.par file)\n");
		printf("\n what: == 3  to split the file [input_file] into two sets \n             (training data and test data)\n");
		printf("\n input_file: name of the data file\n");
		printf("\n test_file:  name of the test data file (if required by the user)\n");
		}
//------------------------
int main(int argc, char * argv[])
{
long what_par;
FILE *out_info, *check_file;
 double TotalTime=QPCpuTime();
 double QPTime=QPCpuTime();
if ( argc == 1 ){
		PrintInstructions();
		return 0;
		}

out_info = fopen("QP-svm.out","w");

what_par = long( atof(argv[1]) );
if ( what_par < 1 || what_par > 3 ) {
	printf("what parameter = %ld is invalid\n", what_par);
	fprintf(out_info,"what parameter = %ld is invalid\n", what_par);
	fclose(out_info);
	return 0;
	}
else	{
	printf("what parameter = %ld \n", what_par);
	fprintf(out_info,"what parameter = %ld \n", what_par);
	}
check_file = fopen(argv[2],"r");
if ( check_file == NULL ){
		printf("what == %ld \nInput file not found (%s). Please verify the command line\n",argv[2]);
		fprintf(out_info,"Input file not found (%s). Please verify the command line\n",argv[2]);
		fclose(out_info);
		fclose(check_file);
		return 0;
		}
fclose(check_file);

// Initialize class with [input_file]
printf("Reading file %s. Please wait...\n",argv[2]);
fprintf(out_info,"Reading file %s. Please wait...\n",argv[2]);

SVMRBFreal SVMQP = SVMRBFreal( argv[2] );

printf("Done reading input file\n");
fprintf(out_info,"Done reading input file\n");



// if no TestData was provided
if ( argc < 4 && SVMQP.GetDataTestMode() >= 0 ) {
	SVMQP.SetDataTestMode( -1 );
	printf("Test file was not provided. No test will be performed \n");
	fprintf(out_info,"Test file was not provided. No test will be performed \n");
	}

// In case Test data file is relevant...
if ( what_par == 2 && SVMQP.GetDataTestMode() >= 0 ) {

	check_file = fopen(argv[3],"r");
	if ( check_file == NULL ){
		printf("Test file %s not found. Please verify the command line\n",argv[3]);
		fprintf(out_info,"Test file %s not found. Please verify the command line\n",argv[3]);
		fclose(out_info);
		fclose(check_file );
		return 0;
		}
	else	{
		// Need to load file...
		fclose(check_file);
		printf("Reading test file %s \n",argv[3]);
		fprintf(out_info,"Reading test file %s \n",argv[3]);
		SVMQP.LoadTestData( argv[3] );
		printf("Done loading test file %s \n",argv[3]);
		fprintf(out_info,"Done loading test file %s \n",argv[3]);
		}
	}

 QPTime=QPCpuTime();
switch ( what_par ) {
	case 1: // Solving QP
		printf("Solving QP C=%lf \n", SVMQP.GetInitParameter());
		fprintf(out_info,"Solving QP C=%lf \n", SVMQP.GetInitParameter());

		SVMQP.SolveQP ( SVMQP.GetInitParameter() );

		printf("Finished Solving QP\n");
		fprintf(out_info,"Finished Solving QP\n");
		break;
	case 2: // Solving Path
		printf("Computing regularized Path [ %lf , %lf ] \n", SVMQP.GetInitParameter(), SVMQP.GetFinalParameter());
		fprintf(out_info,"Computing regularized Path [ %lf , %lf ] \n", SVMQP.GetInitParameter(), SVMQP.GetFinalParameter());

		if ( SVMQP.GetDataTestMode() < 0 ) {
			printf("No test file will be used.\n");
			fprintf(out_info,"No test file will be used.\n");
			}
		if ( SVMQP.GetCurrentPathMode() < 2 ){
			printf("Exact mode\n");
			fprintf(out_info,"Exact mode\n");
			SVMQP.SolvePath ( SVMQP.GetInitParameter(), SVMQP.GetFinalParameter() );
			}
		else {
			printf("Approximate mode\n");
			fprintf(out_info,"Approximate mode\n");
			SVMQP.SolveApproximatePath (SVMQP.GetInitParameter(), SVMQP.GetFinalParameter());
			}

		printf("Finished Computing Path\n");
		fprintf(out_info,"Finished Computing Path\n");
		break;

	case 3: // Splitting Data
		printf("Splitting Data from input file %s. \n Total number of points: %ld .... Train Data Set %ld  Test Data Set %ld.\n", argv[2], SVMQP.GetNumberPoints(), SVMQP.GetNumberPoints()- IntNumber ( 0.15*SVMQP.GetNumberPoints() ), IntNumber(0.15*SVMQP.GetNumberPoints()));
		fprintf(out_info,"Splitting Data from input file %s. \n Total number of points: %ld .... Train Data Set %ld  Test Data Set %ld.\n", argv[2], SVMQP.GetNumberPoints(), SVMQP.GetNumberPoints()- IntNumber ( 0.15*SVMQP.GetNumberPoints() ), IntNumber(0.15*SVMQP.GetNumberPoints()));
		SVMQP.SplitDataSet( TRUE , IntNumber ( 0.15*SVMQP.GetNumberPoints() ) );
		printf("Done splitting file\n");
		fprintf(out_info,"Done splitting file\n");
		break;
	default:
		printf("what parameter = %ld is invalid\n", what_par);
		fprintf(out_info,"what parameter = %ld is invalid\n", what_par);
		break;

	}


 if ( what_par == 1 || what_par == 2 ) {
#if (TimeMode)
	printf("%s_%.0f_%.0f & %d & %d & %d & %d & %.0f & %0.f  & %d & %d & %d \\\\ \n", argv[2], SVMQP.GetInitParameter()  ,SVMQP.GetFinalParameter(),
				SVMQP.Getn(), SVMQP.GetNumberFeatures(), SVMQP.GetNs(), SVMQP.GetNc(),
				SVMQP.GetTotalTime(), SVMQP.GetQPTime(),
				SVMQP.GetNumBreakPoints(),
				SVMQP.GetCountMajorLoop(), SVMQP.GetCountCallsReducedCost() );
	fprintf( out_info, "%s_%.0f_%.0f & %d & %d & %d & %d & %.0f & %0.f  & %d & %d & %d \\\\ \n", argv[2], SVMQP.GetInitParameter()  ,SVMQP.GetFinalParameter(),
				SVMQP.Getn(), SVMQP.GetNumberFeatures(), SVMQP.GetNs(), SVMQP.GetNc(),
				SVMQP.GetTotalTime(), SVMQP.GetQPTime(),
				SVMQP.GetNumBreakPoints(),
				SVMQP.GetCountMajorLoop(), SVMQP.GetCountCallsReducedCost() );
#else
	TotalTime=QPCpuTime()-TotalTime;
        QPTime=QPCpuTime()-QPTime;
	printf("%s_%.0f_%.0f & %d & %d & %d & %d & %.0f & %0.f  & %d & %d & %d \\\\ \n", argv[2], SVMQP.GetInitParameter()  ,SVMQP.GetFinalParameter(),
				SVMQP.Getn(), SVMQP.GetNumberFeatures(), SVMQP.GetNs(), SVMQP.GetNc(),
				TotalTime, QPTime,
				SVMQP.GetNumBreakPoints(),
				SVMQP.GetCountMajorLoop(), SVMQP.GetCountCallsReducedCost() );
	fprintf( out_info, "%s_%.0f_%.0f & %d & %d & %d & %d & %.0f & %0.f  & %d & %d & %d \\\\ \n", argv[2], SVMQP.GetInitParameter()  ,SVMQP.GetFinalParameter(),
				SVMQP.Getn(), SVMQP.GetNumberFeatures(), SVMQP.GetNs(), SVMQP.GetNc(),
				TotalTime, QPTime,
				SVMQP.GetNumBreakPoints(),
				SVMQP.GetCountMajorLoop(), SVMQP.GetCountCallsReducedCost() );
#endif
	}
printf("Program finished.\n");
fflush(out_info);
fclose(out_info);

}






