//---------------------------------------------------------------------------
#include "QP_MemRedCostPath.h"
#include "QPtime.h"
//---------------------------------------------------------------------------
TQPMemRedCostPath::TQPMemRedCostPath()
		:TQPclassPath()
{
CurrentAllocQ           = 0;
MaxNumberofQcolumns     = 0;
BlockAllocQ             = 0;
Nq                      = 0;
tmpVecM                 = NULL;
tmpIndVec               = NULL;
#if ( MemStyleQ == 1 || MemStyleQ == 2 )
		QQ              = NULL;
		QIndex          = NULL;
		InvQIndex       = NULL;
#endif

Iop                     = NULL;
Icp                     = NULL;
CounterNonNegRedCost    = NULL;
QC_redcost              = NULL;

#if     ( ReducedCostMode == 3 )
		Iopp            = NULL;
		Icpp            = NULL;
#endif
CountMajorLoop          = 0;
CountCallsReducedCost   = 0;

NumBreakPoints          = 0;
NumJumps		        = 0;
LowerReducedCost        = NULL;
LowerSlope              = NULL;
QisAlpha                = NULL;
MinRatioStep2Pos        = hpZero;
MinRatioStep2Neg        = hpZero;
NumberOfTies            = 0;
f_deriv                 = hpZero;
InitParameter           = 1.0e0;
FinalParameter          = 1.0e0;
NoPath                  = 0;
NcPath                  = 0;
IoPath                  = NULL;
IcPath                  = NULL;
MaxSizeNoPath           = 0;
MaxSizeNcPath           = 0;
BPtoUpdates             = 0;
MaxToJump				= 4;
CountToJump				= 0;
OldTheta 				= -1.0e0;

ParametricMode 			= FALSE;

BestTestValue 			= hpInf;
BestThetaTestValue 		= hpInf;
LastThetaTested 		= hpInf;
NumberOfTests 			= 100;
BreakPointIntervals 	= 30;//10;//30;

quantile_current 		= 0.25;
tau_value 				= 0.1;
DiffAlpha = 0;

DataTestMode = -1; // Not in use

}

//---------------------------------------------------------------------------
void TQPMemRedCostPath::TQPMemRedCostPathInit( Index Max_n, Index n, hpVec Y0, hpVec V0, hpVec C0 )
{
TQPclassPath::TQPclassPathInit( Max_n, n, Y0, V0, C0 );

CurrentAllocQ           = 0;
MaxNumberofQcolumns     = Max_n;
BlockAllocQ             = min( MaxNumberofQcolumns, long(51) ) - 1;
Nq                      = 0;

memalloc( tmpVecM  , hpReal, 2*Max_n );
memalloc( tmpIndVec, Index , 2*Max_n );

// Memory allocation for Q and L
#if ( MemStyleQ == 1 || MemStyleQ == 2 )
		memalloc( QQ , hpVec, MaxNumberofQcolumns);
        memalloc( QIndex, Index, Max_n );
        memalloc( InvQIndex, Index, MaxNumberofQcolumns );
		VecInit(QIndex, Max_n, Max_n );
        VecInit(InvQIndex, Max_n, MaxNumberofQcolumns );
#endif

// Memory and Initialization for Reduce Cost...
memalloc( Iop, Index, Max_n);
memalloc( Icp, Index, Max_n);
memalloc( CounterNonNegRedCost, IntNumber, Max_n);
memalloc( QC_redcost, hpReal, Max_n );
Nop = n;
Ncp = 0;
for (register Index k = 0 ; k < Nop ; k++ ) Iop[k] = k;
for (register Index k = Nop ; k < Max_n ; k++ ) Iop[k] = Max_n;
VecInit(Icp, Index( Max_n ), Max_n );
VecInit(CounterNonNegRedCost,IntNumber( 0 ), Max_n );
VecInit(QC_redcost, hpZero, Max_n );

#if     ( ReducedCostMode == 3 )
		memalloc( Iopp, Index, Max_n);
        memalloc( Icpp, Index, Max_n);
		MaxSizeNcp = 80;
        MaxSizeNop = 80;
		Nop = MaxSizeNop;
		Ncp = 0;
		for (register Index k = 0 ; k < Nop ; k++ ) Iop[k] = k;
		for (register Index k = Nop ; k < Max_n ; k++ ) Iop[k] = Max_n;
		for (register Index k = Nop ; k < Getn() ; k++ ) Iopp[k-Nop] = k;
		for (register Index k = Getn() - Nop ; k < Max_n ; k++ ) Iopp[k] = Max_n;
        Nopp = n - Nop;
        Ncpp = 0;
        MajorLoop = TRUE;
		VecInit(Icpp, Index( Max_n ), Max_n );
        CountToIoppp = 3;
		CountToIcppp = 3;
#endif


#if ( ReducedCostMode )
		RedCostBar              = -1.0e0;
		CountToIopp             = 3;//1;//50;
		CountToIcpp             = 3;//1;//50;
#else
		RedCostBar              = -1.0e0;
		CountToIopp             = Max_n;
		CountToIcpp             = Max_n;
#endif

memalloc( LowerReducedCost , hpReal, Max_n );
VecInit ( LowerReducedCost , hpZero, Max_n );
memalloc( LowerSlope , hpReal, Max_n );
VecInit ( LowerSlope , hpZero, Max_n );
memalloc( QisAlpha   , hpReal, Max_n );
VecInit ( QisAlpha   , hpZero, Max_n );
MaxSizeNoPath           = 400;   //400
MaxSizeNcPath           = 400;   //400
BPtoUpdates             = 100;   //100
memalloc( IoPath     , Index, 2*MaxSizeNoPath);
memalloc( IcPath     , Index, 2*MaxSizeNcPath);
VecInit( IoPath     , Max_n, 2*MaxSizeNoPath);
VecInit( IcPath     , Max_n, 2*MaxSizeNcPath);
}


//-----------------------------------------------------------------------
void    TQPMemRedCostPath::SolveQP ( hpReal Parameter )
{

ParametricMode = FALSE;

// Initialization of Base...
TQPclassPath::SetTheta( Parameter );

// While we need to introduce variables in Is....
TQPclassPath::SolveQP();


}
//-----------------------------------------------------------------------
QPErrorMsg TQPMemRedCostPath::CheckingParametricStep()
{

if ( NextTheta <= OldTheta ) {
	// Two breakpoints that do not changed the value of theta
	// Need to Jump
	TQPclassPath::PrintErrorMsg("Parameter did not increase.");

	CountToJump++;

	OldTheta = NextTheta;
	if ( CountToJump >= MaxToJump ) {

		// solve QP for new parameter
		hpReal JumpTheta = GetAbsTolError() + 1.0005*max( NextTheta, max( TQPclassPath::GetTheta(), OldTheta ) );
		ParametricMode = FALSE;

		// need to check if it is worth...
				#if ( ReducedCostMode == 3 )
						Nop = 0;
						Nopp = GetNo();
						for ( register Index k = 0 ; k < GetNo() ; k++ )
								Iopp[k] = GetIo(k);
						Ncp = 0;
						Ncpp = GetNc();
						for ( register Index k = 0 ; k < GetNc() ; k++ )
								Icpp[k] = GetIc(k);
				#else
						Nop = GetNo();
						for ( register Index k = 0 ; k < GetNo() ; k++ )
								Iop[k] = GetIo(k);
						Ncp = GetNc();
						for ( register Index k = 0 ; k < GetNc() ; k++ )
								Icp[k] = GetIc(k);
				#endif


		TQPclassPath::JumpSolveQP( JumpTheta );
		ParametricMode = TRUE;


		if ( PathMode < 2 ){
			InitStep2Path();
			}
		else {
			InitApproxStep2Path();
			}
		CountToJump = 0;
		PrintErrorMsg("Jump on Theta");
		return MsgPotentialCycle;
		}
	return MsgOk;
	}
else if ( ParametricStatus == MsgOptimalityLost ){
	ParametricMode = FALSE;
	TQPMemRedCostPath::ReturnPathToOpt ();
	ParametricMode = TRUE;
	PrintErrorMsg("Reoptimized (due to numerical precision)\n");
	return MsgReoptimized;
	}
else {
	OldTheta = TQPclassPath::GetTheta();
	return MsgOk;
	}

}
//-----------------------------------------------------------------------
void    TQPMemRedCostPath::SolvePath ( hpReal InitVal, hpReal FinalVal )
{

InitParameter   = InitVal;
FinalParameter  = FinalVal;

if ( PathMode == 2 ) PathMode = 0; // could be zero or 1...

SolveQP( InitVal );

#if ( DebugMode >= 2 )

		// Verify all Data....
		if ( VerifyQPdata() == MsgQPIncorrect )
				PrintErrorMsg("QP data structure is not consistent: \n **** TERMINATE PROGRAM **** \n");

		if ( VerifyMinReducedCost () < -GetAbsTolError() )
				PrintErrorMsg("Optimality was lost...");

#endif

ParametricMode = TRUE;
#if ( TimeMode )
 InitTimePath ();
#endif

InitStep2Path();

fprintf( out_info, "BreakPoint , Theta , f_obj , Ns , Nc \n");
fprintf( out_info, " %d  %f  %f  %d  %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
#if ( PrintVisual == 0 )
		printf("BreakPoint , Theta , f_obj , Ns , Nc \n");
		printf(" %d  %f  %f  %d  %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
#endif

while ( GetNextBreakPoint () ) {

		// Check for Stuck and Jump if Necessary
		ParametricStatus = CheckingParametricStep();


		if ( ParametricStatus == MsgPotentialCycle ) {
			NumJumps++;
			fprintf( out_info, " %d  %f %f %d %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
			#if ( PrintVisual == 0 )
				printf( " BreakPoint = %d, Theta = %f, f_obj = %f, Ns = %d, Nc = %d \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
			#endif
			continue;
			}

		if ( ParametricStatus == MsgReoptimized ) {
			// Not a new breakpoints...
			// need to get next one...
			fprintf( out_info, " %d  %f %f %d %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
			#if ( PrintVisual == 0 )
				printf( " BreakPoint = %d, Theta = %f, f_obj = %f, Ns = %d, Nc = %d \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
			#endif
			continue;
			}
		NumBreakPoints++;

		//-----------------------------------------------------------------------------
		UpdateEveryIterationPath ();
		//-----------------------------------------------------------------------------

		#if ( TimeMode )
				TimeStart();
		#endif

		ParametricStatus = TQPclassPath::SetTheta( NextTheta );

		Compute_f_deriv();

		switch ( ParametricEvent ){

				case Is2Io:
						ParametricStatus = TQPclassPath::FromBaseToZero ( ParametricIndex );
						break;

				case Is2Ic:
						ParametricStatus = TQPclassPath::FromBaseToC ( ParametricIndex );
						break;

				case Io2Is:
						ParametricStatus = TQPclassPath::AddVariableToBase( ParametricIndex, Io2Is );
						break;

				case Ic2Is:
						ParametricStatus = TQPclassPath::AddVariableToBase( ParametricIndex, Ic2Is );
						break;

				case NoEvent:
						PrintErrorMsg("End of the Parametric path.");
						break;
				default:
						PrintErrorMsg("Parametric Event not well defined");
				}

		if ( ParametricStatus == MsgNumericalProb ){
				PrintErrorMsg("Numerical Instability Detected on Parametric Path: recomputing data structures.\n");
				ParametricStatus = TQPclassPath::RecomputeAll ();
				}

        if ( ParametricStatus == MsgFatal ){
				PrintErrorMsg("A fatal error occured when recomputing the Regularized path.\n");
				break;
				}

		#if ( TimeMode )
                TimeFinish();
				TimeAdd( &TimeUpdates );
		#endif

        if ( ParametricStatus != MsgOk ){
				PrintErrorMsg("Theta might have been wrongly set.");
                break;
				}

		#if ( DebugMode >= 2 )

				// Verify all Data....
				if ( VerifyQPdata() == MsgQPIncorrect ){
						PrintErrorMsg("QP data structure is not consistent: \n **** TERMINATE PROGRAM **** \n");
						break;
						}
				if ( VerifyMinReducedCost () < -GetAbsTolError() )
						PrintErrorMsg("Optimality was lost...");
		#endif

		// output the breakpoint info
		fprintf( out_info, " %d  %f %f %d %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );

		#if ( PrintVisual == 0 )
				printf( " BreakPoint = %d, Theta = %f, f_obj = %f, Ns = %d, Nc = %d \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
		#endif

		if ( EvaluatingTestData() ){
			break;
			}
		}

UpdateEveryIterationPath ();

#if (TimeMode)
fprintf( out_info, " Total Time: %f \n", TimeTotalAll );

#if ( PrintVisual == 0 )
		printf(" Total Time: %f \n", TimeTotalAll );
#endif
#endif
}


//-----------------------------------------------------------------------
void    TQPMemRedCostPath::PathMinRatioTestApproxStep2( hpReal Bound )
{


#if ( TimeMode )
		TimeStart();
#endif

Index index_k;
Index position_k = GetMax_n();
ParametricIndexStep2 = GetMax_n();
ParametricEventStep2 = NoEvent;

hpReal RedCost0 = hpZero;
hpReal RedCostNow = hpInf;

NextTheta       = Bound;
hpReal Delta    = NextTheta - GetTheta();
hpReal Delta_tmp = hpInf;

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );
CopyCurrentDirectionSparse( tmpVecM + GetNs() );

// Io ----------------------------------------
for ( register Index k = 0 ; k < NoPath ; k++ ){

		index_k  =  IoPath[ k ];

		hpReal Qis_d          = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
		LowerSlope[ index_k ] = Qis_d - Getb_deriv() * GetYi(index_k) + GetQicCc(index_k);

		if ( QisAlpha[index_k] == hpInf )
				QisAlpha[ index_k ] = SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() );

		RedCost0 = QisAlpha[ index_k ] - Getb()*GetYi(index_k) - GetVi(index_k)
						+ GetTheta()*GetQicCc(index_k);

/*
		if (    LowerSlope[ index_k ] < - min( GetAbsTolError(), GetRelTolError() * GetTheta() )
			 &&                 Delta >   RedCost0 / ( - LowerSlope[ index_k ] )  ){

				if ( abs( Delta - RedCost0 / (-LowerSlope[ index_k ]) ) < min( GetAbsTolError(), GetRelTolError()*GetTheta()) )
								NumberOfTiesStep2++;
				else            NumberOfTiesStep2 = 0;

				Delta   = max( hpZero , RedCost0 / (-LowerSlope[ index_k ] ));

				ParametricIndexStep2 = index_k;
				ParametricEventStep2 = Io2Is;
				position_k           = k;
				}
*/

		if (    LowerSlope[ index_k ] < - min( GetAbsTolError(), GetRelTolError() * GetTheta() ) ){

			Delta_tmp = max( hpZero , RedCost0 / (-LowerSlope[ index_k ] ));

			if ( Delta >= Delta_tmp ){


				if ( abs( Delta - Delta_tmp ) < min( GetAbsTolError(), GetRelTolError()*GetTheta()) ){
								NumberOfTiesStep2++;
								if ( RedCost0 < RedCostNow ) {
									Delta   = Delta_tmp;
									ParametricIndexStep2 	= index_k;
									ParametricEventStep2 	= Io2Is;
									RedCostNow 				= RedCost0;
									position_k           = k;
									}
								}
				else            {
								NumberOfTiesStep2 = 0;

								Delta   = Delta_tmp;
								ParametricIndexStep2 	= index_k;
								ParametricEventStep2 	= Io2Is;
								RedCostNow 				= RedCost0;
								position_k           = k;
								}

				}
			}



		}

if ( position_k < GetMax_n() ){
		swap( IoPath[ NoPath-1 ], IoPath[ position_k ] );
		position_k = GetMax_n();
		}

// Ic ----------------------------------------
for ( register Index k = 0 ; k < NcPath ; k++ ){

		index_k  =  IcPath[ k ];

		hpReal Qis_d          = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
		LowerSlope[ index_k ] = - Qis_d + Getb_deriv() * GetYi(index_k) - GetQicCc(index_k);

		if ( QisAlpha[index_k] == hpInf )
				QisAlpha[ index_k ] = SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() );

		RedCost0 = - QisAlpha[ index_k ] + Getb()*GetYi(index_k) + GetVi(index_k)
						- GetTheta()*GetQicCc(index_k);

/*		if (    LowerSlope[ index_k ] < - min( GetAbsTolError(), GetRelTolError() * GetTheta() )
			 &&                 Delta >   RedCost0 / (-LowerSlope[ index_k ])  ){

				if ( abs( Delta - RedCost0 /(- LowerSlope[ index_k ]) ) < GetAbsTolError() )
								NumberOfTiesStep2++;
				else            NumberOfTiesStep2 = 0;

				Delta   = max( hpZero , RedCost0 / (-LowerSlope[ index_k ]) );

				ParametricIndexStep2    = index_k;
				ParametricEventStep2    = Ic2Is;
				position_k              = k;
				}
*/
		if (    LowerSlope[ index_k ] < - min( GetAbsTolError(), GetRelTolError() * GetTheta() ) ){

			Delta_tmp = max( hpZero , RedCost0 / (-LowerSlope[ index_k ] ));

			if ( Delta >= Delta_tmp ){


				if ( abs( Delta - Delta_tmp ) < min( GetAbsTolError(), GetRelTolError()*GetTheta()) ){
								NumberOfTiesStep2++;
								if ( RedCost0 < RedCostNow ) {
									Delta   = Delta_tmp;
									ParametricIndexStep2 	= index_k;
									ParametricEventStep2 	= Ic2Is;
									RedCostNow 				= RedCost0;
									position_k           	= k;
									}
								}
				else            {
								NumberOfTiesStep2 = 0;

								Delta   = Delta_tmp;
								ParametricIndexStep2 	= index_k;
								ParametricEventStep2 	= Ic2Is;
								RedCostNow 				= RedCost0;
								position_k           	= k;
								}

				}
			}

		}

if ( position_k < GetMax_n() ){
		swap( IcPath[ NcPath-1 ], IcPath[ position_k ] );
		position_k = GetMax_n();
		}

MinRatioStep2Pos = Delta;

#if ( TimeMode )
		TimeFinish();
		TimeAdd( &TimeBreakStep2 );
#endif


}

//-----------------------------------------------------------------------
boolean    TQPMemRedCostPath::GetApproxBreakPoint ()
{

ParametricEvent = NoEvent;
ParametricIndex = GetMax_n();

if ( GetTheta() >= FinalParameter )
		return FALSE;

NextTheta = FinalParameter;

#if ( TimeMode )
		TimeStart();
#endif

ComputeMinRatioTestStep1();

#if ( TimeMode )
		TimeFinish();
		TimeAdd( &TimeBreakStep1 );
#endif

if ( GetInfiniteDirection() ){
		NextTheta       = GetTheta();
        NumberOfTies    = GetNumberOfTiesStep1();
		ParametricEvent = GetParametricEventStep1();
		ParametricIndex = GetParametricIndexStep1();
		switch( ParametricEvent ){
				case Is2Ic:
						IcPath[ NcPath++ ] = ParametricIndex;
						break;
                case Is2Io:
						IoPath[ NoPath++ ] = ParametricIndex;
						break;
				}
		return TRUE;
		}

PathMinRatioTestApproxStep2( min( FinalParameter, GetCurrentMinRationStep1Pos() + GetTheta()));

if ( GetCurrentMinRationStep1Pos() > GetCurrentMinRationStep2Pos() + min( GetAbsTolError(), GetRelTolError()*GetTheta())  ){
		NextTheta       = GetTheta() + GetCurrentMinRationStep2Pos();
		NumberOfTies    = GetNumberOfTiesStep2();
		ParametricEvent = GetParametricEventStep2();
		ParametricIndex = GetParametricIndexStep2();
		}
else    {
		NextTheta       = GetTheta() + GetCurrentMinRationStep1Pos();
		NumberOfTies    = GetNumberOfTiesStep1();
		ParametricEvent = GetParametricEventStep1();
		ParametricIndex = GetParametricIndexStep1();
		}

switch( ParametricEvent ){
		case Is2Ic:
				if ( NcPath < 2*MaxSizeNcPath - 1 )
						IcPath[ NcPath++ ] = ParametricIndex;
				break;
		case Is2Io:
				if ( NoPath < 2*MaxSizeNoPath - 1 )
						IoPath[ NoPath++ ] = ParametricIndex;
				break;
		case Ic2Is:
				IcPath[ --NcPath ] =  GetMax_n();
				break;
		case Io2Is:
				IoPath[ --NoPath ] =  GetMax_n();
				break;
		}


#if ( TimeMode )
		TimeStart();
#endif

UpdateApproxPathVariable();

#if ( TimeMode )
		TimeFinish();
		TimeAdd( &TimeBreakStep1 );
#endif

if ( NextTheta >= FinalParameter ) {
		ParametricEvent = NoEvent;
		ParametricIndex = GetMax_n();
		}
return TRUE;

}

//-----------------------------------------------------------------------
boolean    TQPMemRedCostPath::CheckApproximateStatus()
{

if ( CheckedPath == TRUE ) {
	CheckedPath = FALSE;
	return FALSE;
	}
else if ( abs( NextTheta - GetTheta() ) > 0.05*GetTheta() ) {
	CheckedPath = TRUE;
	return TRUE;
	}
else if ( NumBreakPoints % BPtoUpdates ){
		CheckedPath = FALSE;
		return FALSE;
		}
else    {
		CheckedPath = TRUE;
		return TRUE;
		}

}

//-----------------------------------------------------------------------
void    TQPMemRedCostPath::InitApproxStep2Path()
{

//.... Select subset of Io and Ic to be used for the Approx Path
Index   index_k;
hpReal  RedCost;
hpReal  Qis_d;
hpReal  RatioRedCost = hpInf;

ComputeMinRatioTestStep1();

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );
CopyCurrentDirectionSparse( tmpVecM + GetNs() );


for ( Index k = 0 ; k < GetNs() ; k++ ){

		index_k = GetIs( k );

		QisAlpha[ index_k ]          =   SprodQi_dIset ( index_k, tmpVecM , tmpIndVec, GetNs() );

		}


hpVec           RedCostVec = tmpVecM   + 2*GetNs();
IndexVec        RedCostInd = tmpIndVec + 2*GetNs();
Index           NumInserted = 0;

VecInit ( RedCostVec, hpInf, MaxSizeNoPath );
VecInit ( RedCostInd, GetMax_n(), MaxSizeNoPath );

// Initialize Iop and Icp....
for ( register Index k = 0 ; k < NoPath ; k++ ){
		IoPath[k] = GetMax_n();
		}
NoPath = 0;

for ( register Index k = 0 ; k < NcPath ; k++ ){
		IcPath[k] = GetMax_n();
		}
NcPath = 0;

// Check Variables in Iopp...............
for ( register Index k = 0 ; k < GetNo() ; k++ ){
		register Index index_k = GetIo( k );
		RedCost  =   SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() ); // Qis alpha_iter
		QisAlpha[ index_k ] = RedCost;
		RedCost += - TQPclassPath::Getb()*TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
		RedCost +=   GetTheta()*GetQicCc ( index_k ); // QC_redcost[ index_k ]; // SprodQi_d ( index_k, C, Ic, Nc);

		Qis_d          = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
		LowerSlope[ index_k ] = Qis_d - Getb_deriv() * GetYi(index_k) + GetQicCc(index_k);

		RatioRedCost = RedCost / ( - LowerSlope[ index_k ] );

		if (    LowerSlope[ index_k ] < - min( GetAbsTolError(), GetRelTolError() * GetTheta() )
			 && RedCostVec[ min( NumInserted, MaxSizeNoPath-1) ] > RatioRedCost ){
				if ( NumInserted == MaxSizeNoPath ){
//						RedCostVec[ MaxSizeNoPath - 1 ] = RedCost / ( - LowerSlope[ index_k ] );
						RedCostVec[ MaxSizeNoPath - 1 ] = RatioRedCost;
						RedCostInd[ MaxSizeNoPath - 1 ] = index_k;
						}
				else    {
//						RedCostVec[ NumInserted ] = RedCost;
						RedCostVec[ NumInserted ] = RatioRedCost;
						RedCostInd[ NumInserted++ ] = index_k;
						}
				for ( register Index j = NumInserted - 1 ; j-- ; )
//						if ( RedCostVec[ j ] > RedCostVec[ j + 1 ] ){
						if ( RedCostVec[ j ] > RatioRedCost ){
								swap ( RedCostVec[ j ], RedCostVec[ j + 1 ]);
								swap ( RedCostInd[ j ], RedCostInd[ j + 1 ]);
								}
						else break;
				}
		}

// Copy to IoPath...
AssignVectors ( IoPath, RedCostInd, NumInserted );
NoPath = NumInserted;

VecInit ( RedCostVec, hpInf, MaxSizeNcPath );
VecInit ( RedCostInd, GetMax_n(), MaxSizeNcPath );
NumInserted = 0;

// Check Variables in Icp...............

for ( register Index k = 0 ; k < GetNc() ; k++ ){
		register Index index_k = GetIc(k);
		RedCost  =  - SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs());
		QisAlpha[ index_k ] = - RedCost;
		RedCost +=    TQPclassPath::Getb()*TQPclassPath::GetYi( index_k ) + TQPclassPath::GetVi( index_k );
		RedCost +=  - GetTheta()*GetQicCc ( index_k );// QC_redcost[ index_k ];// - SprodQi_d ( index_k, C, Ic, Nc);

		// add to the vector...
		Qis_d          = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
		LowerSlope[ index_k ] = - Qis_d + Getb_deriv() * GetYi(index_k) - GetQicCc(index_k);

		RatioRedCost = RedCost / ( - LowerSlope[ index_k ] );

		if (    LowerSlope[ index_k ] < - min( GetAbsTolError(), GetRelTolError() * GetTheta() )
			 && RedCostVec[ min( NumInserted, MaxSizeNoPath-1) ] >   RatioRedCost  ){
				if ( NumInserted == MaxSizeNcPath ){
//						RedCostVec[ MaxSizeNcPath - 1 ] = RedCost / ( - LowerSlope[ index_k ] );
						RedCostVec[ MaxSizeNcPath - 1 ] = RatioRedCost;
						RedCostInd[ MaxSizeNcPath - 1 ] = index_k;
						}
				else    {
//						RedCostVec[ NumInserted ] = RedCost;
						RedCostVec[ NumInserted ] = RatioRedCost;
						RedCostInd[ NumInserted++ ] = index_k;
						}
				for ( register Index j = NumInserted - 1 ; j-- ; )
//						if ( RedCostVec[ j ] > RedCostVec[ j + 1 ] ){
						if ( RedCostVec[ j ] > RatioRedCost ){
								swap ( RedCostVec[ j ], RedCostVec[ j + 1 ]);
								swap ( RedCostInd[ j ], RedCostInd[ j + 1 ]);
								}
						else break;
				}
		}

// Copy to IcPath...
AssignVectors ( IcPath, RedCostInd, NumInserted );
NcPath = NumInserted;

if ( DataTestMode >= 0) {
	InitTestDataIc();
	}

}

//-----------------------------------------------------------------------

// routine to return to optimality
void 	TQPMemRedCostPath::ReturnPathToOpt ()
{
				#if ( ReducedCostMode == 3 )
						Nop = 0;
						Nopp = GetNo();
						for ( register Index k = 0 ; k < GetNo() ; k++ )
								Iopp[k] = GetIo(k);
						Ncp = 0;
						Ncpp = GetNc();
						for ( register Index k = 0 ; k < GetNc() ; k++ )
								Icpp[k] = GetIc(k);
				#else
						Nop = GetNo();
						for ( register Index k = 0 ; k < GetNo() ; k++ )
								Iop[k] = GetIo(k);
						Ncp = GetNc();
						for ( register Index k = 0 ; k < GetNc() ; k++ )
								Icp[k] = GetIc(k);
				#endif


				//TQPMemRedCostPath::SolveQP( GetTheta() );
				ParametricMode = FALSE;

				TQPclassPath::SolveQP();

				ParametricMode = TRUE;

				if ( PathMode < 2 ){
					InitStep2Path();
					}
				else {
					InitApproxStep2Path();
					}

				// Update Subset of Indices...

				#if ( TimeMode )
						TimeFinish();
						TimeAdd( &TimeCorrector );
				#endif

				CheckedPath = TRUE;
}

//------------------------------------------------------------------------------
void    TQPMemRedCostPath::SolveApproximatePath (hpReal InitVal, hpReal FinalVal )
{

InitParameter   = InitVal;
FinalParameter  = FinalVal;

TQPMemRedCostPath::SolveQP( InitVal );

PathMode = 2;
ParametricMode = TRUE;
CheckedPath = TRUE;
#if ( TimeMode )
  InitTimePath ();
#endif

InitApproxStep2Path();

fprintf( out_info, "Solving (Approximatelly) Parametrically: (from %lf to %lf)\n", InitVal, FinalVal);
fprintf( out_info, "BreakPoint, Theta, f_obj, Ns, Nc \n");
fprintf( out_info, " %d  %f  %f  %d  %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );

#if ( PrintVisual == 0 )
			printf("Solving (Approximatelly) Parametrically: (from %lf to %lf)\n", InitVal, FinalVal);
			printf("BreakPoint, Theta, f_obj, Ns, Nc \n");
			printf(" %d  %f  %f  %d  %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
#endif

while ( GetApproxBreakPoint () ) {

		// Check for Stuck and Jump if Necessary
		ParametricStatus = CheckingParametricStep();

		if ( ParametricStatus == MsgPotentialCycle ) {
			NumJumps++;
			continue;
			}

		if ( ParametricStatus == MsgReoptimized ) {
			// Not a new breakpoints...
			// need to get next one...
			fprintf( out_info, " %d  %f %f %d %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
			#if ( PrintVisual == 0 )
				printf( " BreakPoint = %d, Theta = %f, f_obj = %f, Ns = %d, Nc = %d \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );
			#endif
			continue;
			}

		NumBreakPoints++;

		//-----------------------------------------------------------------------------
		UpdateEveryIterationPath ();
		//-----------------------------------------------------------------------------

		#if ( TimeMode )
				TimeStart();
		#endif

		// routine to check status dynammically
		if  (  CheckApproximateStatus()  ) {

				ReturnPathToOpt ();

				if ( EvaluatingTestData() ){
					break;
					}

				}
		else    {

				ParametricStatus = TQPclassPath::SetTheta( NextTheta );

				if ( ParametricStatus != MsgOk ){
						PrintErrorMsg("Theta might have been wrongly set.");
						break;
						}

				Compute_f_deriv();


				switch ( ParametricEvent ){

						case Is2Io:
								ParametricStatus = TQPclassPath::FromBaseToZero ( ParametricIndex );
								break;

						case Is2Ic:
								ParametricStatus = TQPclassPath::FromBaseToC ( ParametricIndex );
								break;

						case Io2Is:
								ParametricStatus = TQPclassPath::AddVariableToBase( ParametricIndex, Io2Is );
								break;

						case Ic2Is:
								ParametricStatus = TQPclassPath::AddVariableToBase( ParametricIndex, Ic2Is );
								break;

						case NoEvent:
								PrintErrorMsg("End of the Parametric path.");
								break;
						default:
								PrintErrorMsg("Parametric Event not well defined.");
						}

				if ( ParametricStatus == MsgNumericalProb ){
						PrintErrorMsg("Numerical Instability Detected on Parametric Path: recomputing data structures.\n");
						ParametricStatus = TQPclassPath::RecomputeAll ();
						}

				if ( ParametricStatus == MsgFatal ){
						PrintErrorMsg("A fatal error occured when recomputing the Regularized path.\n");
						break;
						}

				#if ( TimeMode )
						TimeFinish();
						TimeAdd( &TimeUpdates );
				#endif

				if ( EvaluatingTestData() ){
					break;
					}
				}

				fprintf( out_info, "%d  %f  %f  %d  %d ; \n", NumBreakPoints, GetTheta(), Getf_obj(), GetNs(), GetNc() );

		}

NumBreakPoints -= Index ( floor( NumBreakPoints / BPtoUpdates ) );

//-----------------------------------------------------------------------------
UpdateEveryIterationPath ();
//-----------------------------------------------------------------------------

#if (TimeMode)
fprintf( out_info, " Total Time: %f \n", TimeTotalAll );

#if ( PrintVisual == 0 )
		printf(" Total Time: %f \n", TimeTotalAll );
#endif
#endif
}

//-----------------------------------------------------------------------

void    TQPMemRedCostPath::InitStep2Path ()
{
Index tmpNs = GetNs();
Index tmpNo = GetNo();
Index tmpNc = GetNc();

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );

Index index_k;

for ( Index k = 0 ; k < tmpNo ; k++ ){
		index_k = GetIo( k );

		QisAlpha[ index_k ]          =   SprodQi_dIset ( index_k, tmpVecM , tmpIndVec, GetNs() );
		LowerReducedCost[ index_k ]  =   QisAlpha[ index_k ];
		LowerReducedCost[ index_k ] += - Getb() * TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
		LowerReducedCost[ index_k ] +=   GetTheta() * GetQicCc ( index_k );

		}
for ( Index k = 0 ; k < tmpNc ; k++ ){
		index_k = GetIc( k );

		QisAlpha[ index_k ]          =   SprodQi_dIset ( index_k, tmpVecM , tmpIndVec, GetNs() );
		LowerReducedCost[ index_k ]  = - QisAlpha[ index_k ];
		LowerReducedCost[ index_k ] +=   Getb() * TQPclassPath::GetYi( index_k ) + TQPclassPath::GetVi( index_k );
		LowerReducedCost[ index_k ] += - GetTheta() * GetQicCc ( index_k );

		}
for ( Index k = 0 ; k < tmpNs ; k++ ){
		index_k = GetIs( k );

		QisAlpha[ index_k ]          =   SprodQi_dIset ( index_k, tmpVecM , tmpIndVec, GetNs() );
		LowerReducedCost[ index_k ]  =   0.0e0;

		}


if ( DataTestMode >= 0) {
	InitTestDataIc();
	}
}
//-----------------------------------------------------------------------
boolean   TQPMemRedCostPath::EvaluatingTestData()
{
// variables needed
hpReal CurrentTestValue = hpInf;

switch ( DataTestMode ){
		case 0: // "Exact" (compute for all breakpoints)
				CurrentTestValue = ComputingPerformanceOnTestData();
				break;
		case 1: // Uniformly distributed in [init,final]
				if (  ( GetTheta() - LastThetaTested ) / ( (FinalParameter - InitParameter)/NumberOfTests ) > 1){
						LastThetaTested = GetTheta();
						CurrentTestValue = ComputingPerformanceOnTestData();
						}

				break;
		case 2: // Uniformly distributed in the breakpoints
				if ( NumBreakPoints % BreakPointIntervals == 0 ){
						CurrentTestValue = ComputingPerformanceOnTestData();
						}

				break;
		case 3: // Once after alpha's have changed substantially compared with q_tau

				if ( DiffAlpha >= quantile_current ){
						CurrentTestValue = ComputingPerformanceOnTestData();
						// which should update quantile_current

						}

				break;
		default:
				// Not In Use...
				return FALSE;
				//break;

		}

if ( BestTestValue > CurrentTestValue ) {
		BestThetaTestValue = GetTheta();
		BestTestValue = CurrentTestValue;
		fprintf(  out_info, "Improvement on the Test Data Set Performance: theta = %lf, performance = %lf \n",BestThetaTestValue,BestTestValue);
		#if ( PrintVisual == 0 )
			printf("Improvement on the Test Data Set Performance: theta = %lf, performance = %lf \n",BestThetaTestValue,BestTestValue);
		#endif
		}
else if ( CurrentTestValue < 0.1*hpInf ) {
		fprintf(  out_info, "No Improvement on Test: theta = %lf, performance = %lf \n",GetTheta(),CurrentTestValue);
		#if ( PrintVisual == 0 )
			printf("No Improvement on Test: theta = %lf, performance = %lf \n",GetTheta(),CurrentTestValue);
		#endif

		LastThetaTested = GetTheta();


		// Need to Consider an appropriate criterion
		if ( 0 && BestThetaTestValue < 5 + GetTheta() ){
			fprintf(  out_info, "No improvement on the Test Data Set Performance. Stop computing the path.\n",BestThetaTestValue,BestTestValue);
			return TRUE; // should stop path... no progress
			}
		}

return FALSE;
}

//-----------------------------------------------------------------------
boolean   TQPMemRedCostPath::GetNextBreakPoint ()
{

ParametricEvent = NoEvent;
ParametricIndex = GetMax_n();

if ( GetTheta() >= FinalParameter )
        return FALSE;

NextTheta = FinalParameter;

#if ( TimeMode )
		TimeStart();
#endif

ComputeMinRatioTestStep1();

#if ( TimeMode )
        TimeFinish();
		TimeAdd( &TimeBreakStep1 );
#endif

if ( GetInfiniteDirection() ){
        NextTheta       = GetTheta();
        NumberOfTies    = GetNumberOfTiesStep1();
        ParametricEvent = GetParametricEventStep1();
        ParametricIndex = GetParametricIndexStep1();
        return TRUE;
        }

if ( PathMode == 0 ){
		PathMinRatioTestStep2( min( FinalParameter, GetCurrentMinRationStep1Pos() + GetTheta()));
		}
else	{
		PathMinRatioTestStep2LowerBound( min( FinalParameter, GetCurrentMinRationStep1Pos() + GetTheta()) );
		}

if ( GetCurrentMinRationStep1Pos() > GetCurrentMinRationStep2Pos() + min( GetAbsTolError(), GetRelTolError()*GetTheta())  ){
		NextTheta       = GetTheta() + GetCurrentMinRationStep2Pos();
		NumberOfTies    = GetNumberOfTiesStep2();
		ParametricEvent = GetParametricEventStep2();
		ParametricIndex = GetParametricIndexStep2();
        }
else    {
        NextTheta       = GetTheta() + GetCurrentMinRationStep1Pos();
        NumberOfTies    = GetNumberOfTiesStep1();
        ParametricEvent = GetParametricEventStep1();
		ParametricIndex = GetParametricIndexStep1();
        }

#if ( TimeMode )
        TimeStart();
#endif

UpdatePathVariable();

#if ( TimeMode )
		TimeFinish();
        TimeAdd( &TimeBreakStep1 );
#endif

if ( NextTheta >= FinalParameter ) {
        ParametricEvent = NoEvent;
        ParametricIndex = GetMax_n();
        }
return TRUE;

}
//-----------------------------------------------------------------------
void    TQPMemRedCostPath::UpdateApproxPathVariable()
{
Index index_k;
hpReal Delta = NextTheta - GetTheta();

for ( Index k = 0 ; k < NoPath ; k++ ){

        index_k  = IoPath[ k ];
        if ( QisAlpha[ index_k ] < hpInf )
				QisAlpha[ index_k ] += Delta * ( LowerSlope[ index_k ] +
                                       Getb_deriv()*GetYi( index_k ) - GetQicCc( index_k ) ) ;
        }

for ( Index k = 0 ; k < NcPath ; k++ ){

        index_k = IcPath[ k ];

		if ( QisAlpha[ index_k ] < hpInf )
				QisAlpha[ index_k ] += Delta * ( - LowerSlope[ index_k ] + Getb_deriv()*GetYi( index_k ) - GetQicCc( index_k ) ) ;

		}

for ( Index k = 0 ; k < GetNs() ; k++ ){

		index_k = GetIs( k );

		QisAlpha[ index_k ] = hpInf;

		}
}
//-----------------------------------------------------------------------
void    TQPMemRedCostPath::UpdatePathVariable()
{
Index tmpNs = GetNs();
Index tmpNo = GetNo();
Index tmpNc = GetNc();

Index index_k;


hpReal Delta = NextTheta - GetTheta();

for ( Index k = 0 ; k < tmpNo ; k++ ){

		index_k  = GetIo( k );

		if ( PathMode == 1 ){
				LowerReducedCost[ index_k ] += Delta * LowerSlope[ index_k ];
				}

		if ( QisAlpha[ index_k ] < hpInf )
				QisAlpha[ index_k ] += Delta * ( LowerSlope[ index_k ] +
									   Getb_deriv()*GetYi( index_k ) - GetQicCc( index_k ) ) ;

		}

for ( Index k = 0 ; k < tmpNc ; k++ ){

		index_k = GetIc( k );

		if ( PathMode == 1 ) {
				LowerReducedCost[ index_k ] += Delta * LowerSlope[ index_k ];
				}

		if ( QisAlpha[ index_k ] < hpInf )
				QisAlpha[ index_k ] += Delta * ( - LowerSlope[ index_k ] + Getb_deriv()*GetYi( index_k ) - GetQicCc( index_k ) ) ;

		}

for ( Index k = 0 ; k < tmpNs ; k++ ){

		index_k = GetIs( k );

		if ( PathMode == 1 ) {
				LowerReducedCost[ index_k ] = 0.0e0;
				}

		QisAlpha[ index_k ] = hpInf;

		}
}
//-----------------------------------------------------------------------
void    TQPMemRedCostPath::PathMinRatioTestStep2LowerBound( hpReal Bound )
{

#if ( TimeMode )
		TimeStart();
#endif

Index index_k;

Index tmpNo     = GetNo();
Index tmpNc     = GetNc();

hpReal RedCost  = hpZero;

NextTheta       = Bound;
hpReal Delta    = NextTheta - GetTheta();

ParametricIndexStep2 = GetMax_n();
ParametricEventStep2 = NoEvent;

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );
CopyCurrentDirectionSparse( tmpVecM + GetNs() );

// Computes || d ||_1  where  alpha(NextTheta) = alpha(theta) + Delta * d
hpReal dderiv1 = hpZero;
hpReal dderiv1POS = hpZero;
hpReal dderiv1NEG = hpZero;
if ( PositiveKernel() ){
		for ( register Index k = GetNs() ; k-- ;  ) {
				if ( GetYi( GetIs( k ) )*tmpVecM[ GetNs() + k ]  > 0 )
						dderiv1POS += abs( tmpVecM[ GetNs() + k ] );
				else    dderiv1NEG += abs( tmpVecM[ GetNs() + k ] );
				}
		}
else    {
		for ( register Index k = GetNs() ; k-- ;  ) dderiv1 += abs( tmpVecM[ GetNs() + k ] );
		}


for ( register Index k = 0 ; k < tmpNo ; k++ ){
		index_k  =  GetIo( k );

		if ( PositiveKernel() ){
				if ( GetYi( index_k ) > 0 )
						LowerSlope[index_k] = - dderiv1NEG * GetMaxQi( index_k );
				else    LowerSlope[index_k] = - dderiv1POS * GetMaxQi( index_k );
				LowerSlope[index_k] -=  Getb_deriv() * GetYi( index_k );
				LowerSlope[index_k] +=  GetQicCc ( index_k );
				}
		else    {
				LowerSlope[index_k]  =  - dderiv1 * GetMaxQi( index_k );
				LowerSlope[index_k] -=  Getb_deriv() * GetYi( index_k );
				LowerSlope[index_k] +=  GetQicCc ( index_k );
				}


		if (    LowerSlope[index_k] < -min( GetAbsTolError(), GetRelTolError()*GetTheta() ) &&
				Delta > LowerReducedCost[index_k]/( - LowerSlope[index_k] ) ){

				// Compute true reduced cost...
						if ( QisAlpha[ index_k ] == hpInf ){
								QisAlpha[ index_k ] = SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() );
								}
				// actual slope = < Qis, d >  - b' Yi  + QicCc
						LowerSlope[ index_k ]  = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
						LowerSlope[ index_k ] += - Getb_deriv() * GetYi( index_k ) + GetQicCc ( index_k );

						LowerReducedCost[index_k] = QisAlpha[ index_k ] - Getb()*GetYi( index_k ) - GetVi( index_k ) + GetTheta()*GetQicCc( index_k );

						if (    LowerSlope[index_k] < -min( GetAbsTolError(), GetRelTolError()*GetTheta()) &&
								Delta >= LowerReducedCost[index_k] / ( - LowerSlope[index_k] ) ){

										if ( abs( Delta - LowerReducedCost[index_k] / ( - LowerSlope[index_k] ) ) < GetAbsTolError() )     NumberOfTiesStep2++;
										else     NumberOfTiesStep2 = 0;

										Delta           = max ( 0.0e0, LowerReducedCost[index_k] / ( - LowerSlope[index_k] ) );

										ParametricIndexStep2  = index_k;
										ParametricEventStep2  = Io2Is;

										}
				}
		 else   {
				QisAlpha[ index_k ] = hpInf;
				}

		}

// Ic ---------------------------------------------------------------

for ( register Index k = 0 ; k < tmpNc ; k++ ){
		index_k  =  GetIc( k );

		if ( PositiveKernel() ){
				if ( GetYi( index_k ) < 0 )
						LowerSlope[index_k] = - dderiv1NEG * GetMaxQi( index_k );
				else    LowerSlope[index_k] = - dderiv1POS * GetMaxQi( index_k );
				LowerSlope[index_k] +=  Getb_deriv() * GetYi( index_k );
				LowerSlope[index_k] -=  GetQicCc ( index_k );
				}
		else    {
				LowerSlope[index_k]  = - dderiv1 * GetMaxQi( index_k );
				LowerSlope[index_k] +=  Getb_deriv() * GetYi( index_k );
				LowerSlope[index_k] -=  GetQicCc ( index_k );
				}


		if (    LowerSlope[index_k] < -min( GetAbsTolError(), GetRelTolError()*GetTheta() ) &&
				Delta > LowerReducedCost[index_k]/( - LowerSlope[index_k] ) ){

				// Compute true reduced cost...
						if ( QisAlpha[ index_k ] == hpInf ){
								QisAlpha[ index_k ] = SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() );
								}
				// actual slope = < Qis, d >  - b' Yi  + QicCc
						LowerSlope[ index_k ]  = - SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
						LowerSlope[ index_k ] += Getb_deriv() * GetYi( index_k ) - GetQicCc ( index_k );

						LowerReducedCost[index_k] = - QisAlpha[ index_k ] + Getb()*GetYi( index_k ) + GetVi( index_k ) - GetTheta()*GetQicCc( index_k );

						if (    LowerSlope[index_k] < -min( GetAbsTolError(), GetRelTolError()*GetTheta() ) &&
								Delta >= LowerReducedCost[index_k] / ( - LowerSlope[index_k] ) ){

										if ( abs( Delta - LowerReducedCost[index_k] / ( - LowerSlope[index_k] ) ) < GetAbsTolError() )     NumberOfTiesStep2++;
										else     NumberOfTiesStep2 = 0;

										Delta           = max ( 0.0e0, LowerReducedCost[index_k] / ( - LowerSlope[index_k] ) );

										ParametricIndexStep2  = index_k;
										ParametricEventStep2  = Ic2Is;

										}

				}
		else    {
				QisAlpha[ index_k ] = hpInf;
				}

		}

MinRatioStep2Pos = Delta;

#if ( TimeMode )
		TimeFinish();
		TimeAdd( &TimeBreakStep2 );
#endif

}
//-----------------------------------------------------------------------
void    TQPMemRedCostPath::Compute_f_deriv()
{

Index index_k;

Index tmpNc     = GetNc();

hpReal RedCost  = hpZero;

if ( 1 ) {
		f_deriv = GetNumericDerivate_f ();
		return;
		}

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );

f_deriv = hpZero;
// Ic ----------------------------------------
for ( register Index k = 0 ; k < tmpNc ; k++ ){
		index_k  =  GetIc( k );

		RedCost  = - SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() );
		RedCost +=   Getb()*TQPclassPath::GetYi( index_k ) + TQPclassPath::GetVi( index_k );
		RedCost -=   GetTheta() * GetQicCc ( index_k );

		f_deriv -= RedCost * GetQicCc ( index_k );
		}

}
//-----------------------------------------------------------------------
void    TQPMemRedCostPath::PathMinRatioTestStep2( hpReal Bound )
{

#if ( TimeMode )
		TimeStart();
#endif

hpReal RedCostNow = hpInf;
hpReal Delta_tmp  = hpInf;


Index index_k;
Index tmpNo     = GetNo();
Index tmpNc     = GetNc();

hpReal RedCost0 = hpZero;
//hpReal RedCost  = hpZero;

NextTheta       = Bound;
hpReal Delta    = NextTheta - GetTheta();

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );
CopyCurrentDirectionSparse( tmpVecM + GetNs() );

ParametricIndexStep2 = GetMax_n();

// Io ----------------------------------------
for ( register Index k = 0 ; k < tmpNo ; k++ ){

		index_k  =  GetIo( k );


		hpReal Qis_d          = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
		LowerSlope[ index_k ] = Qis_d - Getb_deriv() * GetYi(index_k) + GetQicCc(index_k);

		if ( QisAlpha[index_k] == hpInf )
				QisAlpha[ index_k ] = SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() );

		RedCost0 = QisAlpha[ index_k ] - Getb()*GetYi(index_k) - GetVi(index_k)
						+ GetTheta()*GetQicCc(index_k);

		if ( RedCost0 < - 10.0*GetAbsTolError() ){
				PrintErrorMsg("Optimality close to have been lost");
				ParametricStatus = MsgOptimalityLost;
				return;
				}


		if (    LowerSlope[ index_k ] < - min( GetAbsTolError(), GetRelTolError() * GetTheta() ) ){

			Delta_tmp = max( hpZero , RedCost0 / (-LowerSlope[ index_k ] ));

			if ( Delta >= Delta_tmp ){


				if ( abs( Delta - Delta_tmp ) < min( GetAbsTolError(), GetRelTolError()*GetTheta()) ){
								NumberOfTiesStep2++;
								if ( RedCost0 < RedCostNow ) {
									Delta   = Delta_tmp;
									ParametricIndexStep2 	= index_k;
									ParametricEventStep2 	= Io2Is;
									RedCostNow 				= RedCost0;
									}
								}
				else            {
								NumberOfTiesStep2 = 0;

								Delta   = Delta_tmp;
								ParametricIndexStep2 	= index_k;
								ParametricEventStep2 	= Io2Is;
								RedCostNow 				= RedCost0;
								}

				}
			}


		}
// Ic ----------------------------------------
for ( register Index k = 0 ; k < tmpNc ; k++ ){

		index_k  =  GetIc( k );

		hpReal Qis_d          = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
		LowerSlope[ index_k ] = - Qis_d + Getb_deriv() * GetYi(index_k) - GetQicCc(index_k);

		if ( QisAlpha[index_k] == hpInf )
				QisAlpha[ index_k ] = SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() );

		RedCost0 = - QisAlpha[ index_k ] + Getb()*GetYi(index_k) + GetVi(index_k)
						- GetTheta()*GetQicCc(index_k);


		if ( RedCost0 < - 10.0*GetAbsTolError() ){
				PrintErrorMsg("Optimality close to have been lost");
				ParametricStatus = MsgOptimalityLost;
				return;

				}


		if (    LowerSlope[ index_k ] < - min( GetAbsTolError(), GetRelTolError() * GetTheta() ) ){

			Delta_tmp = max( hpZero , RedCost0 / (-LowerSlope[ index_k ] ));

			if ( Delta >= Delta_tmp ){


				if ( abs( Delta - Delta_tmp ) < min( GetAbsTolError(), GetRelTolError()*GetTheta()) ){
								NumberOfTiesStep2++;
								if ( RedCost0 < RedCostNow ) {
									Delta   = Delta_tmp;
									ParametricIndexStep2 	= index_k;
									ParametricEventStep2 	= Ic2Is;
									RedCostNow 				= RedCost0;
									}
								}
				else            {
								NumberOfTiesStep2 = 0;

								Delta   = Delta_tmp;
								ParametricIndexStep2 	= index_k;
								ParametricEventStep2 	= Ic2Is;
								RedCostNow 				= RedCost0;
								}

				}
			}
		}

MinRatioStep2Pos = Delta;

#if ( TimeMode )
		TimeFinish();
		TimeAdd( &TimeBreakStep2 );
#endif

}

//-----------------------------------------------------------------------
boolean TQPMemRedCostPath::NeedToGetNewIndex ()
{

CountCallsReducedCost++;

#if ( TimeMode )
		TimeStart();
#endif

boolean         NeedToAdd       = FALSE;
hpReal          RedCost         = hpZero;
Index           place_k         = GetMax_n();
Index           IndexMin        = GetMax_n();
hpReal          RCMin           = hpZero;
TypeEvent       NextEvent     = NoEvent;

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );

#if ( !ReducedCostMode )
		CountToIopp = GetMax_n() + GetIteration();
		CountToIcpp = GetMax_n() + GetIteration();
#endif

// Check Variables in Iop...............
//for ( register Index k = Nop ; k-- ; ){
for ( register Index k = 0 ; k < Nop ; k++ ){
		register Index index_k = Iop[k];

		#if ( DebugMode >= 1 )
			if ( !BelongToIo ( index_k ) ){
					fprintf( out_info, "(NeedToGetNewIndex) Io[%ld] = Index inconsistent %ld, not in Io ... ", k, index_k );
					#if ( PrintVisual == 0 )
						printf( "(NeedToGetNewIndex) Io[%ld] = Index inconsistent %ld, not in Io ... ", k, index_k );
					#endif
					if ( BelongToIc ( index_k ) ) {
						fprintf( out_info,"belongs to Ic.\n");
						#if ( PrintVisual == 0 )
							printf( "belongs to Ic.\n");
						#endif
						}
					else {
						fprintf( out_info,"belongs to Is.\n");
						#if ( PrintVisual == 0 )
							printf( "belongs to Is.\n");
						#endif
						}
					}
		#endif

		RedCost  =   SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() ); // Qis alpha_iter
		RedCost += - TQPclassPath::Getb()*TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
		RedCost +=   GetTheta()*GetQicCc ( index_k );

		// positive reduce cost... increase counter for Iop[k]
		if      ( RedCost > -GetAbsTolError() && ++CounterNonNegRedCost[index_k] > CountToIopp ){
				Iop       [   k   ]  = Iop[Nop-1];
				Iop       [ Nop-1 ]  = TQPclassPath::GetMax_n();
				if ( Iop[ k ] == IndexMin ) place_k = k;
				Nop--;
				k--;
				#if ( ReducedCostMode == 3 )
						Iopp [ Nopp++ ]                   = index_k;
						CounterNonNegRedCost[ index_k ]   = 1;
				#endif
				}
		// negative reduce cost... set counter to zero and check RedCostMin
		else if ( RedCost < -GetAbsTolError() ) {
				CounterNonNegRedCost[ index_k ] = 0;
				if ( RedCost < RCMin ){
						RCMin          = RedCost;
						IndexMin       = index_k;
						place_k        = k;
						NextEvent      = Io2Is;
						NeedToAdd      = TRUE;

						#if ( TimeMode && ReducedCostMode == 2 )
								if ( RedCost <= RedCostBar ) break;
						#endif
						}
				}
		}


// Check Variables in Icp...............
#if ( TimeMode && ReducedCostMode == 2 )
if ( RCMin > RedCostBar )
#endif
//   for ( register Index k = Ncp ; k-- ; ){
   for ( register Index k = 0 ; k < Ncp ; k++ ){
		register Index index_k = Icp[k];

		#if ( DebugMode >= 1 )
			if ( !BelongToIc ( index_k ) ){
					fprintf( out_info, "(NeedToGetNewIndex) Index inconsistent %ld, not in Ic ... ", index_k );
					#if ( PrintVisual == 0 )
						printf( "(NeedToGetNewIndex) Index inconsistent %ld, not in Ic ... ", index_k );
					#endif
					if ( BelongToIo ( index_k ) ) {
						fprintf( out_info,"belongs to Io.\n");
						#if ( PrintVisual == 0 )
							printf( "belongs to Io.\n");
						#endif
						}
					else {
						fprintf( out_info,"belongs to Is.\n");
						#if ( PrintVisual == 0 )
							printf( "belongs to Is.\n");
						#endif
						}
					}
		#endif

		RedCost  =  - SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs());
		RedCost +=    TQPclassPath::Getb()*TQPclassPath::GetYi( index_k ) + TQPclassPath::GetVi( index_k );
		RedCost +=  - GetTheta()*GetQicCc ( index_k );// QC_redcost[ index_k ];// - SprodQi_d ( index_k, C, Ic, Nc);

		// positive reduce cost... increase counter for Icp[k]
		if      ( RedCost > -GetAbsTolError() && ++CounterNonNegRedCost[index_k] > CountToIcpp ){
//        if      ( RedCost > hpZero && ++CounterNonNegRedCost[index_k] > CountToIcpp ){
				Icp       [   k   ]  = Icp[Ncp-1];
				Icp       [ Ncp-1 ]  = TQPclassPath::GetMax_n();
				if ( Icp[ k ] == IndexMin ) place_k = k;
				Ncp--;
				k--;
				#if ( ReducedCostMode == 3 )
						Icpp [ Ncpp++ ]           = index_k;
						CounterNonNegRedCost[ index_k ]  = 1;
				#endif
				}
		// negative reduce cost... set counter to zero and check RedCostMin
		else    if ( RedCost < - GetAbsTolError() )  {
//        else    if ( RedCost < hpZero )  {
				CounterNonNegRedCost[index_k] = 0;
				if ( RedCost < RCMin ){
						RCMin          = RedCost;
						IndexMin       = index_k;
						place_k        = k;
						NextEvent      = Ic2Is;
						NeedToAdd      = TRUE;
						#if ( TimeMode && ReducedCostMode == 2 )
								if ( RedCost <= RedCostBar ) break;
						#endif
						}
				}
		}

#if ( TimeMode && ReducedCostMode == 2 )
	 if ( GetIteration() % 10 == 0  )        UpdateReducedCostBar ();
#endif

#if ( ReducedCostMode == 3 )


// perform Major Iteration
if (  ( !NeedToAdd ) && MajorLoop  ){
					NeedToAdd = TQPMemRedCostPath::MajorIterationsReducedCosts( place_k, IndexMin,
										RCMin, NextEvent );
		}
#endif

// if we already found an index with negative reduce cost... we can stop the search.
if ( NeedToAdd ) {
// Update Data Structures if necesary
		switch ( NextEvent ){
				case Ic2Is:
						Icp       [ place_k ]   = Icp[ Ncp - 1 ];
						Icp       [ Ncp - 1 ]   = TQPclassPath::GetMax_n();
						Ncp--;
						break;

				case Io2Is:
						Iop       [ place_k ]   = Iop[ Nop - 1 ];
						Iop       [ Nop - 1 ]   = TQPclassPath::GetMax_n();
						Nop--;
						break;

				case NoEvent:
						break;
				}

		#if ( TimeMode )
			  TimeFinish();
			  TimeAdd( &TimeReduceCost );
		#endif
		TQPclassPath::SetNewIndexToBeAdded ( IndexMin , RCMin );
		TQPclassPath::SetBaseEvent ( NextEvent );
		MajorLoop = TRUE;
		return NeedToAdd;
		}



// Otherwise... need to search Io and Ic
// Unless we already did it...
#if ( ReducedCostMode == 3 )
	   if ( !NeedToAdd && !MajorLoop && GetNo() == ( Nopp + Nop ) && GetNc() == ( Ncpp + Ncp ) ) {
#else
	   if ( !NeedToAdd && GetNo() == Nop && GetNc() == Ncp ) {
#endif
				#if ( TimeMode )
						TimeFinish();
						TimeAdd( &TimeReduceCost );
				#endif
				SetNewIndexToBeAdded ( TQPclassPath::GetMax_n() , hpZero );
				TQPclassPath::SetBaseEvent ( NextEvent );
				return FALSE; // == NeedToAdd;
				}



#if ( ReducedCostMode == 3 )

		if ( ( Nopp + Nop ) == GetNo() && ( Ncpp + Ncp ) == GetNc() )
				MajorLoop = FALSE;

		Nopp = GetNo();
		Ncpp = GetNc();

		TQPclassPath::CopyCurrentIo( Iopp );
		TQPclassPath::CopyCurrentIc( Icpp );

		VecInit( Iop, GetMax_n(), Nop );
		VecInit( Icp, GetMax_n(), Ncp );
		Nop = 0;
		Ncp = 0;
#else
		Nop = TQPclassPath::GetNo();
		Ncp = TQPclassPath::GetNc();

		TQPclassPath::CopyCurrentIo( Iop );
		TQPclassPath::CopyCurrentIc( Icp );
#endif

VecInit ( CounterNonNegRedCost, IntNumber( 0 ), Getn() );

#if ( TimeMode )
		TimeFinish();
		TimeAdd( &TimeReduceCost );
#endif

NeedToAdd = NeedToGetNewIndex ();

#if ( ReducedCostMode == 3 )
		MajorLoop = TRUE;
#endif
return NeedToAdd;
}

//--------------------------------------------------------------------
#if ( ReducedCostMode == 3 )
boolean         TQPMemRedCostPath::MajorIterationsReducedCosts( Index &place_k, Index &IndexMin,
										hpReal &RCMin, TypeEvent &NextEvent )
{
// Already solved the problem for Iop and Icp...
// Search within Iopp and Icpp for the next MaxSizeNcp and MaxSizeNop indices
// to be added into Ic and Io...

boolean         NeedToAdd       = FALSE;
hpReal          RedCost         = hpZero;
NextEvent       = NoEvent;
place_k         = GetMax_n();
IndexMin        = GetMax_n();
RCMin           = hpZero;

CountMajorLoop++;

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );

hpVec           RedCostVec = tmpVecM   + GetNs();
IndexVec        RedCostInd = tmpIndVec + GetNs();
Index           NumInserted = 0;
VecInit ( RedCostVec, hpInf, MaxSizeNop );
VecInit ( RedCostInd, GetMax_n(), MaxSizeNop );

// Initialize Iop and Icp....
for ( register Index k = 0 ; k < Nop ; k++ ){
		Iopp[Nopp++] = Iop[k];
		CounterNonNegRedCost[ Iop[k] ] = 1;
		Iop[k] = GetMax_n();
		}
Nop = 0;

for ( register Index k = 0 ; k < Ncp ; k++ ){
		Icpp[Ncpp++] = Icp[k];
		CounterNonNegRedCost[ Icp[ k ] ] = 1;
		Icp[k] = GetMax_n();
		}
Ncp = 0;

// Check Variables in Iopp...............
for ( register Index k = 0 ; k < Nopp ; k++ ){
		register Index index_k = Iopp[k];

		#if ( DebugMode >= 1 )
			if ( !BelongToIo ( index_k ) ){
					fprintf( out_info, "(MajorIterationsReducedCosts) Index inconsistent %ld, not in Io ... ", index_k );
					#if ( PrintVisual == 0 )
						printf( "(MajorIterationsReducedCosts) Index inconsistent %ld, not in Io ... ", index_k );
					#endif
					if ( BelongToIc ( index_k ) ) {
						fprintf( out_info,"belongs to Ic.\n");
						#if ( PrintVisual == 0 )
							printf( "belongs to Ic.\n");
						#endif
						}
					else {
						fprintf( out_info,"belongs to Is.\n");
						#if ( PrintVisual == 0 )
							printf( "belongs to Is.\n");
						#endif
						}
					}
		#endif

		RedCost  =   SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs() ); // Qis alpha_iter
		RedCost += - TQPclassPath::Getb()*TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
		RedCost +=   GetTheta()*GetQicCc ( index_k ); // QC_redcost[ index_k ]; // SprodQi_d ( index_k, C, Ic, Nc);

		// positive reduce cost... increase counter for Iop[k]
		if      ( RedCost > -GetAbsTolError() && ++CounterNonNegRedCost[ index_k ] > CountToIoppp ){
				Iopp       [    k   ]  = Iopp[Nopp-1];
				Iopp       [ Nopp-1 ]  = TQPclassPath::GetMax_n();
				if ( Iopp[ k ] == IndexMin ) place_k = k;
				Nopp--;
				k--;
				}

		// add to the vector...
		else if ( RedCost < RedCostVec[ min( NumInserted  , MaxSizeNop-1 ) ] ){
				if ( NumInserted == MaxSizeNop ){
						if ( RedCostInd[ MaxSizeNop - 1 ] != GetMax_n() )
								Iopp[ k ] = RedCostInd[ MaxSizeNop - 1 ];
						RedCostVec[ MaxSizeNop - 1 ] = RedCost;
						RedCostInd[ MaxSizeNop - 1 ] = index_k;
						}
				else    {
						RedCostVec[ NumInserted ] = RedCost;
						RedCostInd[ NumInserted++ ] = index_k;
						Iopp[    k   ]   = Iopp[ Nopp - 1 ];
						Iopp[ Nopp - 1 ] = GetMax_n();
						Nopp--;
						k--;
						}

				for ( register Index j = NumInserted - 1 ; j-- ; )
//						if ( RedCostVec[ j + 1 ] < RedCostVec[ j ] ){
						if ( RedCost < RedCostVec[ j ] ) {
								swap ( RedCostVec[ j ], RedCostVec[ j + 1 ]);
								swap ( RedCostInd[ j ], RedCostInd[ j + 1 ]);
								}
						else break;
				}

		// negative reduce cost... set counter to zero and check RedCostMin
		if ( RedCost < - GetAbsTolError() ) {
				CounterNonNegRedCost[ index_k ] = 0;
				if ( RedCost < RCMin ){

						// add to the top...
						RCMin          = RedCost;
						IndexMin       = index_k;
						NextEvent      = Io2Is;
						NeedToAdd      = TRUE;
						}
				}
		}

// Copy to Iop...
AssignVectors ( Iop, RedCostInd, NumInserted );
Nop = NumInserted;

VecInit ( RedCostVec, hpInf, NumInserted );
VecInit ( RedCostInd, GetMax_n(), NumInserted );
NumInserted = 0;

// Check Variables in Icp...............

   for ( register Index k = 0 ; k < Ncpp ; k++ ){
		register Index index_k = Icpp[k];


		#if ( DebugMode >= 1 )
			if ( !BelongToIc ( index_k ) ){
					fprintf( out_info, "(MajorIterationsReducedCosts) Index inconsistent %ld, not in Io ... ", index_k );
					#if ( PrintVisual == 0 )
						printf( "(MajorIterationsReducedCosts) Index inconsistent %ld, not in Io ... ", index_k );
					#endif
					if ( BelongToIo ( index_k ) ) {
						fprintf( out_info,"belongs to Io.\n");
						#if ( PrintVisual == 0 )
							printf( "belongs to Io.\n");
						#endif
						}
					else {
						fprintf( out_info,"belongs to Is.\n");
						#if ( PrintVisual == 0 )
							printf( "belongs to Is.\n");
						#endif
						}
					}
		#endif

		RedCost  =  - SprodQi_dIset ( index_k, tmpVecM, tmpIndVec, GetNs());
		RedCost +=    TQPclassPath::Getb()*TQPclassPath::GetYi( index_k ) + TQPclassPath::GetVi( index_k );
		RedCost +=  - GetTheta()*GetQicCc ( index_k );// QC_redcost[ index_k ];// - SprodQi_d ( index_k, C, Ic, Nc);

		// positive reduce cost... increase counter for Icp[k]
		if      ( RedCost > -GetAbsTolError() && ++CounterNonNegRedCost[ index_k ] > CountToIcppp ){
				Icpp       [    k   ]  = Icpp[Ncpp-1];
				Icpp       [ Ncpp-1 ]  = TQPclassPath::GetMax_n();
				if ( Icp[ k ] == IndexMin ) place_k = k;
				Ncpp--;
				k--;
				}

		// add to the vector...
		else if ( RedCost < RedCostVec[ min( NumInserted , MaxSizeNcp-1) ] ){
				if ( NumInserted == MaxSizeNcp ){
						Icpp[ k ] = RedCostInd[ MaxSizeNcp - 1 ];
						RedCostVec[ MaxSizeNcp - 1 ] = RedCost;
						RedCostInd[ MaxSizeNcp - 1 ] = index_k;
						}
				else    {
						RedCostVec[ NumInserted ] = RedCost;
						RedCostInd[ NumInserted++ ] = index_k;
						Ncpp--;
						Icpp[   k  ] = Icpp[ Ncpp ];
						Icpp[ Ncpp ] = GetMax_n();
						k--;
						}
				for ( register Index j = NumInserted - 1 ; j-- ; )
//						if ( RedCostVec[ j ] > RedCostVec[ j + 1 ] ){
						if ( RedCost < RedCostVec[ j ] ) {
								swap ( RedCostVec[ j ], RedCostVec[ j + 1 ]);
								swap ( RedCostInd[ j ], RedCostInd[ j + 1 ]);
								}
						else break;
				}

		// negative reduce cost... set counter to zero and check RedCostMin
		if ( RedCost < - GetAbsTolError() )  {
				CounterNonNegRedCost[ index_k ] = 0;
				if ( RedCost < RCMin ){
						RCMin          = RedCost;
						IndexMin       = index_k;
						NextEvent      = Ic2Is;
						NeedToAdd      = TRUE;
						}
				}
		}

// Copy to Icp...
AssignVectors ( Icp, RedCostInd, NumInserted );
Ncp = NumInserted;

if ( NeedToAdd ) place_k = 0;
else             place_k = GetMax_n();
return NeedToAdd;
}
#endif


//--------------------------------------------------------------------
void            TQPMemRedCostPath::RedCostIndexAddedToIc ( Index i )
{

if ( !ParametricMode ) {
   #if ( ReducedCostMode == 3 )

   if ( Ncp < MaxSizeNcp ){
		Icp       [ Ncp ] = i;
		CounterNonNegRedCost[ i ] = 0;
		Ncp++;
		}
   else {
		Icpp       [ Ncpp ] = i;
		CounterNonNegRedCost[ i ] = 0;
		Ncpp++;
		}

   #else

   Icp       [ Ncp ] = i;
   CounterNonNegRedCost[ i ] = 0;
   Ncp++;

   #endif
   }

if ( ParametricMode ){
		Index index_k;
		for ( register Index k = GetNo() ; k-- ; ){
				index_k = GetIo(k);

				if ( PathMode != 1 || QisAlpha[ index_k ] < hpInf ) {
								QisAlpha[ index_k ] -= GetTheta()*GetCi(i)*GetQij( i, index_k);
								}
				}
		for ( register Index k = GetNc() ; k-- ; ){
				index_k = GetIc(k);

				if ( PathMode != 1 || QisAlpha[ index_k ] < hpInf ) {
								QisAlpha[ index_k ] -= GetTheta()*GetCi(i)*GetQij( i, index_k);
								}
				}
		LowerReducedCost[ i ] = 0;
		QisAlpha[ i ] = hpInf;

		if ( DataTestMode >= 0 ) {
					UpdateTestDataIc( i, Is2Ic );
					}

		}

register const hpReal Ci = GetCi( i );
for ( register Index k = Getn() ; k-- ; )
		QC_redcost[ k ] += Ci*GetQij( i, k);
}

//--------------------------------------------------------------------
void            TQPMemRedCostPath::RedCostIndexAddedToIo ( Index i )
{

if ( !ParametricMode ) {

   #if ( ReducedCostMode == 3 )

   if ( Nop < MaxSizeNop ){
		Iop       [ Nop ] = i;
		CounterNonNegRedCost[ i ] = 0;
		Nop++;
		}
   else {
		Iopp       [ Nopp ] = i;
		CounterNonNegRedCost[ i ] = 0;
		Nopp++;
		}

   #else

   Iop       [ Nop ] = i;
   CounterNonNegRedCost[ i ] = 0;
   Nop++;

   #endif
   }

if ( ParametricMode ){
		// Other QisAlpha do not need to be updated since Alpha_i = 0
		LowerReducedCost[ i ] = 0;
		QisAlpha[ i ] = hpInf;
		}

}

//--------------------------------------------------------------------

void            TQPMemRedCostPath::RedCostIndexRemovedFromIc ( Index i )
{
register const hpReal Ci = GetCi( i );
for ( register Index k = Getn() ; k-- ; )
		QC_redcost[ k ] -= Ci*GetQij( i, k);

if ( ParametricMode ){
		LowerReducedCost[ i ] = 0;
		QisAlpha[ i ] = hpInf;
		Index index_k;
		for ( register Index k = GetNo() ; k-- ; ){
				index_k = GetIo(k);

				if ( PathMode != 1 || QisAlpha[ index_k ] < hpInf ){
					QisAlpha[ index_k ] += GetTheta()*Ci*GetQij( i, index_k);
					}

				}
		for ( register Index k = GetNc() ; k-- ; ){
				index_k = GetIc(k);

				if ( PathMode != 1 || QisAlpha[ index_k ] < hpInf ){
						QisAlpha[ index_k ] += GetTheta()*Ci*GetQij( i, index_k);
						}
				}
		if ( DataTestMode >= 0 ) {
					UpdateTestDataIc( i, Ic2Is );
					}
		}

}

//--------------------------------------------------------------------
void            TQPMemRedCostPath::RedCostIndexRemovedFromIo ( Index i )
{

// No need to update other QisAlpha
LowerReducedCost[ i ] = 0;
QisAlpha[ i ] = hpInf;
}

//--------------------------------------------------------------------

TQPMemRedCostPath::~TQPMemRedCostPath()
{

if ( Icp )      delete( Icp );
if ( Iop )      delete( Iop );

if ( CounterNonNegRedCost )     delete( CounterNonNegRedCost );

if ( QC_redcost )               delete ( QC_redcost );

if ( tmpVecM )                  delete ( tmpVecM );
if ( tmpIndVec )                delete ( tmpIndVec );

#if ( MemStyleQ == 1 || MemStyleQ == 2 )
		if ( QQ )               DeleteQ();
		if ( QIndex )           delete( QIndex );
		if ( InvQIndex )        delete( InvQIndex );
#endif

#if ( ReducedCostMode == 3 )
		if ( Iopp )             delete( Iopp );
		if ( Icpp )             delete( Icpp );
#endif

if ( LowerReducedCost )         delete ( LowerReducedCost );
if ( LowerSlope )               delete ( LowerSlope );
if ( QisAlpha )                 delete ( QisAlpha );
if ( IoPath )                   delete ( IoPath );
if ( IcPath )                   delete ( IcPath );


}




//-----------------------------------------------------------------------------
QPErrorMsg      TQPMemRedCostPath::AddColumnToQ( Index i ) // Used only if MemStyle == 1 or 2
{

#if ( MemStyleQ == 1 || MemStyleQ == 2 )
   if ( QIndex[ i ] < Index( CurrentAllocQ ) ){
        // already compute and still in the memory...
		if ( QIndex[ i ] > Nq ){
                Index place_i  = QIndex[ i ];

				swap( QQ[ place_i ], QQ[ Nq ] );

                QIndex[ i ]       = Nq; //QIndex[ InvQIndex[ Nq ] ];

                if ( InvQIndex[ Nq ] < GetMax_n() ){
                        QIndex[ InvQIndex[ Nq ] ] = place_i;
                        }

                InvQIndex[ place_i ] = InvQIndex[ Nq ];
                InvQIndex[ Nq ] = i;
                }
        Nq++;
        }
   else {
        //Information Not in the Memory

        if ( Nq >= Index( CurrentAllocQ )  ){
                // we also need more memory

                if ( Index ( MaxNumberofQcolumns - CurrentAllocQ ) <= Nq ) {
						// Need to overwrite
                        Index place_Nq   = Index( rand() % ( Nq ) );
                        Index who_Nq     = InvQIndex[ Nq ];
                        hpVec const QNq  = QQ[ Nq ];

                        QQ[ Nq ]         = QQ[ place_Nq ];
                        QQ[ place_Nq ]   = QNq;

                        QIndex[ who_Nq ] = place_Nq;

						QIndex[ InvQIndex[ place_Nq ] ] = TQPclassPath::GetMax_n();
						InvQIndex[ place_Nq ] = who_Nq;
						}
				else    { // Allocate more memory
						for ( register Index j = BlockAllocQ ; j-- ; ){
								memalloc( QQ[ CurrentAllocQ++ ], hpReal, TQPclassPath::GetMax_n() );
								#if     ( MemStyleQ == 2 )
										VecInit ( QQ[ CurrentAllocQ - 1 ] , 10.0*hpInf, TQPclassPath::GetMax_n() );
								#endif
								}
						}
				}

		if ( Nq < Index( CurrentAllocQ ) && InvQIndex[ Nq ]  < TQPclassPath::GetMax_n() ){
				// We have space to spare and we are storing something useful in QQ[Nq]
				Index who_Nq            = InvQIndex[ Nq ];
				Index new_place_Nq      = Nq + Index( rand() % ( CurrentAllocQ - Nq ) );
				Index who_deleted       = InvQIndex[ new_place_Nq ];

				if ( new_place_Nq != Nq ){
						swap( QQ[ new_place_Nq ] , QQ[ Nq ] );

						if ( who_deleted < TQPclassPath::GetMax_n() ) {
								swap( QIndex[ who_Nq ], QIndex[ who_deleted ]);
								QIndex[ who_deleted ] = TQPclassPath::GetMax_n();
								}
						else QIndex[ who_Nq ] = new_place_Nq;
						swap( InvQIndex[ new_place_Nq ], InvQIndex[ Nq ] );
						}

				#if ( MemStyleQ == 2 )
						VecInit ( QQ[ Nq ] , 10.0*hpInf, TQPclassPath::GetMax_n() );
				#endif
				}

		if ( InvQIndex[ Nq ] < TQPclassPath::GetMax_n() )
				QIndex[ InvQIndex[ Nq ] ] = TQPclassPath::GetMax_n();
		InvQIndex[ Nq ] = i;
		QIndex[ i ] = Nq;

		#if ( MemStyleQ == 1 )
				GetDataQi( QQ[ Nq ], i );
		#endif

		if ( Nq < Index( MaxNumberofQcolumns ) ) Nq++;
		}
#endif
return MsgOk;
}


//-----------------------------------------------------------------------------
QPErrorMsg      TQPMemRedCostPath::FreeMemQ( Index i )
{
#if ( MemStyleQ == 1 || MemStyleQ == 2 )
        Index aux_i = QIndex[ i ];
		if ( aux_i < TQPclassPath::GetMax_n() ){
                swap( QQ[ aux_i ] , QQ[ Nq - 1 ] );
				QIndex[ InvQIndex[ Nq - 1 ] ] = aux_i;
                QIndex[ i ] = Nq - 1;

                InvQIndex[ aux_i ] = InvQIndex[ Nq - 1 ];
                InvQIndex[ Nq - 1 ] = i;

                #if     ( ReducedCostMode == 3 )

                #endif
                Nq--;
                }
#endif

return MsgOk;

}

//-----------------------------------------------------------------------------
void            TQPMemRedCostPath::DeleteQ()
{

for( register Index k = CurrentAllocQ ; k-- ; )
        delete [] QQ[ k ];
delete [] QQ;
}


//-----------------------------------------------------------------------------

void            TQPMemRedCostPath::PrintConfigurationInfo ()
{
fprintf( out_info,"------- Configuration of this Run: -------\n");
fprintf( out_info,"Memory Style: %d, CurrentAllocQ = %d, Maximum Allowed = %d, Nq = %d, Size of Block Allocation: %d \n", MemStyleQ, CurrentAllocQ, MaxNumberofQcolumns, Nq, BlockAllocQ);
fprintf( out_info,"Reduce Cost Mode: %d, CountToIopp = %d, CountToIcpp = %d, RedCostBar = %f\n", ReducedCostMode, CountToIopp, CountToIcpp, RedCostBar);
fprintf( out_info,"Exact Computation of <r1,r1> and <r1,r2> ? %d \n",ExactMode);
fprintf( out_info,"Absolute Error Tolerance: %f     Relative Error Tolerance: %f \n",GetAbsTolError(),GetRelTolError());

fflush(out_info);
}

//-----------------------------------------------------------------------------

QPErrorMsg      TQPMemRedCostPath::VerifyQPMem ()
{
QPErrorMsg DebugStatus = MsgOk;


#if ( MemStyleQ == 1 || MemStyleQ == 2 )

if ( Nq != GetNs() )
        DebugStatus = MsgQPIncorrect;
for ( register Index k = 0 ; k < Nq ; k++ ){
        if ( InvQIndex[ k ] < GetMax_n() ){
                if ( QIndex[ InvQIndex[ k ] ] != k  ){
                        DebugStatus = MsgQPIncorrect;
                        }
                }
        }
#endif
return  DebugStatus;

}

//----------------------------------------------------------------------------
//  Time Variables...
//----------------------------------------------------------------------------
#if ( TimeMode )
void TQPMemRedCostPath::InitTimePath ()
{
TimeKKTsystemQP         = TimeKKTsystem;
TimeReduceCostQP        = TimeReduceCost;
TimeAddToBaseQP         = TimeAddToBase;
TimeRemoveFromBaseQP    = TimeRemoveFromBase;
TimeInitQP              = TimeInit;
TimeTotalAllQP          = TimeTotalAll;

TimeKKTsystem           = hpZero;
TimeReduceCost          = hpZero;
TimeAddToBase           = hpZero;
TimeRemoveFromBase      = hpZero;
TimeTotalAll            = hpZero;
//ftime (&TimeStartAll);
TimeStartAll = QPCpuTime();

TimeBreakStep1          = hpZero;
TimeBreakStep2          = hpZero;
TimeUpdates             = hpZero;
}
#endif


/*

        if ( RedCost < hpZero ){

                // Compute true reduced cost...

                WSumVectors( tmpVecM + 2*GetNs(), 1.0e0, tmpVecM, Delta, tmpVecM + GetNs(), GetNs() );

                RedCost  =  SprodQi_dIset ( index_k, tmpVecM + 2*GetNs(), tmpIndVec, GetNs() );
                RedCost -=  ( Getb() + Delta*Getb_deriv() )*TQPclassPath::GetYi( index_k ) + TQPclassPath::GetVi( index_k );
                RedCost +=  NextTheta * GetQicCc ( index_k );


                LowerReducedCost[ index_k ] = min( LowerReducedCost[ index_k ], RedCost);

                if ( RedCost <= 0 ){

                        hpReal Qis_d       = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
                        hpReal slope       = Qis_d - Getb_deriv() * GetYi( index_k ) + GetQicCc ( index_k );
                        if ( slope < -GetAbsTolError () ) {
                                NextTheta      -= RedCost / slope;
                                NextTheta       = max ( GetTheta(), NextTheta );
                                Delta           = NextTheta - GetTheta();

                                if ( RedCost >= 0 )     NumberOfTiesStep2++;
                                else                    NumberOfTiesStep2 = 0;

                                ParametricIndexStep2  = index_k;
                                ParametricEventStep2 = Io2Is;
                                }
						}
                }
        else    {
                LowerReducedCost[ index_k ] = min( LowerReducedCost[ index_k ], RedCost);
				}

		}


for ( register Index k = 0 ; k < tmpNc ; k++ ){
        index_k  =  GetIc( k );


        if ( PositiveKernel() ){
                RedCost  =  LowerReducedCost[ index_k ];
                if ( GetYi( index_k ) > 0 )
                        RedCost -=  Delta * dderiv1POS * GetMaxQi( index_k );
                else    RedCost -=  Delta * dderiv1NEG * GetMaxQi( index_k );
                RedCost +=  Delta * Getb_deriv() * GetYi( index_k );
                RedCost -=  Delta * GetQicCc ( index_k );
                }
        else    {
                RedCost  =  LowerReducedCost[ index_k ];
                RedCost -=  Delta * dderiv1 * GetMaxQi( index_k );
                RedCost +=  Delta * Getb_deriv() * GetYi( index_k );
                RedCost -=  Delta * GetQicCc ( index_k );
                }


        hpReal RedCost0 = -SprodQi_dIset ( index_k, tmpVecM , tmpIndVec, GetNs() );
        RedCost0 -= - Getb() * TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
        RedCost0 -=   GetTheta() * GetQicCc ( index_k );
        if ( RedCost0 < LowerReducedCost[ index_k ] - GetAbsTolError() ){
                RedCost0 = 0;
                }

        WSumVectors( tmpVecM + 2*GetNs(), 1.0e0, tmpVecM, Delta, tmpVecM + GetNs(), GetNs() );

        RedCost0  =   -SprodQi_dIset ( index_k, tmpVecM + 2*GetNs(), tmpIndVec, GetNs() );
		RedCost0 -= - ( Getb() + Delta*Getb_deriv() )*TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
        RedCost0 +=   -NextTheta * GetQicCc ( index_k );
        if ( RedCost0 < RedCost - GetAbsTolError() ){
				RedCost0 = 0;
                }

        if ( RedCost < hpZero ){
				// Compute true reduced cost...
				WSumVectors( tmpVecM + 2*GetNs(), 1.0e0, tmpVecM, Delta, tmpVecM + GetNs(), GetNs() );

                RedCost  = - SprodQi_dIset ( index_k, tmpVecM + 2*GetNs(), tmpIndVec, GetNs() );
				RedCost +=   (Getb() + Delta*Getb_deriv())*TQPclassPath::GetYi( index_k ) + TQPclassPath::GetVi( index_k );
                RedCost -=   NextTheta * GetQicCc ( index_k );

				LowerReducedCost[ index_k ] = min( LowerReducedCost[ index_k ], RedCost);

                if ( RedCost <= 0 ){

                        hpReal Qis_d    = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
                        hpReal slope    = -Qis_d + Getb_deriv() * GetYi( index_k ) - GetQicCc ( index_k );
                        if ( slope < -GetAbsTolError () ) {
                                NextTheta      -= RedCost / slope;
                                NextTheta       = max ( GetTheta(), NextTheta );
                                Delta           = NextTheta - GetTheta();

                                if ( RedCost >= 0 )     NumberOfTiesStep2++;
                                else                    NumberOfTiesStep2 = 0;

                                ParametricIndexStep2 = index_k;
                                ParametricEventStep2 = Ic2Is;
                                }

                        }
                }
        else    {
                LowerReducedCost[ index_k ] = min( LowerReducedCost[ index_k ], RedCost);
                }

        }

MinRatioStep2Pos = Delta;

#if ( TimeMode )
        TimeFinish();
        TimeAdd( &TimeBreakStep2 );
#endif

}


//-----------------------------------------------------------------------
void    TQPMemRedCostPath::PathMinRatioTestStep2( hpReal Bound )
{

#if ( TimeMode )
        TimeStart();
#endif

Index index_k;
Index tmpNo     = GetNo();
Index tmpNc     = GetNc();

hpReal RedCost  = hpZero;

NextTheta       = Bound;
hpReal Delta    = NextTheta - GetTheta();

CopyCurrentPointSparse( tmpVecM );
TQPclassPath::CopyCurrentIs( tmpIndVec );
CopyCurrentDirectionSparse( tmpVecM + GetNs() );

// Computes || d ||_1  where  alpha(NextTheta) = alpha(theta) + Delta * d
hpReal dderiv1 = hpZero;
for ( register Index k = GetNs() ; k-- ;  ) dderiv1 += abs( tmpVecM[ GetNs() + k ] );


// Io ----------------------------------------
for ( register Index k = 0 ; k < tmpNo ; k++ ){
        index_k  =  GetIo( k );

        WSumVectors( tmpVecM + 2*GetNs(), 1.0e0, tmpVecM, Delta, tmpVecM + GetNs(), GetNs() );

        RedCost  =   SprodQi_dIset ( index_k, tmpVecM + 2*GetNs(), tmpIndVec, GetNs() );
        RedCost += - ( Getb() + Delta*Getb_deriv() )*TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
        RedCost +=   NextTheta * GetQicCc ( index_k );

        if ( RedCost <= 0 ){
//        if ( RedCost <= - GetAbsTolError() ){
                hpReal Qis_d       = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
                hpReal slope       = Qis_d - Getb_deriv() * GetYi( index_k ) + GetQicCc ( index_k );

                if ( slope < - GetAbsTolError () ) {
                                NextTheta      -= RedCost / slope;
                                NextTheta       = max ( GetTheta(), NextTheta );
                                Delta           = NextTheta - GetTheta();


								if ( RedCost >= 0 )     NumberOfTiesStep2++;
                                else                    NumberOfTiesStep2 = 0;

                                ParametricIndexStep2  = index_k;
                                ParametricEventStep2 = Io2Is;
                                }

                }
        else if ( 0 && RedCost <= 0 ) {
				hpReal Qis_d       = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
                hpReal slope       = Qis_d - Getb_deriv() * GetYi( index_k ) + GetQicCc ( index_k );

                if ( slope < - GetAbsTolError () ) {
                        NumberOfTiesStep2++;
                        }
                }
        }



// Ic ----------------------------------------
for ( register Index k = 0 ; k < tmpNc ; k++ ){
        index_k  =  GetIc( k );

        WSumVectors( tmpVecM + 2*GetNs(), 1.0e0, tmpVecM, Delta, tmpVecM + GetNs(), GetNs() );

        RedCost  = - SprodQi_dIset ( index_k, tmpVecM + 2*GetNs(), tmpIndVec, GetNs() );
        RedCost +=   ( Getb() + Delta*Getb_deriv() )*TQPclassPath::GetYi( index_k ) + TQPclassPath::GetVi( index_k );
        RedCost -=   NextTheta * GetQicCc ( index_k );

        if ( RedCost <= 0 ){
//        if ( RedCost <= - GetAbsTolError() ){
                hpReal Qis_d    = SprodQi_dIset ( index_k, tmpVecM + GetNs(), tmpIndVec, GetNs() );
                hpReal slope    = -Qis_d + Getb_deriv() * GetYi( index_k ) - GetQicCc ( index_k );

                if ( slope < -GetAbsTolError () ) {
                                NextTheta      -= RedCost / slope;
                                NextTheta       = max ( GetTheta(), NextTheta );
                                Delta           = NextTheta - GetTheta();

                                if ( RedCost >= 0 )     NumberOfTiesStep2++;
                                else                    NumberOfTiesStep2 = 0;

								ParametricIndexStep2 = index_k;
								ParametricEventStep2 = Ic2Is;
                                }
                }
        }

MinRatioStep2Pos = Delta;

#if ( TimeMode )
        TimeFinish();
        TimeAdd( &TimeBreakStep2 );
#endif

}





        CopyCurrentPointSparse( tmpVecM );
        TQPclassPath::CopyCurrentIs( tmpIndVec );
        for ( Index k = 0 ; k < GetNo() ; k++ ){
                Index index_k = GetIo( k );

                hpReal RedCost0 = SprodQi_dIset ( index_k, tmpVecM , tmpIndVec, GetNs() );
                RedCost0 += - Getb() * GetYi( index_k ) - GetVi( index_k );
                RedCost0 +=   GetTheta() * GetQicCc ( index_k );
                if ( RedCost0 < LowerReducedCost[ index_k ] - GetAbsTolError() ){
                        RedCost0 = 0;
                        }
                }




        hpReal RedCost0 = SprodQi_dIset ( index_k, tmpVecM , tmpIndVec, GetNs() );
        RedCost0 += - Getb() * TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
        RedCost0 +=   GetTheta() * GetQicCc ( index_k );
        if ( RedCost0 < LowerReducedCost[ index_k ] - GetAbsTolError() ){
                RedCost0 = 0;
                }

        WSumVectors( tmpVecM + 2*GetNs(), 1.0e0, tmpVecM, Delta, tmpVecM + GetNs(), GetNs() );

        RedCost0  =   SprodQi_dIset ( index_k, tmpVecM + 2*GetNs(), tmpIndVec, GetNs() );
        RedCost0 += - ( Getb() + Delta*Getb_deriv() )*TQPclassPath::GetYi( index_k ) - TQPclassPath::GetVi( index_k );
        RedCost0 +=   ( GetTheta() + Delta ) * GetQicCc ( index_k );
		if ( RedCost0 < LowerReducedCost[index_k] + Delta * LowerSlope[index_k] - GetAbsTolError() ){
                RedCost0 = 0;
                }

*/
