//---------------------------------------------------------------------------
#include "QPclassPath.h"
#include "QPtime.h"
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Class  TQPclassPath
//---------------------------------------------------------------------------
TQPclassPath::TQPclassPath()
{
Ns = 0;
Nc = 0;
No = 0;
Iteration = 0;
InnerIter = 0;
BlockAllocL             = 0;
CurrentAllocL           = 0;

out_info = fopen("QPreport.svm", "w");

#if ( TimeMode )
		TimeKKTsystem = 0;
		TimeAddToBase = 0;
		TimeRemoveFromBase = 0;
		TimeReduceCost = 0;
		TimeInit = 0;
		// ftime (&TimeStartAll);
		TimeStartAll = QPCpuTime();
#endif

Is = NULL;
Io = NULL;
Ic = NULL;
PlaceIs = NULL;
PlaceIc = NULL;
PlaceIo = NULL;
Y = NULL;
C = NULL;
V = NULL;
r1 = NULL;
r2 = NULL;
alpha = NULL;
lin_obj = NULL;
alpha_iter = NULL;
dd = NULL;
tmpVec = NULL;
L = NULL;

NoBias = FALSE;

theta = 1;
DataConsistentTheta = TRUE;
z1 = NULL;
z2 = NULL;
MinRatioStep1Pos = hpZero;
MinRatioStep1Neg = hpZero;
InfiniteDirection = FALSE;
r1z1            = hpZero;
r1z2            = hpZero;

lin_cte_obj     = hpZero;
quad_cte_obj    = hpZero;
cte_obj         = hpZero;
f_obj           = hpZero;

PrintMode 		=  1;
MaxIteration 	= 20000;
OldIndex 		= 0;
Max_n 			= 0;
n				= 0;
Old_f_obj 		= hpInf;

Iteration = 0;
InnerIter = 0;
}


//-----------------------------------------------------------------------------
QPErrorMsg TQPclassPath::TQPclassPathInit( Index Max_n0, Index n0, hpVec Y0, hpVec V0, hpVec C0 )
{
n  = n0;
Ns = 0;
Nc = 0;
No = n;
Max_n = Max_n0;

Iteration = 0;
InnerIter = 0;

BlockAllocL             = min( Max_n, Index(51) ) - 1;

QPStatus                = MsgOk;
CountMsgInvalidIndex    = 0;
CountMsgNumericProblem  = 0;
DegreeDependence        = 0;
LocalOptima             = FALSE;
BindingIndex            = Max_n;
StepSize                = 0.0e0;
BaseEvent               = NoEvent;
NewIndexToBeAdded       = Max_n;

OldIndex = Max_n;

// Memory for Indices
memalloc( Is      , Index , Max_n );
memalloc( Ic      , Index , Max_n );
memalloc( Io      , Index , Max_n );
memalloc( PlaceIs , Index , Max_n );
memalloc( PlaceIc , Index , Max_n );
memalloc( PlaceIo , Index , Max_n );

VecInit ( PlaceIs , Max_n , Max_n );
VecInit ( PlaceIc , Max_n , Max_n );
VecInit ( Is      , Max_n , Max_n );
VecInit ( Ic      , Max_n , Max_n );
for (register Index k = Max_n ; k-- ; ){
		Io[k]      = k;
		PlaceIo[k] = k;
		}
// Memory for Problem Data (all but Q)
memalloc( Y , hpReal, Max_n );
memalloc( C , hpReal, Max_n );
memalloc( V , hpReal, Max_n );

AssignVectors( Y, Y0, n );
AssignVectors( V, V0, n );
AssignVectors( C, C0, n );
if ( Max_n - n > 0 ) VecInit ( Y + n, hpZero, Max_n - n );
if ( Max_n - n > 0 ) VecInit ( V + n, hpZero, Max_n - n );
if ( Max_n - n > 0 ) VecInit ( C + n, hpZero, Max_n - n );

// Memory for Workspace
memalloc( r1, hpReal, Max_n );
memalloc( r2, hpReal, Max_n );
memalloc( alpha,      hpReal, Max_n );
memalloc( lin_obj,    hpReal, Max_n );
memalloc( alpha_iter, hpReal, Max_n );
memalloc( dd,         hpReal, Max_n );
memalloc( tmpVec,     hpReal, 2*Max_n );
memalloc( z1, hpReal, Max_n );
memalloc( z2, hpReal, Max_n );

b=hpZero;
b_iter = hpZero;
YcCc = hpZero;
VecInit(alpha_iter, hpZero, Max_n );
VecInit(alpha, hpZero, Max_n );
VecInit(dd, hpZero, Max_n );
VecInit(lin_obj, hpZero, Max_n );
VecInit(r1, hpZero, Max_n );
VecInit(r2, hpZero, Max_n );
VecInit(z1, hpZero, Max_n );
VecInit(z2, hpZero, Max_n );

memalloc( L  , hpVec, Max_n);
NewIndexToBeAdded       = Max_n;
ReducedCostMin          = hpZero;

InfiniteDirectionCount  = 0;
Case0Count              = 0;
Case1Count              = 0;
Case2Count              = 0;

// Other Constants....
AbsTolError = 1.0e-7;
RelTolError = 1.0e-7;

lin_cte_obj     = hpZero;
quad_cte_obj    = hpZero;
cte_obj  = hpZero;
f_obj    = hpZero;
b_deriv  = hpZero;

return MsgOk;
}
//-----------------------------------------------------------------------
void TQPclassPath::PrintCurrentSolution ( FILE *out )
{
Index k;
hpReal val=0.0e0;
for (  k = 0 ; k < n ; k++ )
		{
		if      ( PlaceIs[k] < Max_n )
				val = alpha_iter[PlaceIs[k]];
		else if ( PlaceIc[k] < Max_n )
				val = theta*C[k];
		else    val = 0.0e0;
		fprintf( out, "%d  %lf \n", k+1,  val );
		}
}
//-----------------------------------------------------------------------
void TQPclassPath::PrintCurrentSolutionSparse ( FILE *out )
{
Index k;
hpReal val=0.0e0;
for (  k = 0 ; k < n ; k++ )
		{
		if ( PlaceIo[k] < Max_n ) continue;

		if      ( PlaceIs[k] < Max_n )
				val = alpha_iter[PlaceIs[k]];
		else    val = theta*C[k];
		fprintf( out, "%d  %lf \n", k+1,  val );
        }
}

//---------------------------------------------------------------------------
void     TQPclassPath::SetNoBias( boolean val )
{
if ( InnerIter == 0 ){
		NoBias = val;
		}
else    {
		TQPclassPath::PrintErrorMsg ("NoBias option cannot be changed after solving the problem.");
        }
}

//-----------------------------------------------------------------------
QPErrorMsg   TQPclassPath::CheckCycling ()
{

// Check for Potential Cycle...
if ( OldIndex == NewIndexToBeAdded && Old_f_obj <= f_obj ){
	// if Precision was already updated too much
	if ( AbsTolError <= 1.0e-9 ) 	return MsgCycle;
	// else ask for more precision
	AbsTolError *= 0.5;
	RelTolError *= 0.5;
	Old_f_obj = f_obj;
	return MsgPotentialCycle;
	}
else {
	// No signs of cycling
	OldIndex  = NewIndexToBeAdded;
	Old_f_obj = f_obj;
	// if high precision we can relax
	if ( AbsTolError <= 0.5e-7 ){
		AbsTolError *= 2;
		RelTolError *= 2;
		}
	return MsgOk;
	}

}


//-----------------------------------------------------------------------
QPErrorMsg TQPclassPath::SolveQP()
{
Iteration = 0;
while ( NeedToGetNewIndex() && Iteration <= MaxIteration ) {

		QPStatus = CheckCycling ();

		if ( QPStatus == MsgCycle ){
				PrintErrorMsg("Cycle detected, exiting the program.\n");
				break;
				}

		// Increase the number of iterations
		Iteration++;

		// From either Io or Ic (with primes...)
		QPStatus = TQPclassPath::AddVariableToBase();

		if ( QPStatus == MsgNumericalProb ){
				PrintErrorMsg("Numerical Instability Detected: recomputing data structures.");
				QPStatus = TQPclassPath::RecomputeAll ();
				}

		if ( QPStatus == MsgFatal ){
				PrintErrorMsg("A fatal error occured when recomputing the data structures.");
				break;
				}

		if ( QPStatus == MsgInvalidIndex ){
				PrintErrorMsg("New Variable to Be Added Not Consistent with Current Data.");
				#if ( PrintVisual == 0 )
					printf ( "Inconsistent Index = %ld and Event = %ld \n ", NewIndexToBeAdded, BaseEvent );
				#endif
				
				CountMsgInvalidIndex++;
				// Allow for some errors... as long as we keep consistency...
				if ( CountMsgInvalidIndex > Index( max( 10, int( 0.02 * Iteration ) ) ) ) break;
				}

		#if ( DebugMode >= 2 )
				// Verify all Data....
				if ( VerifyQPdata () == MsgQPIncorrect ){
						PrintErrorMsg("QP data structure is not consistent: \n **** TERMINATE PROGRAM **** \n");
						break;
						}
		#endif

		//Find the optimal solution for the current base
		QPStatus = TQPclassPath::SolveCurrentBase ();

		if ( QPStatus == MsgFatal ){
				PrintErrorMsg("Fatal Error After Solving Current Base: \n **** TERMINATE PROGRAM **** \n");
				break;
				}

		// Compute Objective Function
		ComputeObjFuncFast();

//-----------------------------------------------------------------------------
		UpdateEveryIteration ();
//-----------------------------------------------------------------------------

		#if ( DebugMode >= 2 )
				// Verify all Data....
				if ( VerifyQPdata () == MsgQPIncorrect ){
						PrintErrorMsg("QP data structure is not consistent: \n **** TERMINATE PROGRAM **** \n");
						break;
						}
		#endif

		if ( PrintMode >= 2 )
				PrintBasicInfo ();


		} // End of outer loop -----------------------------------------------

#if ( DebugMode )
// Final Optimality Check under DebugMode-------------------------------------
		if ( VerifyMinReducedCost () < -AbsTolError ){
				PrintErrorMsg("Reduced Costs Are NOT Nonnegative --- Solution NOT optimal");
				}
		else    {
				PrintInformativeMsg ("Final solution is Feasible and All Reduced Costs Nonnegative\n --- Solution is Optimal ----");
				}
		if ( VerifyQPdata () == MsgQPIncorrect ){
				PrintErrorMsg("QP data structure is not consistent at the end.");
				}
#endif

if ( MaxIteration == Iteration ){
		PrintErrorMsg("Maximum number of iterations reached: optimization stopped.");
		}

if ( PrintMode >= 2 ){
		PrintBasicInfo ();
		PrintConfigurationInfo ();
		}

UpdateEveryIteration ();


return QPStatus;
}

//-----------------------------------------------------------------------
QPErrorMsg TQPclassPath::SolveCurrentBase ()
{

QPErrorMsg CurrentBaseStatus = MsgOk;
do      {

		// Increase the Number of Inner Iterations
        InnerIter++;

        CurrentBaseStatus = TQPclassPath::SolveKKT ();

        if ( CurrentBaseStatus == MsgNumericalProb ){
                PrintErrorMsg("Numerical Instability Detected: recomputing data structures.\n");
                CurrentBaseStatus = TQPclassPath::RecomputeAll ();
                }

		if ( CurrentBaseStatus == MsgFatal ){
                PrintErrorMsg("A fatal error occured when recomputing the data structures.\n");
                break;
                }

        CurrentBaseStatus = TQPclassPath::MoveAlongDirection ();

        if ( CurrentBaseStatus == MsgNumericalProb ){
                PrintErrorMsg("Numerical Instability Detected: recomputing data structures.\n");
                CurrentBaseStatus = TQPclassPath::RecomputeAll ();
                }

        if ( CurrentBaseStatus == MsgFatal ){
                PrintErrorMsg("A fatal error occured when recomputing the data structures.\n");
                break;
                }
		#if ( DebugMode >= 3 )
				if ( VerifyQPdata () == MsgQPIncorrect ){
                        PrintErrorMsg("Data structure after MoveAlongDirection is not consistent.\n");
                        }
        #endif

        } while ( !LocalOptima )  ; // End of inner loop ---------------------

return CurrentBaseStatus;
}


//-----------------------------------------------------------------------
QPErrorMsg TQPclassPath::SolveKKT ()
{
#if ( TimeMode )
		TimeStart();
#endif

hpReal denominator;
//... remember: r1 and r2 were updated when we updated L.
//... we will need only one backsolve per iteration.
switch ( DegreeDependence ){

        case 0: // Qss is non-singular
                //-------------------------------------------------------
                Case0Count++;

                // b = (<r1,r2> - <Yc,Cc>) / <r1,r1>
                // b = 0,  if NoBias == TRUE;
				if ( NoBias ){
                        b = 0.0e0;
                        }
                else    {
                        #if ( ExactMode )
                                b  = ( sprod( r1, r2, Ns) - theta*sprod( Y, C, Ic, Nc) ) / sprod( r1,r1,Ns);
                        #else
                                b  = ( r1r2 - theta*YcCc ) / r1r1;
                        #endif
                        }

                // tmpVec = b r1 - r2
                WSumVectors ( tmpVec, b, r1, -1.0e0, r2, Ns);

				// alpha = (L^T)^{-1} tmpVec
                TriangularSystemTranspose( L, tmpVec, alpha, Ns);

                // dd = alpha - alpha_iter
                DiffVectors ( dd, alpha, alpha_iter, Ns);

                LocalOptima = TRUE;
                break; //.... end of Case 0....


        case 1: // L has exactly 1 linear dependent row...
                //-------------------------------------------------------
                Case1Count++;

                                denominator = 0;

				if ( !NoBias ){
                        #if ( ExactMode )
								r1r1  = sprod( r1, r1, Ns-1 );
                                r1r2  = sprod( r1, r2, Ns-1 );
                        #endif

                        r1Lh1 = sprod( r1, L[ Ns-1 ], Ns-1 ); // DegreeDependence = 1
                        r2Lh1 = sprod( r2, L[ Ns-1 ], Ns-1 ); // DegreeDependence = 1

                        denominator = Y[ Is[Ns-1] ] - r1Lh1;
                        }

                if ( !NoBias && abs( denominator ) > AbsTolError ){
                        // Unique Solution Can Be Found... despite the rank deficiency of Qss

                        b =  ( lin_obj[ Ns - 1 ] - r2Lh1 ) / denominator;
						alpha[ Ns - 1 ] = ( - theta*YcCc + r1r2 - b * r1r1 ) / denominator;

                        // tmpVec = b r1 - alpha[Ns-1] Lh
                        WSumVectors ( tmpVec, b, r1, -alpha[ Ns-1 ], L[ Ns - 1], Ns);

                        // tmpVec -= r2
                        DiffVectors ( tmpVec,tmpVec, r2, Ns);

                        // alpha = (L^T)^{-1} [ b r1 - r2 - alpha[Ns-1] Lh ]
                        TriangularSystemTranspose( L, tmpVec, alpha, Ns - 1 ); // Ns-1 = Ns - DegreeDependence

						// dd = alpha - alpha_iter
                        DiffVectors ( dd, alpha, alpha_iter, Ns);

                        LocalOptima = TRUE;
                        }
				else  { // Infinite Direction will be found...
                        InfiniteDirectionCount++;

						// dd = (L^T)^{-1} L_h
                        TriangularSystemTranspose( L, L[ Ns - 1], dd, Ns - DegreeDependence );

						dd[ Ns - 1 ] = -1;

						//if ( sprod ( dd, lin_obj, Ns) > 0 ) // need to multiply by -1
						if ( BaseEvent == Io2Is )
								ScalingVector( dd, -1.0e0, dd, Ns );

						LocalOptima = FALSE;

						b = b_iter;
						}

				break; //.... end of Case 1....

		default: // L has more than 1 linear dependent row...
				//-------------------------------------------------------
				// Always infinite direction....
				Case2Count++;

				b = b_iter;

				denominator = 0;

				LocalOptima = FALSE;

				InfiniteDirectionCount++;

				if ( !NoBias ){
						// recall: Lh2 is the last row of L
						r1Lh2 = sprod( r1, L[Ns - 1], Ns - DegreeDependence);

						denominator = Y[ Is[ Ns-1 ] ] - r1Lh2;
						}

				if ( !NoBias && abs( denominator ) > AbsTolError ) {

						// we have some freedom to choose Lh1, we should not need to recompute it...
						r1Lh1        = sprod( r1, L[ Ns - 2 ], Ns - DegreeDependence);
						hpReal tmphp = Y[ Is[ Ns - 2 ] ] - r1Lh1;
						tmphp       /= denominator;

						for ( register Index i = Ns - DegreeDependence ; i < Ns - 1 ; )
								dd[ i++ ] = hpZero;

						dd[ Ns - 2 ] = -1.0e0;
						dd[ Ns - 1 ] = tmphp;

						WSumVectors ( tmpVec, 1.0e0, L[ Ns-2 ], -tmphp, L[Ns -1], Ns - DegreeDependence);

						TriangularSystemTranspose( L, tmpVec, dd, Ns - DegreeDependence );

						// Check if dd or -dd is an infinite direction...
//						if ( sprod ( dd, lin_obj, Ns) > 0 ) // need to multiply by -1
						if ( (tmphp < 0 && BaseEvent == Io2Is) || ( tmphp > 0 && BaseEvent == Ic2Is ) )
								ScalingVector( dd, -1.0e0, dd, Ns );

						}
                else { // denominator == 0

						TriangularSystemTranspose( L, L[ Ns - 1], dd, Ns - DegreeDependence );

                        for ( register Index i = Ns - DegreeDependence ; i < Ns ; )
                                dd[ i++ ] = hpZero;
                        dd[ Ns - 1 ] = -1.0e0;

                        // Check if dd or -dd is an infinite direction...
                        if ( sprod ( dd, lin_obj, Ns) > 0 ) // need to multiply by -1
                                ScalingVector( dd, -1.0e0, dd, Ns );
                        }

                LocalOptima = FALSE;
                break; //.... end of Case 2....
        }

#if ( TimeMode )
        TimeFinish();
        TimeAdd( &TimeKKTsystem );
#endif

return MsgOk;
}

//-----------------------------------------------------------------------
QPErrorMsg TQPclassPath::JumpSolveQP( hpReal JumpTheta )
{

if ( JumpTheta < AbsTolError ){
	PrintErrorMsg("Theta cannot jump to nearly zero values.");
	return MsgFatal;
	}

// ensures feasibility.... for a good starting point
// I no longer have an optimal solution...
ScalingVector ( alpha_iter, JumpTheta/theta, alpha_iter,  Ns );
b_iter +=  (JumpTheta/theta)* ( r1z1 - YcCc )/r1r1;


// need to update other quantities...
theta = JumpTheta;
WSumVectors ( r2, theta, z1, -1.0e0, z2, Ns - DegreeDependence);
for( register Index k = 0 ; k < Ns ; k++ )
		lin_obj[ k ] = theta*GetQicCc( Is[ k ] ) - V[ Is[ k ] ];
cte_obj = theta*lin_cte_obj + theta*theta*quad_cte_obj;

#if ( ExactMode )
		r1r2 = theta*r1z1-r1z2;
#endif

DataConsistentTheta = TRUE;

// let the code solve now...
return TQPclassPath::SolveQP();

}
//---------------------------------------------------------------------------
QPErrorMsg     TQPclassPath::SetTheta( hpReal val )
{
QPErrorMsg UpdateThetaStatus = MsgOk;

if ( val < hpZero ){
		PrintErrorMsg("Theta cannot have negative value (problem infeasible).");
		return MsgFatal;
		}

hpReal delta = hpZero;
if ( val - theta > hpZero ){
		delta = val - theta;
		}


if ( Ns && delta > MinRatioStep1Pos + AbsTolError ) {
		PrintErrorMsg("This value of theta violates feasibility constraints for this basis.");
		PrintErrorMsg("Step Reduced");
		delta = MinRatioStep1Pos;
		val   = theta + delta;
		UpdateThetaStatus = MsgNumericalProb;
		}


theta = val;

if ( delta > 0 ){
		WSumVectors ( r2, theta, z1, -1.0e0, z2, Ns - DegreeDependence);
		for( register Index k = 0 ; k < Ns ; k++ )
				lin_obj[ k ] = theta*GetQicCc( Is[ k ] ) - V[ Is[ k ] ];
		}

cte_obj = theta*lin_cte_obj + theta*theta*quad_cte_obj;

#if ( ExactMode )

		r1r2 = theta*r1z1-r1z2;

#endif

// Note that reduced cost is not a responsability of this class...

if ( delta > hpZero ){
		WSumVectors( alpha_iter, 1.0e0, alpha_iter, delta, dd, Ns);
		b_iter += delta * b_deriv;
		}
else if ( InfiniteDirection ){
		WSumVectors( alpha_iter, 1.0e0, alpha_iter, MinRatioStep1Pos, dd, Ns);
		}
hpReal f_obj_old = f_obj;

ComputeObjFuncFast();

if ( delta > AbsTolError )
		f_deriv_num = (f_obj - f_obj_old) / delta;

DataConsistentTheta = TRUE;
return MsgOk;
}

//---------------------------------------------------------------------------

void   TQPclassPath::ComputeMinRatioTestStep1()
{
// Current Solution is optimal implies unique solution of KKT
hpReal denominator;
InfiniteDirection = FALSE;

if ( !Ns ){
        MinRatioStep1Pos = hpInf;
        MinRatioStep1Neg = hpInf;
        BaseEvent   = NoEvent;
        return;
        }

switch ( DegreeDependence ){
        case 0:
                #if ( ExactMode )
                        r1r2 = sprod( r1, r2, Ns);
                        r1z1 = sprod( r1, z1, Ns);
						r1r1 = sprod( r1, r1, Ns);
						YcCc = sprod( Y, C, Ic, Nc);
				#endif

				if ( NoBias ){
                        b = 0;
                        b_deriv = 0;
                        }
                else    {
                        b  = ( r1r2 - theta*YcCc ) / r1r1;

                        b_deriv = ( r1z1 - YcCc )/r1r1;
                        }

				WSumVectors( tmpVec, b_deriv, r1, -1.0e0, z1, Ns - DegreeDependence );
                // tmpVec has the derivatives of alpha_iter
                TriangularSystemTranspose( L, tmpVec, dd, Ns - DegreeDependence );
                break;

        case 1:
                #if ( ExactMode )
                        r1z1 = sprod( r1, z1, Ns - DegreeDependence);
                        r1r1 = sprod( r1, r1, Ns - DegreeDependence);
                        YcCc = sprod( Y, C, Ic, Nc);
                        r1Lh1 = sprod( r1, L[ Ns-1 ], Ns-1 ); // DegreeDependence = 1
                        r2Lh1 = sprod( r2, L[ Ns-1 ], Ns-1 ); // DegreeDependence = 1
                #endif

                denominator = Y[ Is[ Ns - 1 ] ] - r1Lh1;

				if ( abs( denominator ) > AbsTolError ){

						if ( NoBias ){
                                b=0;
                                b_deriv = 0;
                                }
                        else    {
								b =  ( lin_obj[ Ns - 1 ] - r2Lh1 ) / denominator;

                                b_deriv      = GetQicCc ( Is[ Ns - 1 ] ) - sprod( L[ Ns - 1 ], z1, Ns - DegreeDependence );
                                b_deriv     /= denominator;
                                }
                        dd[ Ns - 1 ] = ( - YcCc + r1z1 - b_deriv * r1r1 ) / denominator;

                        WSumVectors( tmpVec, b_deriv, r1, -1.0e0, z1, Ns - DegreeDependence );
                        WSumVectors( tmpVec, 1.0e0, tmpVec, -dd[ Ns - 1 ], L[ Ns - 1 ], Ns - DegreeDependence );

						TriangularSystemTranspose( L, tmpVec, dd, Ns - DegreeDependence );
						}
                else    {
                        // Shouldn't happen...
						PrintErrorMsg("Infinite Direction generated when we should have \n unique solution of the KKT system" );

						b_deriv = 0;

                        TriangularSystemTranspose( L, L[ Ns - 1], dd, Ns - DegreeDependence );

                        dd[ Ns - 1 ] = -1;

                        for ( register Index k = 0 ; k < Ns ; k++ ) tmpVec[k] = GetQicCc( Is[k] );

						// Need to minimize < QscCc , d >
//                        if ( sprod ( dd, tmpVec, Ns) < 0 ) // need to multiply by -1
						if ( alpha_iter[ Ns - 1 ] < AbsTolError ) // need to multiply by -1
                                ScalingVector( dd, -1.0e0, dd, Ns );

                        InfiniteDirection = TRUE;
                        }

                break;

        case 2: // L has more than 1 linear dependent row...
                //-------------------------------------------------------
                // Always infinite direction....

                // recall: Lh2 is the last row of L
                r1Lh2 = sprod( r1, L[Ns - 1], Ns - DegreeDependence);

                denominator = Y[ Is[ Ns-1 ] ] - r1Lh2;

                if ( abs( denominator ) > AbsTolError ) {

                        // we have some freedom to choose Lh1, we should not need to recompute it...
                        r1Lh1 = sprod( r1, L[ Ns - 2 ], Ns - DegreeDependence);
                        hpReal tmphp = Y[ Is[ Ns - 2 ] ] - r1Lh1;
                        tmphp /= denominator;

                        for ( register Index i = Ns - DegreeDependence ; i < Ns - 1 ; )
                                dd[ i++ ] = hpZero;

                        dd[ Ns - 2 ] = -1.0e0;
                        dd[ Ns - 1 ] = tmphp;

						b_deriv = 0;

                        WSumVectors ( tmpVec, 1.0e0, L[ Ns-2 ], -tmphp, L[Ns -1], Ns - DegreeDependence);

                        TriangularSystemTranspose( L, tmpVec, dd, Ns - DegreeDependence );

                        for ( register Index k = 0 ; k < Ns ; k++ ) tmpVec[k] = GetQicCc( Is[k] );

                        // Check if dd or -dd is an infinite direction...
//                        if ( sprod ( dd, tmpVec, Ns) < 0 ) // need to multiply by -1
                        if ( alpha_iter[ Ns - 1 ] < AbsTolError ) // need to multiply by -1
                                ScalingVector( dd, -1.0e0, dd, Ns );

                        }
                else { // denominator == 0

                        b_deriv = 0;

                        TriangularSystemTranspose( L, L[ Ns - 1], dd, Ns - DegreeDependence );

                        for ( register Index i = Ns - DegreeDependence ; i < Ns ; )
                                dd[ i++ ] = hpZero;
                        dd[ Ns - 1 ] = -1.0e0;

                        for ( register Index k = 0 ; k < Ns ; k++ ) tmpVec[k] = GetQicCc( Is[k] );

                        // Check if dd or -dd is an infinite direction...
                        //if ( sprod ( dd, tmpVec, Ns) < 0 ) // need to multiply by -1
						if ( alpha_iter[ Ns - 1 ] < AbsTolError ) // need to multiply by -1
                                ScalingVector( dd, -1.0e0, dd, Ns );


                        }

                InfiniteDirection = TRUE;
                break; //.... end of Case 2....

        default: // Shouldn't happen
                PrintErrorMsg("Changing the parameter with two (or more) linear dependent rows");
                break;
        }

// dd has the derivatives of alpha_iter
// Current Solution is Optimal for theta...

MinRatioStep1Pos = hpInf;
MinRatioStep1Neg = hpInf;
BaseEvent   = NoEvent;
BindingIndex = GetMax_n();
NumberOfTiesStep1 = 0;
// Min-Ratio Test

if ( !InfiniteDirection ){
        for ( register Index k = Ns ; k-- ; ){
				if      ( dd[k] < -AbsTolError && MinRatioStep1Pos >= alpha_iter[k] / ( - dd[k] ) ){
						if ( MinRatioStep1Pos <= alpha_iter[k] / ( - dd[k] ) )
                                NumberOfTiesStep1++;
                        else    {
								NumberOfTiesStep1 = 0;
                                //MinRatioStep1Pos  = max( hpZero , alpha_iter[k] / ( - dd[k] ));
								MinRatioStep1Pos  = alpha_iter[k] / ( - dd[k] );
                                }
                        BindingIndex    = Is[k];
						BaseEvent       = Is2Io;
                        }
                else if ( dd[k] >  C[Is[k]] + AbsTolError && MinRatioStep1Pos >= (theta*C[ Is[k] ]-alpha_iter[k])/( dd[k] - C[Is[k]] ) ) {
                        if ( MinRatioStep1Pos <= (theta*C[ Is[k] ]-alpha_iter[k])/( dd[k] - C[Is[k]] ) )
                                NumberOfTiesStep1++;
                        else    {
                                NumberOfTiesStep1 = 0;
                                //MinRatioStep1Pos= max( hpZero , (theta*C[ Is[k] ]-alpha_iter[k])/( dd[k] - C[Is[k]] ) );
                                MinRatioStep1Pos= (theta*C[ Is[k] ]-alpha_iter[k])/( dd[k] - C[Is[k]] ) ;
                                }
                        BindingIndex    = Is[k];
                        BaseEvent       = Is2Ic;
                        }

/*
				if      ( dd[k] < C[Is[k]] - AbsTolError && MinRatioStep1Neg >= (theta*C[ Is[k] ]-alpha_iter[k])/(C[Is[k]] - dd[k])  ){
						if ( MinRatioStep1Neg <= (theta*C[ Is[k] ]-alpha_iter[k])/(C[Is[k]] - dd[k]) )
								;//NumberOfTiesStep1++;
						else    {
								//NumberOfTiesStep1       = 0;
								MinRatioStep1Neg        = max( hpZero , (theta*C[ Is[k] ]-alpha_iter[k])/(C[Is[k]] - dd[k]));
								}
						}
				else if ( dd[k] > AbsTolError && MinRatioStep1Neg >= alpha_iter[k]/dd[k] ) {
						if ( MinRatioStep1Neg <= alpha_iter[k]/dd[k] )
								;//NumberOfTiesStep1++;
						else    {
								//NumberOfTiesStep1       = 0;
								MinRatioStep1Neg        = max( hpZero , alpha_iter[k]/dd[k]);
								}
						}
*/

				}
		}
else    { // Infinite Direction == TRUE
		  // theta does not change...
		for ( register Index k = Ns ; k-- ; ){
				if      ( dd[k] < -AbsTolError && MinRatioStep1Pos >= alpha_iter[k] / ( - dd[k] ) ){
						if ( MinRatioStep1Pos <= alpha_iter[k] / ( - dd[k] ) )
								NumberOfTiesStep1++;
						else    {
								NumberOfTiesStep1 = 0;
								//MinRatioStep1Pos  = max( hpZero , alpha_iter[k] / ( - dd[k] ));
								MinRatioStep1Pos  = alpha_iter[k] / ( - dd[k] );
								}
						BindingIndex    = Is[k];
						BaseEvent       = Is2Io;
						}
				else if ( dd[k] >  AbsTolError && MinRatioStep1Pos >= (theta*C[ Is[k] ]-alpha_iter[k])/ dd[k] ) {
						if ( MinRatioStep1Pos <= (theta*C[ Is[k] ]-alpha_iter[k])/ dd[k]  )
								NumberOfTiesStep1++;
						else    {
								NumberOfTiesStep1 = 0;
								MinRatioStep1Pos= (theta*C[ Is[k] ]-alpha_iter[k])/ dd[k]  ;
								}
						BindingIndex    = Is[k];
						BaseEvent       = Is2Ic;
						}

/*
				if      ( dd[k] < - AbsTolError && MinRatioStep1Neg >= (theta*C[ Is[k] ]-alpha_iter[k])/(- dd[k])  ){
						if ( MinRatioStep1Neg <= (theta*C[ Is[k] ]-alpha_iter[k])/( - dd[k]) )
								NumberOfTiesStep1++;
						else    {
								NumberOfTiesStep1       = 0;
								MinRatioStep1Neg        = max( hpZero , (theta*C[ Is[k] ]-alpha_iter[k])/(- dd[k]));
								}
						}
				else if ( dd[k] > AbsTolError && MinRatioStep1Neg >= alpha_iter[k]/dd[k] ) {
						if ( MinRatioStep1Neg <= alpha_iter[k]/dd[k] )
								NumberOfTiesStep1++;
						else    {
								NumberOfTiesStep1       = 0;
								MinRatioStep1Neg        = max( hpZero , alpha_iter[k]/dd[k]);
								}
						}
*/
				}



		}
}

//---------------------------------------------------------------------------
QPErrorMsg      TQPclassPath::AddVariableToBase( Index  i , TypeEvent WhereNext )
{
NewIndexToBeAdded       = i;
BaseEvent               = WhereNext;
return TQPclassPath::AddVariableToBase ();
}
//---------------------------------------------------------------------------
QPErrorMsg TQPclassPath::AddVariableToBase ()
{
#if ( TimeMode )
        TimeStart();
#endif


QPErrorMsg AddVarStatus = MsgOk;

switch ( BaseEvent ){
        case Io2Is:
                // First Ensure that NewIndexToBeAdded is consistent...
                if (    PlaceIs[ NewIndexToBeAdded ] <  Max_n ||
                        PlaceIc[ NewIndexToBeAdded ] <  Max_n ||
                        PlaceIo[ NewIndexToBeAdded ] == Max_n    )
                        AddVarStatus = MsgInvalidIndex;
                else    AddVarStatus = FromZeroToBase ( NewIndexToBeAdded );
                break;
        case Ic2Is:
                // First Ensure that NewIndexToBeAdded is consistent...
                if (    PlaceIs[ NewIndexToBeAdded ] <  Max_n ||
                        PlaceIo[ NewIndexToBeAdded ] <  Max_n ||
                        PlaceIc[ NewIndexToBeAdded ] == Max_n    )
                        AddVarStatus = MsgInvalidIndex;
                else    AddVarStatus = FromCToBase ( NewIndexToBeAdded );
                break;
        default:
                AddVarStatus = MsgInvalidIndex;
        }

#if ( TimeMode )
        TimeFinish();
		TimeAdd( &TimeAddToBase );
#endif

return AddVarStatus;
}

//-----------------------------------------------------------------------
//
void TQPclassPath::MoveIndexTo_Ic ( Index i, hpReal QicCc, hpReal Qii )
{

// constants
YcCc         +=  Y[i]*C[i];
lin_cte_obj  += -V[i]*C[i];
quad_cte_obj += (0.5*C[i]*Qii*C[i] + QicCc*C[i]);
cte_obj       = theta*lin_cte_obj + theta*theta*quad_cte_obj;

// linear part of the objective function...
for ( register Index k = Ns ; k-- ; )
		lin_obj[ k ] += theta*C[i]*GetQij( Is[k], i );

// Reduced Cost structure...


Ic      [ Nc ] = i;
PlaceIc [ i  ] = Nc;
Nc++;
RedCostIndexAddedToIc ( i );
}

//-----------------------------------------------------------------------
void TQPclassPath::MoveIndexOut_Ic ( Index i , hpReal QicCc, hpReal Qii )
{
register Index place_i = PlaceIc[ i ];

// constants...
YcCc         -=  Y[i]*C[i];
lin_cte_obj  -= -V[i]*C[i];
quad_cte_obj -= 0.5*C[i]*Qii*C[i] + QicCc*C[i];
cte_obj       = theta*lin_cte_obj + theta*theta*quad_cte_obj;

// linear part of the objective function...
for ( register Index k = Ns ; k-- ; )
		lin_obj[ k ] -= theta*C[i]*GetQij(Is[k], i );

// Indices...
if ( place_i < Nc - 1 ){
        Ic      [ place_i  ] = Ic  [ Nc - 1 ];
        PlaceIc [ Ic[Nc-1] ] = place_i;
        }
Ic      [ Nc - 1 ] = Max_n;
PlaceIc [    i   ] = Max_n;
Nc--;

// Reduced Cost structure...
RedCostIndexRemovedFromIc ( i );

}

//-----------------------------------------------------------------------
void TQPclassPath::MoveIndexTo_Io ( Index i )
{
Io      [ No ] = i;
PlaceIo [ i  ] = No;
No++;
RedCostIndexAddedToIo ( i );
}

//-----------------------------------------------------------------------
void TQPclassPath::MoveIndexOut_Io ( Index i )
{
register const Index place_i = PlaceIo[ i ];

if ( place_i < No - 1 ){
        Io      [ place_i  ] = Io  [ No - 1 ];
        PlaceIo [ Io[No-1] ] = place_i;
        }
Io      [ No - 1 ] = Max_n;
PlaceIo [    i   ] = Max_n;
No--;

RedCostIndexRemovedFromIo ( i );
}

//-----------------------------------------------------------------------
void TQPclassPath::MoveIndexTo_Is ( Index i, hpReal linobj, hpReal InitVal )
{
Is        [ Ns ] = i;
PlaceIs   [ i  ] = Ns;
lin_obj   [ Ns ] = linobj;
alpha_iter[ Ns ] = InitVal;
Ns++;
}
//-----------------------------------------------------------------------
void TQPclassPath::MoveIndexOut_Is ( Index i )
{
register const Index place_i = PlaceIs[ i ];

if ( place_i < Ns - 1 ){
        alpha_iter[ place_i ]   = alpha_iter[ Ns - 1 ];
        alpha     [ place_i ]   = alpha     [ Ns - 1 ];
        dd        [ place_i ]   = dd        [ Ns - 1 ];
        lin_obj   [ place_i ]   = lin_obj   [ Ns - 1 ];

        Is        [ place_i  ]  = Is[ Ns - 1 ];
        PlaceIs   [ Is[Ns-1] ]  = place_i;
        }

alpha_iter[ Ns - 1 ] = hpZero;
alpha     [ Ns - 1 ] = hpZero;
dd        [ Ns - 1 ] = hpZero;
lin_obj   [ Ns - 1 ] = hpZero;
Is        [ Ns - 1 ] = Max_n;
PlaceIs   [   i    ] = Max_n;
Ns--;
}


//-----------------------------------------------------------------------
QPErrorMsg TQPclassPath::FromBaseToZero ( Index i )
{
// The following informaiton is updated here:
// Q, L, Is, Io, r1, r2, lin_obj, r1r1, r1r2, r1Lh1...

register const Index place_i = PlaceIs[ i ];

if ( place_i == Max_n ) return MsgInvalidIndex;

if ( place_i >= Ns - DegreeDependence ){
        // Index to be removed is linear dependent
        // r1, r2 do not chance...

        if ( place_i < Ns - 1 ) {
                // if not the last row...
                TQPclassPath::SwapLDMemL ( place_i, Ns-1);
                }

        TQPclassPath::FreeMemL( Ns-1 );
		FreeMemQ( i );

        TQPclassPath::MoveIndexOut_Is ( i );
        TQPclassPath::MoveIndexTo_Io ( i );
        DegreeDependence--;
        return MsgOk;
        }

//----------------------------------------------------------------------------
//  Index To Be Removed is Linear Independent...
//  Apply Givens Rotations:
//  One simple rotation for every linear independent item below i.

for ( register Index j = place_i + 1 ; j < Ns - DegreeDependence ; j++ ){

        register const hpReal Lji = L[ j ][ place_i ];

        if ( abs(Lji) < AbsTolError ) // Lij = 0 already...
                continue;

        L[ j ][ place_i ] = 0;
        register const hpReal Ljj   = L[ j ][ j ];
		register const hpReal delta = L[ j ][ j ] = sqrt( Ljj*Ljj + Lji*Lji );

        register hpReal Cos =  Ljj / delta;
        register hpReal Sin =  Lji / delta;

        // we need to apply the Given Rotations to all rows (including the linear dependent).
        for ( register Index k = j+1; k < Ns ; k++ )
                GivenRotationPostMultiplication( Cos,  Sin, L[ k ][ place_i ], L[ k ][ j ]);

        GivenRotationPostMultiplication( Cos,  Sin, r1[ place_i ], r1[ j ]);
        GivenRotationPostMultiplication( Cos,  Sin, r2[ place_i ], r2[ j ]);
        GivenRotationPostMultiplication( Cos,  Sin, z1[ place_i ], z1[ j ]);
        GivenRotationPostMultiplication( Cos,  Sin, z2[ place_i ], z2[ j ]);
        }

//-----------------------------------------------------------------------
// Need to check if the Linear Dependent items became independent...
// Store information of the column i for the linear dependent rows.
if ( DegreeDependence ){
        for ( register Index k = 0 ; k < DegreeDependence ; k++ )
                tmpVec[k] = L[ Ns - DegreeDependence + k ][ place_i ];
        }

// Removing ith row and ith column from L
// Updating other data structure: lin_obj, r1, r2, Is, Io,...
TQPclassPath::RemoveRowColumnL( place_i );

#if ( !ExactMode )
        r1r1 -= r1[ place_i ] * r1[ place_i ];
        r1r2 -= r1[ place_i ] * r2[ place_i ];
        r1z1 -= r1[ place_i ] * z1[ place_i ];
        r1z2 -= r1[ place_i ] * z2[ place_i ];
#endif

for ( register Index k = place_i ; k < Ns - 1 ; k++ ){
        PlaceIs[ Is[ k + 1 ] ]--;
        Is[ k ]         = Is[ k + 1 ];
        r1[ k ]         = r1[ k + 1 ];
        r2[ k ]         = r2[ k + 1 ];
        z1[ k ]         = z1[ k + 1 ];
        z2[ k ]         = z2[ k + 1 ];
        lin_obj[ k ]    = lin_obj[ k+1 ];
        alpha_iter[k]   = alpha_iter[ k+1];
        alpha[k]        = alpha[k+1];
        dd[k]           = dd[k+1];
        }
r1     [ Ns - 1 ]  = hpZero;
r2     [ Ns - 1 ]  = hpZero;
z1     [ Ns - 1 ]  = hpZero;
z2     [ Ns - 1 ]  = hpZero;
Is     [ Ns - 1 ]  = i;
PlaceIs[   i    ]  = Ns - 1;
FreeMemQ( i );
TQPclassPath::MoveIndexOut_Is ( i );
TQPclassPath::MoveIndexTo_Io ( i );

//-----------------------------------------------------------------------------
// Now I can add any new linear independent element that used to be dependent
const Index sizeLI = Ns - DegreeDependence;
for ( register Index k = sizeLI; k < Ns ; k++ ){

        // I should compare with the exact mode...
        // hpReal delta = tmpVec[k - sizeLI];

        register Index ind_k = Is[k];
        hpReal Qkk = GetQij(ind_k,ind_k);

        hpReal delta = hpZero;
		register const hpReal delta2 =  Qkk - norm( L[k], Ns - DegreeDependence );

		if ( delta2 < - max( AbsTolError, RelTolError*Qkk) ) {
                // Recompute from the scratch...
                CountMsgNumericProblem++;
                return MsgNumericalProb;
                }
        else if ( delta2 > max( AbsTolError, RelTolError*Qkk) ){
                // It is Linear Independent
                delta = sqrt ( delta2 );
                DegreeDependence--;
                TQPclassPath::AddRowToLI ( ind_k, k, delta );
                }
        else    {
                // It is Linear Dependent
                L[k][k] = delta;
                }
        }
//-----------------------------------------------------------------------------
return MsgOk;
}
//-----------------------------------------------------------------------
QPErrorMsg TQPclassPath::FromBaseToC ( Index i )
{
// The following informaiton is updated here:
// Q, L, Is, Ic, YcCc, r1, r2, lin_obj, cte_obj, QC_redcost, r1r1, r1r2, r1Lh1...

register const Index place_i = PlaceIs[ i ];

if ( place_i == Max_n ) return MsgInvalidIndex;

hpReal QicCc = SprodQi_d ( i, C, Ic, Nc); // i should not be in C yet...
hpReal Qii   = GetQij(i,i);
TQPclassPath::MoveIndexTo_Ic ( i, QicCc, Qii );

if ( place_i >= Ns - DegreeDependence ){
        // Index is in the Linear Dependent part of L...
        // r1 do not chance... BUT r2 DO CHANGE!!!

        // No need to backsolve...    r2 = r2 + theta * C[i] * L[place_i]
        // z2 is not affected... only z1 = z1 + C[i] * L[place_i]
        for ( register Index k = Ns - DegreeDependence ; k-- ; ){
                r2[ k ] += theta*C[i]*L[place_i][k];
                z1[ k ] += C[i]*L[place_i][k];
                }

		if ( place_i < Ns - 1 ) {
                // if it is not the last row, swap...
				TQPclassPath::SwapLDMemL ( place_i, Ns-1);
                }

        // Exactly the last row of L
        TQPclassPath::FreeMemL( Ns-1 );
        FreeMemQ( i );

        TQPclassPath::MoveIndexOut_Is ( i );
        DegreeDependence--;
        return MsgOk;
        }

//----------------------------------------------------------------------------
//  Index is in the Linear Independent part of L
//  Apply Givens Rotations:
//  One simple rotation for every linear independent item below i.
//  Store Information: (1) ith row will be used to cheaply update r2.
for ( register Index k = 0 ; k <= place_i ; k++)  tmpVec[ Max_n + k ] = L[ place_i ][ k ];
for ( register Index k = place_i+1 ; k < Ns - DegreeDependence ; k++)  tmpVec[ Max_n + k ] = hpZero;
for ( register Index j = place_i + 1 ; j < Ns - DegreeDependence ; j++ ){

        register hpReal Lji = L[ j ][ place_i ];

        if ( abs(Lji) < AbsTolError ) // Lij = 0 already...
                continue;
        L[ j ][ place_i ] = 0;

        register hpReal Ljj = L[ j ][ j ];
        register const hpReal delta = L[ j ][ j ] = sqrt( Ljj*Ljj + Lji*Lji );

        register hpReal Cos =  Ljj / delta;
        register hpReal Sin =  Lji / delta;

        // we need to apply the Given Rotations to all rows (including the linear dependent).
        // Remember: Only the place_i and j columns are changed...
        for ( register Index k = j+1; k < Ns ; k++ )
                GivenRotationPostMultiplication( Cos,  Sin, L[ k ][ place_i ], L[ k ][ j ]);
        // Storing Information (1):
        tmpVec[Max_n + j] = Sin*tmpVec[Max_n + place_i ];
        tmpVec[Max_n + place_i ] *= Cos;

        GivenRotationPostMultiplication( Cos,  Sin, r1[ place_i ], r1[ j ]);
        GivenRotationPostMultiplication( Cos,  Sin, r2[ place_i ], r2[ j ]);
        GivenRotationPostMultiplication( Cos,  Sin, z1[ place_i ], z1[ j ]);
		GivenRotationPostMultiplication( Cos,  Sin, z2[ place_i ], z2[ j ]);
        }

//-----------------------------------------------------------------------
// Store information: (2) elements of ith column of the linear dependent rows.

if ( DegreeDependence ){
        for ( register Index k = 0 ; k < DegreeDependence ; k++ )
                tmpVec[k] = L[ Ns - DegreeDependence + k ][ place_i ];
        }

// Removing ith row and ith column from L
// Updating other data structure: lin_obj, r1, r2, Is, Io,...
TQPclassPath::RemoveRowColumnL( place_i );
// After the Given Rotations Correction, G r2, we need to adjust for extra term in Ic...
// Divine Intervention... the row deleted from the matrix L is exactly what we needed...
WSumVectors( r2, 1.0e0, r2, theta*C[i], tmpVec + Max_n, Ns - DegreeDependence );
WSumVectors( z1, 1.0e0, z1, C[i], tmpVec + Max_n, Ns - DegreeDependence );

#if ( !ExactMode )
        r1r1 -= r1[ place_i ] * r1[ place_i ];
        r1r2 -= r1[ place_i ] * r2[ place_i ];
        r1z1 -= r1[ place_i ] * z1[ place_i ];
        r1z2 -= r1[ place_i ] * z2[ place_i ];
#endif

for ( register Index k = place_i ; k < Ns - 1 ; k++ ){
        Is[ k ] = Is[ k + 1 ];
        PlaceIs[ Is[ k + 1 ] ]--;
        r1[ k ] = r1[ k + 1 ];
        r2[ k ] = r2[ k + 1 ];
        z1[ k ] = z1[ k + 1 ];
        z2[ k ] = z2[ k + 1 ];
        lin_obj[ k ] = lin_obj[ k+1 ];
        alpha_iter[ k ] = alpha_iter[ k+1];
        alpha[ k ]      = alpha[ k + 1 ];
        dd[ k ]         = dd [ k + 1] ;
        }
r1     [ Ns - 1 ]  = hpZero;
r2     [ Ns - 1 ]  = hpZero;
z1     [ Ns - 1 ]  = hpZero;
z2     [ Ns - 1 ]  = hpZero;
Is[ Ns - 1 ] = i;
PlaceIs[ i ] = Ns-1;
TQPclassPath::MoveIndexOut_Is ( i );
FreeMemQ( i );

//-----------------------------------------------------------------------------
// Need to check if the Linear Dependent items became independent...
// Now I can add any new linear independent element that used to be dependent
const Index sizeLI = Ns - DegreeDependence;
for ( register Index k = sizeLI; k < Ns ; k++ ){

        // I should compare with the exact mode...
        // hpReal delta = tmpVec[k - sizeLI];

        register Index ind_k = Is[k];
        hpReal Qkk = GetQij(ind_k,ind_k);

        hpReal delta = hpZero;
        register const hpReal delta2 =  Qkk - norm( L[k], Ns - DegreeDependence );

        if ( delta2 < - max( AbsTolError, RelTolError*Qkk) ) {
                // Recompute from the scratch...
                CountMsgNumericProblem++;
                return MsgNumericalProb;
                }
        else if ( delta2 > max( AbsTolError, RelTolError*Qkk) ){
                // Linear Independent
                delta = sqrt ( delta2 );
                DegreeDependence--;
                TQPclassPath::AddRowToLI ( ind_k, k, delta );
                }
        else    {
                // Linear Dependent
                L[k][k] = delta;
                }
        }
//-----------------------------------------------------------------------------
return MsgOk;
}
//-----------------------------------------------------------------------
QPErrorMsg TQPclassPath::FromCToBase ( Index i )
{
// The following informaiton is updated here:
// Q, L, Is, Ic, r1, r2, lin_obj, cte_obj, YcCc, r1r1, r1r2, r1Lh1...

// Allocate Memory if necessary... (we might already have i stored)
TQPclassPath::AllocateL( i );

AddColumnToQ( i );

//----------------------------------------------------------------------------
// First: Remove i from Ic (update all related data) and put i on Is

hpReal QicCc    = SprodQi_d ( i, C, Ic, Nc);
hpReal Qii      = GetQij(i,i);

QicCc -= Qii*C[i];
TQPclassPath::MoveIndexOut_Ic ( i, QicCc, Qii );
TQPclassPath::MoveIndexTo_Is  ( i, theta*QicCc - V[i], theta*C[i] );

if ( Qii < hpZero ){
        PrintErrorMsg("Recomputing All: FATAL ERROR... Qii < 0\n");
        return MsgFatal;
        }
//----------------------------------------------------------------------------
// The matrix L is "empty"...
if ( Ns == 1 ) {
        register const hpReal delta = sqrt( Qii );
        L[ 0 ][ 0 ] = delta;
        r1[ 0 ] = Y[ i ] / delta;
        r2[ 0 ] = lin_obj[0] / delta;
        z1[ 0 ] = QicCc / delta;
        z2[ 0 ] = V[i] / delta;

        #if ( !ExactMode )
                r1r1    = r1[ 0 ]*r1[ 0 ];
                r1r2    = r1[ 0 ]*r2[ 0 ];
                r1z1    = r1[ 0 ]*z1[ 0 ];
                r1z2    = r1[ 0 ]*z2[ 0 ];
        #endif

        return MsgOk;
        }

// tmpVec stores Qi[Is]...
for ( register Index k = Ns - 1 - DegreeDependence ; k-- ; )  tmpVec[k] = GetQij( i , Is[k] );
TriangularSystem( L, tmpVec , L[ Ns - 1 ], Ns - 1 - DegreeDependence );

//-----------------------------------------------------------------------------
// r2 must be updated. Note that there is no need to back solve...
// Divine Intervention... r2_new = r2_old - C[i] L[Ns]
WSumVectors( r2, 1.0e0, r2, -theta*C[i], L[Ns-1], Ns - 1 - DegreeDependence);
WSumVectors( z1, 1.0e0, z1, -C[i], L[Ns-1], Ns - 1 - DegreeDependence);

// POTENTIAL TO SAVE... sprod(r2,L[Ns],Ns-DegreeDependence) is used other times...
#if ( !ExactMode )
		hpReal r1L = sprod( r1, L[Ns-1], Ns - 1 - DegreeDependence);
        r1r2 -= theta*C[i]*r1L;
		r1z1 -= C[i]*r1L;
#endif


// New component of r2 is added on AddRowToLI if called
// First need to test if it is Linear Independent

register const hpReal delta2 =  Qii - norm( L[Ns-1], Ns - 1 - DegreeDependence );
if ( delta2 < - max(AbsTolError, RelTolError*Qii) ){
        // Recompute from the Scratch...
        CountMsgNumericProblem++;
        return MsgNumericalProb;
        }
else if ( delta2 > max(AbsTolError, RelTolError*Qii) ){
        TQPclassPath::AddRowToLI ( i, Ns - 1, sqrt( delta2 ) );
        }
else    {
        // Linear Dependent
        TQPclassPath::AddRowToLD ( i, Ns - 1 );

		}
return MsgOk;
}


//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
QPErrorMsg TQPclassPath::FromZeroToBase ( Index i )
{
// The following informaiton is updated here:
// Q, L, Is, Io, r1, r2, lin_obj, r1r1, r1r2, r1Lh1...

//----------------------------------------------------------------------------
// Allocate Memory
TQPclassPath::AllocateL( i );

AddColumnToQ( i );

//----------------------------------------------------------------------------
// First remove i from Io and put i on Is
hpReal QicCc = SprodQi_d( i, C , Ic, Nc);
hpReal const Qii   = GetQij(i,i);

TQPclassPath::MoveIndexOut_Io ( i );
TQPclassPath::MoveIndexTo_Is ( i , theta*QicCc - V[i], hpZero );

if ( Qii < hpZero ){
        PrintErrorMsg("Recomputing All: FATAL ERROR... Qii < 0\n");
        return MsgFatal;
        }

//----------------------------------------------------------------------------
// The matrix is "empty"... but we still need to look to Ic...
if ( Ns == 1 ) {
        register hpReal delta = sqrt( Qii );
        L[ 0 ][ 0 ] = delta;
        r1[ 0 ] = Y[ i ] / delta;
        r2[ 0 ] = lin_obj[0] / delta;
        z1[ 0 ] = QicCc / delta;
        z2[ 0 ] = V[ i ] / delta;
        #if ( !ExactMode )
                r1r1 = r1[ 0 ]*r1[ 0 ];
                r1r2 = r1[ 0 ]*r2[ 0 ];
                r1z1 = r1[ 0 ]*z1[ 0 ];
                r1z2 = r1[ 0 ]*z2[ 0 ];
        #endif

        return MsgOk;
        }

// tmpVec stores Qi[Is]...
for ( register Index k = Ns - 1 - DegreeDependence ; k-- ; ) tmpVec[k] = GetQij( i , Is[k] );
TriangularSystem( L, tmpVec , L[ Ns - 1 ], Ns - 1 - DegreeDependence );

register const hpReal delta2 = Qii - norm( L[Ns - 1], Ns - 1 - DegreeDependence );
if ( delta2 < - max( AbsTolError, RelTolError * Qii) ){
        // Recompute from the scratch...
        CountMsgNumericProblem++;
        return MsgNumericalProb;
        }
else if ( delta2 > max( AbsTolError, RelTolError * Qii) ){
        // Linear Independent
        TQPclassPath::AddRowToLI ( i, Ns-1, sqrt( delta2 ) );
        }
else    {
        // Linear Dependent
        AddRowToLD ( i, Ns-1);
        }


return MsgOk;
}
//-----------------------------------------------------------------------
QPErrorMsg TQPclassPath::MoveAlongDirection ()
{

QPErrorMsg CurrentStatus = MsgOk;
#if ( TimeMode )
        TimeStart();
#endif

// Recall that:
// LocalOptima = FALSE (found Infinite Direction)
// LocalOptima = TRUE  (Unique Solution for the KKT)
if   ( LocalOptima )    StepSize    = 1.0e0;
else                    StepSize    = hpInf;
BaseEvent    = NoEvent;
BindingIndex = GetMax_n();

// Min-Ratio Test
// StepSize = min { StepSize,  min { alpha_iter_i / (-dd_i ) : dd_i < 0 }  ,  min { [ theta*c_i - alpha_iter_i ] / dd_i : dd_i > 0 }
for ( register Index k = Ns ; k-- ; ){
        if      ( dd[k] < -AbsTolError && StepSize > alpha_iter[k] / ( - dd[k] ) ){
                StepSize     = alpha_iter[k] / ( - dd[k] );
                BindingIndex = Is[k];
                LocalOptima  = FALSE;
                BaseEvent    = Is2Io;
                }
        else if ( dd[k] > AbsTolError && StepSize > (theta*C[ Is[k] ]-alpha_iter[k])/dd[k] ) {
                StepSize     = (theta*C[ Is[k] ]-alpha_iter[k])/dd[k];
                BindingIndex = Is[k];
                LocalOptima  = FALSE;
                BaseEvent    = Is2Ic;
                }
        }

// Update alpha_iter = alpha + StepSize * dd
if ( LocalOptima ) {
        AssignVectors( alpha_iter, alpha, Ns);
        b_iter = b;
        }
else    {
		WSumVectors ( alpha_iter, 1.0e0, alpha_iter, StepSize, dd, Ns );
        b_iter += StepSize*(b-b_iter);
        }

// if LocalOptima = FALSE (or BaseEvent != NoEvent),
// we need to fix variables which implies in update structure.
switch ( BaseEvent ){
		case Is2Ic:
                CurrentStatus = TQPclassPath::FromBaseToC ( BindingIndex );
                if ( !Ns )      LocalOptima = TRUE;
                break;

        case Is2Io:
                CurrentStatus = TQPclassPath::FromBaseToZero ( BindingIndex );
                if ( !Ns )      LocalOptima = TRUE;
                break;

        case NoEvent:
                break;
}

#if ( TimeMode )
        TimeFinish();
        TimeAdd( &TimeRemoveFromBase );
#endif

return CurrentStatus;
}

//-----------------------------------------------------------------------------

void      TQPclassPath::ComputeObjFuncFast()
{

f_obj = cte_obj - (0.5) * b_iter * theta * YcCc + 0.5 * sprod( lin_obj, alpha_iter, Ns );

}

//-----------------------------------------------------------------------------
// Note that: (1)   place_i >= Ns - 2 - DegreeDependence  = Size of current LI block
//            (2)   place_i =  Ns if adding variable from Ic to Io.
//            (3)   place_i <= Ns
// Also Updates r1 and r2.
//
//    Ls     0    0      Ns rows
//    Ld1    0    0      d1
//     v   delta  0      1
//    Ld2    0    0      d2      d1 + d2  = DegreeDependence
//
//    First index after Ls is (Ns - DegreeDependence)
//    Must be called with Ns rows on L, from 0..Ns-1
void TQPclassPath::AddRowToLI ( Index i, Index place_i, hpReal delta )
{
register Index PlaceInsert = Ns - 1 - DegreeDependence;
if (  place_i > PlaceInsert ){
        // New row is linear independent: insert before dependent rows
        // Need to swap lines to keep L.I. lines together.
        // recall: Ns-DegreeDependence is the index of the first linear dependent item
        Index who_PlaceInsert = Is[ PlaceInsert ];
        SwapVectors( L[place_i], L[ PlaceInsert ], PlaceInsert );

        // Update Indices
        swap (Is[ place_i ], Is[ PlaceInsert ]);
        swap (PlaceIs[ i ], PlaceIs[ who_PlaceInsert ]);
        swap (lin_obj[ place_i ], lin_obj[ PlaceInsert ]);
        swap (alpha_iter[ place_i ], alpha_iter[ PlaceInsert ]);
        }
// From now on place_i == Ns - DegreeDependence
// Here the new row is already just below the LI part,
// that is PlaceIs[ i ] = Ns - DegreeDependence
// Need to update r1, r2.
register const hpReal r1Add = ( Y[i] - sprod( r1,L[ PlaceInsert ], PlaceInsert) )/ delta;
register const hpReal z1Add = ( GetQicCc( Is[ PlaceInsert ] ) - sprod( z1,L[PlaceInsert], PlaceInsert) )/ delta;
register const hpReal z2Add = ( V[ Is[ PlaceInsert ] ] - sprod( z2,L[PlaceInsert], PlaceInsert) )/ delta;
register const hpReal r2Add = theta*z1Add - z2Add;

r1[ PlaceInsert ] = r1Add;
r2[ PlaceInsert ] = r2Add;
z1[ PlaceInsert ] = z1Add;
z2[ PlaceInsert ] = z2Add;

#if ( !ExactMode )
        r1r1 += r1Add*r1Add;
        r1r2 += r1Add*r2Add;
        r1z1 += r1Add*z1Add;
        r1z2 += r1Add*z2Add;
#endif

// Need to add a new column to the linear dependent rows
for ( register Index k = DegreeDependence ; k-- ; )
        L[Ns - 1 - k ][ PlaceInsert ] =
                ( GetQij( i, Is[ Ns - 1 - k ]) - sprod( L[ PlaceInsert ], L[Ns-1-k], PlaceInsert) )/ delta;

L[PlaceInsert][PlaceInsert] = delta;

// Updates of LD1r1, LD1r2, LD2r1, LD2r2
//........
}

//----------------------------------------------------------------
// DegreeDependence is incremented, and Diagonal is set to zero.
//
void TQPclassPath::AddRowToLD ( Index i, Index place_i )
{
L[ place_i ][ place_i ] = hpZero;

DegreeDependence++;

// Updates of LD1r1, LD1r2, LD2r1, LD2r2
//........
}
//--------------------------------------------------------------------
// D E S T R U C T O R -----
//--------------------------------------------------------------------

TQPclassPath::~TQPclassPath()
{
if ( Is )       delete( Is );
if ( Ic )       delete( Ic );
if ( Io )       delete( Io );
if ( PlaceIs )  delete( PlaceIs );
if ( PlaceIc )  delete( PlaceIc );
if ( PlaceIo )  delete( PlaceIo );
if ( Y )        delete( Y );
if ( C )        delete( C );
if ( V )        delete( V );
if ( r1 )       delete( r1 );
if ( r2 )       delete( r2 );
if ( lin_obj )  delete( lin_obj );
if ( alpha )    delete( alpha );
if ( alpha_iter ) delete( alpha_iter );
if ( dd )       delete( dd );
if ( tmpVec )   delete( tmpVec );

TQPclassPath::DeleteL();

if ( z1 )       delete( z1 );
if ( z2 )       delete( z2 );

if ( PrintMode >= 1 )
		fclose(out_info);


}

//-----------------------------------------------------------------------------
// Memory Allocation...
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
QPErrorMsg      TQPclassPath::AllocateL( Index i )
{

if ( IntNumber( Ns ) >= CurrentAllocL ){
        for ( register Index k = BlockAllocL ; k-- ; ){

                #if ( L_LowerTri )
                memalloc( L[ CurrentAllocL ], hpReal, CurrentAllocL + 1 ); // Lower triangular matrix
                VecInit ( L[ CurrentAllocL ], hpZero, CurrentAllocL + 1 );
                CurrentAllocL++;
                #else
                memalloc( L[ CurrentAllocL ], hpReal, Max_n ); // Lower triangular matrix
                VecInit ( L[ CurrentAllocL ], hpZero, Max_n );
                CurrentAllocL++;

                #endif

                }

        }
return MsgOk;
}

//-----------------------------------------------------------------------------
QPErrorMsg      TQPclassPath::SwapLDMemL ( Index from_i, Index to_i)
{
// Remember, both lines are Linear Dependent (LD)
// at most Ns - DegreeDependence nonzero elements on each line.

#if ( L_LowerTri )
// If L is lower triangular we cannot simply change pointers...
        SwapVectors( L[from_i], L[to_i], Ns-DegreeDependence );

#else
// L was allocated as a full matrix, simply change pointers...
        hpVec AuxVec = L[from_i];
        L[from_i] = L[to_i];
        L[to_i] = AuxVec;
#endif
return MsgOk;
}

//-----------------------------------------------------------------------------
QPErrorMsg      TQPclassPath::FreeMemL( Index Is_index )
{
// We could deallocate memory of L if it is critical...
for ( register Index k = Is_index+1 ; k-- ;)
        L[Is_index][k] = 0.0e0;

return MsgOk;
}

//-----------------------------------------------------------------------------
QPErrorMsg      TQPclassPath::RemoveRowColumnL( Index i )
{
#if (L_LowerTri)

for ( register Index k = i+1; k < Ns ; k++ )
        ShiftLeft ( L[k], i, k );
for ( register Index k = i; k < Ns - 1 ; k++ )
        AssignVectors( L[k], L[k+1], k + 1);
VecInit ( L[Ns-1], 0.0e0, Ns );

#else

hpVec auxtmp = L[i];
for ( register Index k = i ; k < Ns - 1 ; k++ )
        L[ k ] = L[ k + 1 ];
L[Ns - 1] = auxtmp;
VecInit ( L[Ns-1], 0.0e0, Ns );

for ( register Index k = i; k < Ns - 1 ; k++ )
        ShiftLeft ( L[k], i, k+1 );

#endif


return MsgOk;
}

//-----------------------------------------------------------------------------
void            TQPclassPath::DeleteL()
{

for( register Index k = CurrentAllocL ; k-- ; )
        delete [] L[ k ];
delete [] L;
}


//-----------------------------------------------------------------------------
// Printing Info
//-----------------------------------------------------------------------------

void            TQPclassPath::PrintBasicInfo ()
{
if ( PrintMode ){
		fprintf( out_info," Iteration: %d ------------------ \n", Iteration);
		fprintf( out_info,"Number of Invalid Index to be Added: %d\n",CountMsgInvalidIndex);
		fprintf( out_info,"Number of Recomputations Performed (due to numerical problems): %d\n", CountMsgNumericProblem);
		fprintf( out_info,"Memory Style for L: %d, CurrentAllocL = %d, Size of Block Allocation: %d \n", L_LowerTri, CurrentAllocL, BlockAllocL);
		fprintf( out_info,"Ns = %d, No = %d, Nc = %d, DegreeDependence = %d, LocalOpitima = %d \n",Ns,No,Nc, DegreeDependence, LocalOptima);
		fprintf( out_info,"Inner Loops: %d\n", InnerIter);
		fprintf( out_info,"Objective Function: %f   and  Multiplier b = %f, Step Size = %f  \n", f_obj, b, StepSize);
		fprintf( out_info,"NewIndex = %d, ReducedCost = %f, Last Event = %d\n", NewIndexToBeAdded, ReducedCostMin, BaseEvent );

		#if ( TimeMode )

		fprintf( out_info,"Time:\n");
		TimeTotal();
		if ( TimeTotalAll > AbsTolError ){
				fprintf( out_info,"KKT:%.2f Add: %.2f Remove: %.2f RC: %.2f Init:%.2f Total: %f\n",
						  TimeKKTsystem/TimeTotalAll, TimeAddToBase/TimeTotalAll,
						  TimeRemoveFromBase/TimeTotalAll, TimeReduceCost/TimeTotalAll,
						  TimeInit/TimeTotalAll, TimeTotalAll );
				}
		else    {
				fprintf( out_info,"Total Time = 0\n");
				}

		#endif
		fflush(out_info);
		}
}

//-----------------------------------------------------------------------------
void            TQPclassPath::PrintErrorMsg ( char *s )
{
if ( PrintMode ){
		fprintf( out_info,"**** QPclassPath Message ***** :");
		if ( PrintMode > 0 ) fprintf( out_info,"Iteration: %d, InnerIter: %d, Ns = %d, Nc = %d, No = %d \n",
						Iteration, InnerIter, Ns, Nc, No);
		fprintf( out_info,"%s\n",s);
		}

#if ( PrintVisual == 0 )
	printf ( "%s\n",s );
#endif
}
//-----------------------------------------------------------------------------
void            TQPclassPath::PrintInformativeMsg ( char *s )
{
if ( PrintMode ){
		fprintf( out_info,"**** QPclassPath Informative Message  *****\n");
		fprintf( out_info,"Iteration: %d, InnerIter: %d, Ns = %d, Nc = %d, No = %d \n",
						Iteration, InnerIter, Ns, Nc, No);
		fprintf( out_info,"%s\n",s);
		}
#if ( PrintVisual == 0 )
	printf ( "%s\n",s );
#endif
}

//-----------------------------------------------------------------------------
hpReal          TQPclassPath::ReadOptimalValue()
{
hpReal t = hpZero;
for ( register Index k = Ns ; k-- ; )
        t+= 0.5*alpha_iter[k]*SprodQi_dIset ( Is[k], alpha_iter, Is, Ns);

t+= sprod(lin_obj,alpha_iter, Ns);

t+= cte_obj;

return t;
}

//-----------------------------------------------------------------------------
// Error Analysis... and Recomputation...
//-----------------------------------------------------------------------------
QPErrorMsg TQPclassPath::VerifyQPdata ()
{
hpReal NormError;
QPErrorMsg  DebugStatus = MsgOk;

// For r1....
TriangularSystem( L, Y , tmpVec, Ns - DegreeDependence, Is );
DiffVectors( tmpVec, tmpVec, r1, Ns );
NormError = norm( tmpVec, Ns - DegreeDependence );
if ( NormError > AbsTolError ){
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg ("r1 is different from backsolve: r1 = L-1 Y.\n");
        }

for ( register Index k = 0 ; k < Ns - DegreeDependence ; k++ ){
        tmpVec[k] = sprod(L[k],r1,k+1);
        tmpVec[k+Max_n] = Y[Is[k]];
        }
DiffVectors( tmpVec, tmpVec, tmpVec+Max_n, Ns - DegreeDependence);
NormError = norm( tmpVec, Ns - DegreeDependence );
if ( NormError > AbsTolError ){
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg ("r1 does not solve L r1 = Y.\n");
        }



// For lin_obj...
for (register Index k = 0 ; k < Ns ; k++ ){
        tmpVec[k]       = theta*SprodQi_d ( Is[k], C, Ic, Nc);
        tmpVec[Max_n+k] = V[Is[k]];
        }
DiffVectors( tmpVec + Max_n, tmpVec, tmpVec + Max_n, Ns  );
DiffVectors( tmpVec, tmpVec+ Max_n, lin_obj, Ns );
NormError = norm( tmpVec, Ns );
if ( NormError > AbsTolError ){
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg ("lin_obj is wrong...\n");
        }

// For r2...
TriangularSystem( L, tmpVec + Max_n, tmpVec, Ns - DegreeDependence );
DiffVectors( tmpVec, tmpVec, r2, Ns - DegreeDependence );
NormError = norm( tmpVec, Ns );
if ( NormError > AbsTolError ){
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg ("r2 is different from backsolve of current L.\n");
        }
for ( register Index k = 0 ; k < Ns - DegreeDependence ; k++ ){
        tmpVec[k] = sprod( L[k], r2, k+1);
        tmpVec[k+Max_n] = theta*SprodQi_d ( Is[k], C, Ic, Nc) - V[Is[k]];
        }
DiffVectors( tmpVec, tmpVec, tmpVec+Max_n, Ns - DegreeDependence);
NormError = norm( tmpVec, Ns - DegreeDependence );
if ( NormError > AbsTolError ){
        DebugStatus = MsgQPIncorrect;
		PrintErrorMsg ("r2 does not solve L r2 = (QscCc - Vs).\n");
        }

// Verifying cte_obj
tmpVec[0] = - sprod( C , V, Ic, Nc );
if ( abs( tmpVec[0] - lin_cte_obj ) > max(AbsTolError, RelTolError*abs(tmpVec[0])) ){
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg ("lin_cte_obj is not consistent with Ic.\n");
		}
tmpVec[1] = 0.0e0;
for ( register Index i = 0 ; i < Nc ; i++ )
		tmpVec[1] += 0.5*C[Ic[i]]*SprodQi_d ( Ic[i], C, Ic, Nc );
if ( abs( tmpVec[1] - quad_cte_obj ) > max(AbsTolError, RelTolError*abs(tmpVec[1])) ){
		DebugStatus = MsgQPIncorrect;
        PrintErrorMsg ("quad_cte_obj is not consistent with Ic.\n");
        }

NormError = abs( theta*tmpVec[0]+theta*theta*tmpVec[1] - cte_obj );
if ( NormError > max(AbsTolError, RelTolError*abs(cte_obj)) ){
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg ("cte_obj is not consistent with Ic.\n");
        }
// Verifying YcCc
tmpVec[0] = sprod( C , Y, Ic, Nc );
NormError = abs( tmpVec[0] - YcCc );
if ( NormError > AbsTolError ){
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg ("YcCc is not consistent with Ic\n");
        }

// Sets data...
Index Count = 0;
// For Is...
for ( register Index k = 0 ; k < Max_n ; k++ )
        if ( PlaceIs[ k ] < Max_n ) {
                Count++;
                if ( Is[ PlaceIs[ k ] ] != k ){
                        DebugStatus = MsgQPIncorrect;
                        PrintErrorMsg ("Is and PlaceIs are not consistent:");
                        fprintf( out_info," see %d.\n",k);
                        }
                }
if ( Count != Ns ) {
		DebugStatus = MsgQPIncorrect;
        PrintErrorMsg("More that Ns elements with PlaceIs < Max_n.\n");
		}

// For Ic...
Count = 0;
for ( register Index k = 0 ; k < Max_n ; k++ )
        if ( PlaceIc[ k ] < Max_n ) {
                Count++;
                if ( Ic[ PlaceIc[ k ] ] != k ){
                        DebugStatus = MsgQPIncorrect;
                        PrintErrorMsg("Ic and PlaceIc are not consistent:");
                        fprintf( out_info," see %d.\n",k);
                        }
                }
if ( Count != Nc ) {
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg("More that Nc elements with PlaceIc < Max_n.\n");
        }

// For Io...
Count = 0;
for ( register Index k = 0 ; k < Max_n ; k++ )
        if ( PlaceIo[ k ] < n ) {
                Count++;
                if ( Io[ PlaceIo[ k ] ] != k ){
                        DebugStatus = MsgQPIncorrect;
                        PrintErrorMsg ("Io and PlaceIo are not consistent:");
                        fprintf( out_info," see %d.\n",k);
                        }
                }
if ( Count != No ) {
        DebugStatus = MsgQPIncorrect;
        PrintErrorMsg("More that No elements with PlaceIo < Max_n.\n");
        }

// QC_redcost...
for ( register Index k = Ns ; k-- ; )
        if ( abs( GetQicCc( Is[k] ) - SprodQi_d ( Is[k], C, Ic, Nc) ) > AbsTolError ){
                DebugStatus = MsgQPIncorrect;
                PrintErrorMsg("QC_redcost is not consitent: ");
                fprintf(out_info,"see %d\n",Is[k]);
                }
for ( register Index k = No ; k-- ; )
        if ( abs( GetQicCc( Io[k] ) - SprodQi_d ( Io[k], C, Ic, Nc) ) > AbsTolError ){
                DebugStatus = MsgQPIncorrect;
				PrintErrorMsg("QC_redcost is not consitent: ");
                fprintf(out_info,"see %d\n",Io[k]);
				}
for ( register Index k = Nc ; k-- ; )
        if ( abs( GetQicCc( Ic[k] ) - SprodQi_d ( Ic[k], C, Ic, Nc) ) > AbsTolError ){
                DebugStatus = MsgQPIncorrect;
                PrintErrorMsg("QC_redcost is not consitent: ");
                fprintf(out_info,"see %d\n",Ic[k]);
                }


        DebugStatus = VerifyQPMem ();
#if ( DebugMode  )


        if ( DebugStatus != MsgOk ){
                PrintErrorMsg("QP Mem with inconsistency.\n");
                }
        // Verifying the Matrix L itself...
        for ( register Index i = 0 ; i < Ns ; i++ )
                for ( register Index j = 0 ; j <= i ; j++ ){
                        hpReal t=hpZero;
                        for ( register Index k = 0 ; k <= min(i,j) ; k++ )
                                t += L[i][k]*L[j][k]; // k<= min(i,j)
                        if ( abs( t - GetQij(Is[i],Is[j]) ) > AbsTolError ){
                                DebugStatus = MsgQPIncorrect;
                                PrintErrorMsg("L is not consitent: ");
                                fprintf( out_info,"see entry (%d,%d).\n",i,j);
                                }
                        }

        // Veryfying Q itself... (our representation...)
        for ( register Index i = 0 ; i < Ns ; i++ )
                for ( register Index j = 0 ; j <= i ; j++ ){
                        if (abs( GetQij(Is[i],Is[j]) - GetQij(Is[j],Is[i]) ) > AbsTolError ){
                                DebugStatus = MsgQPIncorrect;
                                PrintErrorMsg("QP mem is not symmetric: ");
                                fprintf( out_info,"see entry (%d,%d).\n",Is[i],Is[j]);
                                }
                        if (abs( GetQij(Is[i],Is[j]) - GetDataQij(Is[i],Is[j]) ) > AbsTolError ){
                                DebugStatus = MsgQPIncorrect;
                                PrintErrorMsg("QP mem differs from Data: ");
                                fprintf( out_info,"see entry (%d,%d).\n",Is[i],Is[j]);                                }
                                }
#endif

return DebugStatus;
}
//----------------------------------------------------------------------------
hpReal  TQPclassPath::VerifyMinReducedCost ()
{
hpReal RCmin            = hpZero;
hpReal RC               = hpZero;
hpReal NormError        = hpZero;
//... First check if alpha_iter is feasible...
for ( register Index k = 0; k < Ns ; k++ ){
        if ( alpha_iter[k] < -AbsTolError )
                PrintErrorMsg("Current Point has at least one Negative Component < - tol\n");
        if ( alpha_iter[k] > theta*C[Is[k]] + AbsTolError )
                PrintErrorMsg("Current Point has at least one Component Alpha[k] > C[k] + tol \n");
        NormError += Y[Is[k]]*alpha_iter[k];
        }
for ( register Index k = 0; k < Nc ; k++ ){
        NormError += Y[Ic[k]]*theta*C[Ic[k]];
        }
if ( abs(NormError) > AbsTolError )
        PrintErrorMsg("\nCurrent Point the Violates Linear Constraint \n");

for ( register Index k = 0; k < No ; k++ ){
        RC  = hpZero;
        RC += SprodQi_dIset( Io[k], alpha_iter, Is, Ns );
        RC += theta*SprodQi_dIset( Io[k], C, Ic, Nc );
        RC -= V[Io[k]];
        RC -= b*Y[Io[k]];
        if ( RC < RCmin ) RCmin = RC;
        }

for ( register Index k = 0; k < Nc ; k++ ){
        RC  = hpZero;
        RC -= SprodQi_dIset( Ic[k], alpha_iter, Is, Ns );
        RC -= theta*SprodQi_dIset( Ic[k], C, Ic, Nc );
        RC += V[Ic[k]];
        RC += b*Y[Ic[k]];
        if ( RC < RCmin ) RCmin = RC;
        }

Index Count = 0;
for ( Index k = 0 ; k < n ; k++ ){
        if      ( PlaceIs[ k ] < Max_n ){
                Count++;
                }
        else if ( PlaceIo[ k ] < Max_n ){
				Count++;
                }
		else if ( PlaceIc[ k ] < Max_n ){
                Count++;
                }
        }
if ( Count != n ){
        PrintErrorMsg("Missing Index...");
        }

return RCmin;
}
//----------------------------------------------------------------------------
// Routine To Deal with Numeric Problems...
// Recompute: L, r1, r2, DegreeDependence, lin_obj, YcCc, cte_obj
// Keep current alpha_iter...
QPErrorMsg TQPclassPath::RecomputeAll ()
{
// L and DegreeDependence
L[ 0 ][ 0 ] = sqrt( GetQij( Is[0], Is[0] ) );
DegreeDependence = 0;
for ( register Index place_i = 1 ; place_i < Ns ; place_i++ ){
        Index i = Is[place_i];
        for ( register Index k = place_i - DegreeDependence ; k-- ; ) tmpVec[k] = GetQij( i , Is[k] );
        TriangularSystem( L, tmpVec , L[ place_i ], place_i - DegreeDependence );
        hpReal const Qii = GetQij(i,i);
        register const hpReal delta2 = Qii - norm( L[place_i], place_i - DegreeDependence );
        register hpReal delta = hpZero;

        if ( delta2 < - max( AbsTolError, RelTolError * Qii) ){ // Numerical Errors...
                // Fatal Numerical Problems...
                // Recompute from the scratch...
                PrintErrorMsg("Recomputing All: FATAL ERROR... delta2 < - tol\n");
                return MsgFatal;

                }
        else if ( delta2 > max( AbsTolError, RelTolError * Qii) ){

                delta = sqrt( delta2 );

                if ( DegreeDependence ){

                        SwapVectors( L[place_i], L[place_i - DegreeDependence ], place_i - DegreeDependence );
                        // Update Indices
                        swap (Is[ place_i ], Is[place_i - DegreeDependence ]);
                        swap (PlaceIs[ i ], PlaceIs[ Is[place_i - DegreeDependence ] ]);
						swap (alpha_iter[ place_i ], alpha_iter[ place_i - DegreeDependence ]);

						// Need to add a new column to the linear dependent rows
                        for ( register Index k = DegreeDependence ; k-- ; )
                                L[place_i - k ][place_i - DegreeDependence ] =
                                        ( GetQij( i, Is[ place_i - k ]) - sprod( L[place_i - DegreeDependence ], L[place_i-k], place_i-DegreeDependence) )/ delta;

                        }
                L[place_i - DegreeDependence][place_i - DegreeDependence] = delta;
                }
        else    {
                DegreeDependence++;
                L[place_i][place_i] = delta;
                }

        }

// r1...
TriangularSystem( L, Y , r1, Ns - DegreeDependence, Is );
for ( register Index k = Ns - DegreeDependence ; k < Ns ; k++ ) r1[k] = hpZero;

// z1...
for (register Index k = 0 ; k < Ns ; k++ )
        tmpVec[k] = SprodQi_d ( Is[k], C, Ic, Nc);
TriangularSystem( L, tmpVec, z1, Ns - DegreeDependence );
for ( register Index k = Ns - DegreeDependence ; k < Ns ; k++ ) z1[k] = hpZero;

// z2...
for (register Index k = 0 ; k < Ns ; k++ )
        tmpVec[Max_n + k] = V[Is[k]];
TriangularSystem( L, tmpVec + Max_n, z2, Ns - DegreeDependence );
for ( register Index k = Ns - DegreeDependence ; k < Ns ; k++ ) z2[k] = hpZero;

// lin_obj...
WSumVectors( lin_obj, theta, tmpVec, -1.0e0, tmpVec + Max_n, Ns );

// r2...
WSumVectors( r2, theta, z1, -1.0e0, z2, Ns - DegreeDependence );
for ( register Index k = Ns - DegreeDependence ; k < Ns ; k++ ) r2[k] = hpZero;

// <Yc,Cc>...
YcCc = sprod( C , Y, Ic, Nc );

// cte_obj...
lin_cte_obj = - sprod( C , V, Ic, Nc );
quad_cte_obj = hpZero;
for ( register Index i = 0 ; i < Nc ; i++ )
        quad_cte_obj += 0.5*C[Ic[i]]*SprodQi_d ( Ic[i], C, Ic, Nc );
cte_obj = theta*lin_cte_obj + theta*theta*quad_cte_obj;

return MsgOk;
}




//---------------------------------
//---------------------------------
/*OLD...

/*
// recall that can only remove elements of the base
if ( Ns ) {
        Index tmpNs;
        IndexVec tmpIs;
        memalloc( tmpIs, Index, Ns );
        CopyCurrentIs( tmpIs );
        hpVec tmpAlpha;
        memalloc( tmpAlpha, hpReal, Ns );
        CopyCurrentPointSparse( tmpAlpha );
        tmpNs = GetNs();
        SolveCurrentBase ();

        d_norm1 = hpZero;
        for (   Index k = 0 ; k < tmpNs ; k++ ){
                if ( PlaceIs[ tmpIs[ k ] ] < Max_n ){
                        // still in the base
                        d_norm1 += abs( tmpAlpha[ k ] - alpha_iter[ PlaceIs[ tmpIs[ k ] ] ] );
                        }
                else if ( PlaceIc[ tmpIs[ k ] ] < Max_n ){
                        // moved to Ic
                        d_norm1 += abs( tmpAlpha[ k ] - theta*C[ tmpIs[ k ] ] );
                        }
                else    {
                        // moved to Io
                        d_norm1 += abs( tmpAlpha[ k ] );
                        }
                }
        delete ( tmpAlpha );
        delete ( tmpIs );
        }
*/



