//---------------------------------------------------------------------------

#include "SVMRBFreal.h"

//---------------------------------------------------------------------------


SVMRBFreal::SVMRBFreal( char *in_file )
		: TQPMemRedCostPath()
{
NumberPoints    = 0;
NumberFeatures  = 0;
DataVal         = NULL;
DataInd         = NULL;
DataNonZero     = NULL;
DataLabels      = NULL;


TestDataVal		= NULL;
TestDataInd     = NULL;
TestDataNonZero = NULL;
TestDataLabels  = NULL;
CcYcKij         = NULL;


Kernel_Type     = 2;
poly_c = 0.0e0;
poly_d = 0.0e0;
sigma           = 1.0e0;
sigmainv        = 1.0e0/(2.0e0*sigma*sigma);

SVMInitialized  = FALSE;

#if ( TimeMode )
		TimeStart();
#endif

//QPErrorMsg SVMStatus = SVMRBFreal::ReadDataSpam ( in_file );
//QPErrorMsg SVMStatus = SVMRBFreal::ReadDataDense ( in_file );
//QPErrorMsg SVMStatus = SVMRBFreal::ReadDataDenseMilind ( "color_correlogram_split1.tdb", "outdoor_split1.tdb"  );
QPErrorMsg SVMStatus = SVMRBFreal::ReadData ( in_file );


if ( SVMStatus != MsgOk ){
		PrintErrorMsg( "Input File Error" );
		return;
		}

PerformanceFunction = 0;
PrintEveryBreakPoint = 20;
PrintEveryIteration  = 100;

// Try to read parameters from PathParameter.par
SVMRBFreal::ReadParameters();


hpVec C0;
hpVec V0;

memalloc(C0, hpReal,  NumberPoints);
memalloc(V0, hpReal,  NumberPoints);
VecInit (C0, 100.0e0, NumberPoints);
VecInit (V0, 1.0e0,   NumberPoints);

TQPMemRedCostPath::TQPMemRedCostPathInit( 2*NumberPoints, NumberPoints, Y0, V0, C0 );

delete (C0);
delete (V0);

#if ( TimeMode )
		TimeFinish();
		TimeAdd( &TimeInit );
#endif


SVMInitialized = TRUE;
}
//---------------------------------------------------------------------------
SVMRBFreal::~SVMRBFreal()
{
delete (Y0);
for ( register Index k = 0 ; k < NumberPoints ; k++ ){
		delete [] DataVal[k];
		delete [] DataInd[k];
		}
delete [] DataVal;
delete [] DataInd;
delete [] DataNonZero;
delete [] DataLabels;

if ( CcYcKij )	delete [] CcYcKij;
if ( TestDataVal ) delete [] TestDataVal;
if ( TestDataInd ) delete [] TestDataInd;
if ( TestDataNonZero ) delete [] TestDataNonZero;
if ( TestDataLabels ) delete [] TestDataLabels;

}

//---------------------------------------------------------------------------
void SVMRBFreal::SolveSVM ()
{

if  ( SVMInitialized ){
        TQPclassPath::SolveQP();
        }
else    {
        PrintErrorMsg("Initialization was not performed.");
        }

}
//---------------------------------------------------------------------------
QPErrorMsg SVMRBFreal::ReadParameters()
{
hpReal aux_real;
IntNumber aux_int;
hpReal kernel_sigma, kernel_poly_c, kernel_poly_d;
FILE *in_par;
int c;

in_par = fopen("PathParameter.par","r");
if ( !in_par ) {
		return (MsgFileError);
		}
else{
	fscanf(in_par,"// Default Parameter File for Path_SVM");
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	fscanf(in_par,"// Parameters for the QP");
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	fscanf(in_par,"%lf Absolute Precision", &aux_real);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPclassPath::SetAbsPrecision(aux_real);
	fscanf(in_par,"%lf Relative Precision", &aux_real);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPclassPath::SetRelPrecision(aux_real);
	fscanf(in_par,"%ld Bias Term (1 == is included)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
        ( aux_int == 1 ) ? SetNoBias ( 0 ) : SetNoBias ( 1 ); 
	fscanf(in_par,"%ld MaxIteration", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPclassPath::SetMaxIteration(aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // jump line

	fscanf(in_par,"// Memory and Reduced Cost Strategy");
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	fscanf(in_par,"%ld MaxSizeNop", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetMaxSizeNop( aux_int );
	fscanf(in_par,"%ld MaxSizeNcp", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetMaxSizeNcp( aux_int );
	fscanf(in_par,"%ld CoutToIopp", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetCountToIopp( aux_int );
	fscanf(in_par,"%ld CoutToIcpp", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetCountToIcpp( aux_int );
	fscanf(in_par,"%ld CoutToIoppp", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetCountToIoppp( aux_int );
	fscanf(in_par,"%ld CoutToIcppp", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetCountToIcppp( aux_int );
	fscanf(in_par,"%ld BlockAllocQ (if <= 0 it is ignored)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	if ( aux_int <= 0 ) TQPMemRedCostPath::SetBlockAllocQ( aux_int );
	fscanf(in_par,"%ld MaxNumberofQcolumns (if 0 it is ignored)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	if ( aux_int <= 0 ) TQPMemRedCostPath::SetMaxNumberofQcolumns( aux_int );
	fscanf(in_par,"%ld BlockAllocL (if <= 0 it is ignored)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	if ( aux_int <= 0 ) TQPclassPath::SetBlockAllocL( aux_int );
	while ( (c = fgetc(in_par)) != '\n') ; // jump line

	fscanf(in_par,"// Evaluating Solution on Test Data (Cross validation parameters)");
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	fscanf(in_par,"%ld DataTestMode", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetDataTestMode( aux_int );
	fscanf(in_par,"%ld PerformanceFunction", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	SetPerformanceFunction( aux_int );
	fscanf(in_par,"%ld BreakPointIntervals	(relevant only if DataTestMode == 2)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetBreakPointIntervals( aux_int );
	fscanf(in_par,"%ld NumberOfTests 		(relevant only if DataTestMode == 1)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetNumberOfTests( aux_int );
	while ( (c = fgetc(in_par)) != '\n') ; // jump line

	fscanf(in_par,"// Exact and Approximate Path Parameters");
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	fscanf(in_par,"%lf Initial Parameter", &aux_real);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	SetInitParameter( aux_real );
	fscanf(in_par,"%lf Final Parameter", &aux_real);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	SetFinalParameter( aux_real );
	fscanf(in_par,"%ld PathMode", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetPathMode( aux_int );
	fscanf(in_par,"%ld MaxSizeNoPath		(relevant only if Approx. Path)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetMaxSizeNoPath( aux_int );
	fscanf(in_par,"%ld MaxSizeNcPath		(relevant only if Approx. Path)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetMaxSizeNcPath( aux_int );
	fscanf(in_par,"%ld BPtoUpdates		(relevant only if Approx. Path)", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetBPtoUpdates( aux_int );
	fscanf(in_par,"%ld MaxToJump", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPMemRedCostPath::SetMaxToJump( aux_int );
	while ( (c = fgetc(in_par)) != '\n') ; // jump line

	fscanf(in_par,"// Output Preferences");
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	fscanf(in_par,"%ld PrintMode", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	TQPclassPath::SetPrintMode ( aux_int  );
	fscanf(in_par,"%ld PrintEveryIteration", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	SetPrintEveryIteration( aux_int );
	fscanf(in_par,"%ld PrintEveryBreakPoint", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	SetPrintEveryBreakPoint( aux_int );
	while ( (c = fgetc(in_par)) != '\n') ; // jump line

	fscanf(in_par,"// Kernel Parameters");
	while ( (c = fgetc(in_par)) != '\n') ; // comments
	fscanf(in_par,"%ld Kernel Mode", &aux_int);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
		fscanf(in_par,"%lf Kernel Parameter sigma", &kernel_sigma);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
		fscanf(in_par,"%lf Kernel Parameter poly c", &kernel_poly_c);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
		fscanf(in_par,"%lf Kernel Parameter poly d", &kernel_poly_d);
	while ( (c = fgetc(in_par)) != '\n') ; // comments
        SetKernel ( aux_int, kernel_sigma, kernel_poly_c, kernel_poly_d );

	fclose(in_par);
	}
return (MsgOk);

}
//----------------------------------------------------------------------------
void     SVMRBFreal::SetKernel ( Index Kernel_Type0, hpReal sigma0, hpReal poly_c0, hpReal poly_d0 )
{
Kernel_Type = Kernel_Type0;
sigma = sigma0;
if ( sigma > 0 ){
		sigmainv = 1.0e0/(2.0e0*sigma*sigma);
		}
poly_c = poly_c0;
poly_d = poly_d0;
}
//---------------------------------------------------------------------------
QPErrorMsg SVMRBFreal::ReadData( char *in_file )
{
FILE *in;
FILE *in_aux;
Index NumNonZero;
int c,clast;

in = fopen(in_file,"r");
if ( in == NULL ) {
		return MsgFileError;
        }

NumberPoints = 0;

while ( (c = fgetc(in)) != EOF ){
		if ( c == '\n' ) NumberPoints++;
		clast = c;
        }
fclose(in);
if ( clast !='\n' ) NumberPoints++;

in = fopen(in_file,"r");
in_aux = fopen(in_file,"r");

memalloc( DataVal, DataVecR, 2*NumberPoints);
memalloc( DataInd, IndexVec, 2*NumberPoints);
memalloc( DataNonZero, Index, 2*NumberPoints);
memalloc( DataLabels, hpReal, 2*NumberPoints);

memalloc( Y0, hpReal, 2*NumberPoints);

IntNumber aux1=0;
DataTypeR aux2=0;
for ( register Index k = 0 ; k < NumberPoints ; k++ ){
		fscanf( in , "%d", &aux1);
        fscanf( in_aux , "%d", &aux1);
        Y0[k] = hpReal ( aux1 );
        DataLabels[ k ] = hpReal ( aux1 );

        NumNonZero = 0;
		while ( (c = fgetc(in_aux)) != '\n' && c != EOF ){
                if ( c == ':' ) {
                        fscanf(in_aux,"%lf", &aux2);
                        if ( abs(aux2)> DataTypeR(0) ) NumNonZero++;        // new feature

						}
				if ( c == '#' ) {
						//fscanf(in_aux,"\n"); // comments...
						while ( (c = fgetc(in_aux)) != '\n' && c != EOF) ; // comments
						}
				}

		memalloc( DataVal[k], DataTypeR, NumNonZero );
		memalloc( DataInd[k], Index, NumNonZero );
		DataNonZero[k] = NumNonZero;
		for ( register Index j = 0 ; j < NumNonZero ; ){
				fscanf(in,"%d:%lf",&aux1,&aux2);
				NumberFeatures = max( NumberFeatures, Index( aux1 ) );
				if ( abs(aux2)> 0  ){
						DataInd[k][j] = aux1;
						DataVal[k][j] = aux2;
						j++;
						}
				}
		while ( (c = fgetc(in)) != '\n' && c != EOF) ; // comments
		}
fclose(in);
fclose(in_aux);
return MsgOk;
}

//----------------------------------------------------------------------------
// will split the data file in the memory into two data sets at random
//
QPErrorMsg SVMRBFreal::SplitDataSet( boolean balance, Index NumTest )
{
//Index NumTrain;
if ( NumTest >= NumberPoints ){
		printf( " File not created (number of desired test points exceed total number of points). \n");
		return MsgOk;
		}

//NumTrain = NumberPoints - NumTest;
if ( balance ){
		IndexVec PositiveLabels;
		IndexVec NegativeLabels;
		Index NumPositive = Index ( 0 );
		Index NumNegative = Index ( 0 );

		hpReal Ratio = hpReal( NumTest )/hpReal(NumberPoints);

		PositiveLabels = new(Index[NumberPoints+1]);
		NegativeLabels = new(Index[NumberPoints+1]);

		for ( Index i = 0 ; i< NumberPoints ; i++ ){
				if ( DataLabels[i] > 0 ){
						PositiveLabels[NumPositive++] = i;
						}
				else    NegativeLabels[NumNegative++] = i;
				}

		RandomizeVector ( PositiveLabels, NumPositive );
		RandomizeVector ( NegativeLabels, NumNegative );

		FILE *out;
		out = fopen("TestData.doc","w");
		Index NumNonZero;
		for ( Index i = 0 ; i < NumPositive*Ratio ; i++ ){
				fprintf(out,"1");
				NumNonZero = DataNonZero[ PositiveLabels[i] ];
				for ( Index j = 0 ; j<NumNonZero ; j++ ){
						fprintf(out," %d:%.4lf", DataInd[ PositiveLabels[i] ][ j ],DataVal[ PositiveLabels[i] ][ j ]);
						}
				fprintf(out,"\n");
				}
		for ( Index i = 0 ; i < NumNegative*Ratio ; i++ ){
				fprintf(out,"-1");
				NumNonZero = DataNonZero[ NegativeLabels[i] ];
				for ( Index j = 0 ; j<NumNonZero ; j++ ){
						fprintf(out," %d:%.4lf", DataInd[ NegativeLabels[i] ][ j ],DataVal[ NegativeLabels[i] ][ j ]);
						}
				fprintf(out,"\n");
				}
		fclose(out);

		out = fopen("TrainData.doc","w");
		for ( Index i = NumPositive - 1 ; i >= NumPositive*Ratio ; i-- ){
				fprintf(out,"1");
				NumNonZero = DataNonZero[ PositiveLabels[i] ];
				for ( Index j = 0 ; j<NumNonZero ; j++ ){
						fprintf(out," %d:%.4lf", DataInd[ PositiveLabels[i] ][ j ],DataVal[ PositiveLabels[i] ][ j ]);
						}
				fprintf(out,"\n");
				}
		for ( Index i = NumNegative - 1 ; i >= NumNegative*Ratio ; i-- ){
				fprintf(out,"-1");
				NumNonZero = DataNonZero[ NegativeLabels[i] ];
				for ( Index j = 0 ; j<NumNonZero ; j++ ){
						fprintf(out," %d:%.4lf", DataInd[ NegativeLabels[i] ][ j ],DataVal[ NegativeLabels[i] ][ j ]);
						}
				fprintf(out,"\n");						
				}
        fclose(out);
        delete [] PositiveLabels;
        delete [] NegativeLabels;
        }
else    { // balance between positive and negative points is not desired
        IndexVec AllLabels;

        AllLabels = new(Index[NumberPoints+1]);

        for ( Index i = 0 ; i< NumberPoints ; i++ ){
                AllLabels[i] = i;
                }

		RandomizeVector ( AllLabels, NumberPoints );

        FILE *out;
		Index NumNonZero;
        out = fopen("TestData.doc","w");
		for ( Index i = 0 ; i < NumTest ; i++ ){
				fprintf(out,"1");
                NumNonZero = DataNonZero[ AllLabels[i] ];
                for ( Index j = 0 ; j<NumNonZero ; j++ ){
						fprintf(out," %d:%.4lf", DataInd[ AllLabels[i] ][ j ],DataVal[ AllLabels[i] ][ j ]);
						}
				}
		fclose(out);

		out = fopen("TrainnigData.doc","w");
		for ( Index i = NumberPoints - 1 ; i >= NumTest ; i-- ){
				fprintf(out,"1");
				NumNonZero = DataNonZero[ AllLabels[i] ];
				for ( Index j = 0 ; j<NumNonZero ; j++ ){
						fprintf(out," %d:%.4lf", DataInd[ AllLabels[i] ][ j ],DataVal[ AllLabels[i] ][ j ]);
						}
                }
        fclose(out);
        delete [] AllLabels;
		}
return MsgOk;
}

//---------------------------------------------------------------------------
QPErrorMsg SVMRBFreal::ReadDataSpam( char *in_file )
{
FILE *in;
FILE *in_aux;
Index NumNonZero;
Index NumFeature;
int c;

// count Number of Points...
in = fopen(in_file,"r");
if ( in == NULL ) return MsgFatal;
NumberPoints = 0;
while ( (c = fgetc(in)) != EOF ){
        if ( c == '\n' ) NumberPoints++;
		}
NumberPoints++; // there is no '\n' on the last line...
fclose(in);

in = fopen(in_file,"r");
if ( in == NULL ) return MsgFatal;
NumFeature = 0;
c = ' ';
while ( c  != '\n' ){
		c = fgetc(in);
        if ( c != ' '  ){
                NumFeature++;
                while ( c != ' ' && c != '\n' ) c = fgetc(in);
                }
        }
fclose(in);

NumFeature--;

in = fopen(in_file,"r");
in_aux = fopen(in_file,"r");

memalloc( DataVal, DataVecR, 2*NumberPoints);
memalloc( DataInd, IndexVec, 2*NumberPoints);
memalloc( DataNonZero, Index, 2*NumberPoints);
memalloc( DataLabels, hpReal, 2*NumberPoints);

memalloc( Y0, hpReal, 2*NumberPoints);

IntNumber aux1=0;
hpReal    aux3=0;

for ( register Index k = 0 ; k < NumberPoints ; k++ ){
        NumNonZero = 0;
        for ( register Index j = 0 ; j < NumFeature ; j++ ){
                fscanf( in_aux , "%lf", &aux3);
                if ( abs(aux3) > GetAbsTolError() ){
                        NumNonZero++;
						}
				}
        fscanf(in_aux,"%d",&aux1);
        if ( aux1 > 0.5 )       Y0[k] = 1.0e0;
        else                    Y0[k] = -1.0e0;

        DataLabels[ k ] = Y0[k];

		if ( k < NumberPoints - 1 )
                while ( (c = fgetc(in_aux)) != '\n' ) ; // comments

        memalloc( DataVal[k], DataTypeR, NumNonZero );
        memalloc( DataInd[k], Index, NumNonZero );
        DataNonZero[k] = NumNonZero;
		Index Feature = 0;
        for ( register Index j = 0 ; j < NumNonZero ; ){
                fscanf(in,"%lf",&aux3);
                if ( abs(aux3) > GetAbsTolError() ){
                        DataInd[k][j] = Feature+1;
                        DataVal[k][j] = aux3;
                        j++;
                        }
                Feature++;
                }
        if ( k == NumberPoints - 1 ) break; // there is no '\n' on the last line...
        c = '0';
        while ( c != '\n' ) { (c = fgetc(in)); }  // comments
        }
fclose(in);
fclose(in_aux);

return MsgOk;
}
//------------------------------------------------------------------------------
//---------------------------------------------------------------------------
QPErrorMsg SVMRBFreal::ReadDataDense( char *in_file )
{
FILE *in;
FILE *in_aux;
Index NumNonZero;
Index NumFeature;
int c;

// count Number of Points...
in = fopen(in_file,"r");
if ( in == NULL ) return MsgFatal;
NumberPoints = 0;
while ( (c = fgetc(in)) != EOF ){
		if ( c == '\n' ) NumberPoints++;
        }
NumberPoints++; // there is no '\n' on the last line...
fclose(in);

in = fopen(in_file,"r");
if ( in == NULL ) return MsgFatal;
NumFeature = 0;
c = ' ';
while ( c  != '\n' ){
        c = fgetc(in);
        if ( c != ' '  ){
                NumFeature++;
                while ( c != ' ' && c != '\n' ) c = fgetc(in);
                }
        }
fclose(in);

NumFeature--;

in = fopen(in_file,"r");
in_aux = fopen(in_file,"r");

memalloc( DataVal, DataVecR, 2*NumberPoints);
memalloc( DataInd, IndexVec, 2*NumberPoints);
memalloc( DataNonZero, Index, 2*NumberPoints);
memalloc( DataLabels, hpReal, 2*NumberPoints);
memalloc( Y0, hpReal, 2*NumberPoints);

IntNumber aux1=0;
hpReal    aux3=0;

for ( register Index k = 0 ; k < NumberPoints ; k++ ){
        NumNonZero = 0;
        for ( register Index j = 0 ; j < NumFeature ; j++ ){
				fscanf( in_aux , "%lf", &aux3);
				NumNonZero++;
                }
        fscanf(in_aux,"%d",&aux1);
        if ( aux1 > 0.5 )       Y0[k] = 1.0e0;
        else                    Y0[k] = -1.0e0;

        DataLabels[k] = Y0[k];

        if ( k < NumberPoints - 1 )
                while ( (c = fgetc(in_aux)) != '\n' ) ; // comments

        memalloc( DataVal[k], DataTypeR, NumNonZero );
        memalloc( DataInd[k], Index, NumNonZero );
        DataNonZero[k] = NumNonZero;
        Index Feature = 0;
        for ( register Index j = 0 ; j < NumNonZero ; ){
				fscanf(in,"%lf",&aux3);
                DataInd[k][j] = Feature+1;
                DataVal[k][j] = aux3;
                j++;
                Feature++;
                }
        if ( k == NumberPoints - 1 ) break; // there is no '\n' on the last line...
        c = '0';
        while ( c != '\n' ) { (c = fgetc(in)); }  // comments
        }
fclose(in);
fclose(in_aux);

return MsgOk;
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//---------------------------------------------------------------------------
QPErrorMsg SVMRBFreal::ReadDataDenseMilind2( char *in_file, char *in_file2 )
{
FILE *in;
FILE *in2;
Index NumNonZero;
Index NumFeature;
int c;
IntNumber aux1=0;
IntNumber aux2=0;
hpReal    aux3=0;

// count Number of Points...
in = fopen(in_file,"r");// color_correlogram...
if ( in == NULL ) return MsgFatal;
NumberPoints = 0;
int JumpLines = 0;
while ( JumpLines < 1 ){
        if ( (c = fgetc(in)) == '\n' || c == EOF ) JumpLines++;
        }
// Read Number of points: in file goes from 0 to aux2-1
fscanf(in,"  %d  %d\n",&aux1,&aux2);
NumberPoints = aux2;

// Read Number of Features
fscanf(in,"%d  %d ",&aux1,&aux2);
NumberFeatures = aux2;
NumFeature = NumberFeatures;
fclose(in);

// Open both the file...
in  = fopen(in_file,"r");// color_correlogram...
in2 = fopen(in_file2,"r");// outdoor_split...
if ( in == NULL ) return MsgFatal;
if ( in2 == NULL ) return MsgFatal;
JumpLines = 0;
while ( JumpLines < 2 ){
        if ( (c = fgetc(in)) == '\n' || c == EOF ) JumpLines++;
        }
JumpLines = 0;
while ( JumpLines < 2 ){
        if ( (c = fgetc(in2)) == '\n' || c == EOF ) JumpLines++;
		}

// allocate memory for points...
memalloc( DataVal, DataVecR, NumberPoints);
memalloc( DataInd, IndexVec, NumberPoints);
memalloc( DataNonZero, Index, NumberPoints);
memalloc( DataLabels, hpReal, NumberPoints);
memalloc( Y0, hpReal, NumberPoints);

for ( register Index k = 0 ; k < NumberPoints ; k++ ){

// ignore first two integer entries
        fscanf( in, "%d %d", &aux1, &aux2);
        NumNonZero = NumberFeatures; // here aux2 == NumberFeatures
// read values of the features...
        memalloc( DataVal[k], DataTypeR, NumberFeatures );
        memalloc( DataInd[k], Index, NumberFeatures );
        Index Feature = 0;
		DataNonZero[k] = 0;

        for ( register Index j = 0 ; j < NumberFeatures ; j++, Feature++ ){
				fscanf(in," %lf",&aux3);
                if ( abs(aux3) > GetAbsTolError() ) {
                        DataInd[k][DataNonZero[k]]   = Feature+1;
                        DataVal[k][DataNonZero[k]++] = aux3;
                        }
                }

// read Label from in2.....
        while ( (c = fgetc(in2)) != '#' ) ;
        fscanf(in2," %lf\n", &aux3);
		if ( aux1 > 0.5 )       Y0[k] = 1.0e0;
        else                    Y0[k] = -1.0e0;

        DataLabels[k] = Y0[k];

        if ( k < NumberPoints - 1 )
                while ( (c = fgetc(in)) != '\n' ) ; // comments
		}
fclose(in);
fclose(in2);

return MsgOk;
}
//---------------------------------------------------------------------------
QPErrorMsg SVMRBFreal::ReadDataDenseMilind( char *in_file, char *in_file2 )
{
FILE *in;
FILE *in2;
Index NumNonZero;
Index NumFeature;
int c;
IntNumber aux1=0;
IntNumber aux2=0;
hpReal    aux3=0;

// count Number of Points...
in = fopen(in_file,"r");// color_correlogram...
if ( in == NULL ) return MsgFatal;
NumberPoints = 0;
int JumpLines = 0;
while ( JumpLines < 1 ){
        if ( (c = fgetc(in)) == '\n' || c == EOF ) JumpLines++;
        }
// Read Number of points: in file goes from 0 to aux2-1
fscanf(in,"  %d  %d\n",&aux1,&aux2);
NumberPoints = aux2;

// Read Number of Features
fscanf(in,"%d  %d ",&aux1,&aux2);
NumberFeatures = aux2;
NumFeature = NumberFeatures;
fclose(in);

// Open both the file...
in  = fopen(in_file,"r");// color_correlogram...
in2 = fopen(in_file2,"r");// outdoor_split...
if ( in == NULL ) return MsgFatal;
if ( in2 == NULL ) return MsgFatal;
JumpLines = 0;
while ( JumpLines < 2 ){
        if ( (c = fgetc(in)) == '\n' || c == EOF ) JumpLines++;
        }
JumpLines = 0;
while ( JumpLines < 2 ){
        if ( (c = fgetc(in2)) == '\n' || c == EOF ) JumpLines++;
        }

// allocate memory for points...
memalloc( DataVal, DataVecR, NumberPoints);
memalloc( DataInd, IndexVec, NumberPoints);
memalloc( DataNonZero, Index, NumberPoints);
memalloc( DataLabels, hpReal, NumberPoints);
memalloc( Y0, hpReal, NumberPoints);

for ( register Index k = 0 ; k < NumberPoints ; k++ ){

// ignore first two integer entries
        fscanf( in, "%d %d", &aux1, &aux2);
        NumNonZero = NumberFeatures; // here aux2 == NumberFeatures
// read values of the features...
        memalloc( DataVal[k], DataTypeR, NumberFeatures );
        memalloc( DataInd[k], Index, NumberFeatures );
        DataNonZero[k] = NumberFeatures;
		Index Feature = 0;
        for ( register Index j = 0 ; j < NumberFeatures ; j++ ){
                fscanf(in," %lf",&aux3);
				DataInd[k][j] = Feature+1;
                DataVal[k][j] = aux3;
                Feature++;
                }
// read Label from in2.....
        while ( (c = fgetc(in2)) != '#' ) ;
		fscanf(in2," %lf\n", &aux3);
		if ( aux3 > 0.5 )       Y0[k] = 1.0e0;
		else                    Y0[k] = -1.0e0;
        DataLabels[k] = Y0[k];
        if ( k < NumberPoints - 1 )
                while ( (c = fgetc(in)) != '\n' ) ; // comments
        }
fclose(in);
fclose(in2);

return MsgOk;
}
//------------------------------------------------------------------------------
void SVMRBFreal::WriteDataSparseSVMLight()
{
FILE *out1;

out1 = fopen("adult-a.txt","w");

for ( Index i = 0 ; i < NumberPoints ; i++ ){
		fprintf( out1, "%d ",int(DataLabels[i]));
        for ( Index j = 0 ; j < DataNonZero[ i ]; j++ )
                fprintf( out1, "%d:%.8f ", DataInd[i][j], DataVal[i][j] );
		fprintf(out1,"\n");
        }
fclose(out1);
}
//-----------------------------------------------------------------------------
void SVMRBFreal::WriteDataSparse()
{
FILE *out1;
FILE *out2;

out1 = fopen("ttt_pnt.txt","w");
IntNumber Pnt = 0;
fprintf( out1, "%ld\n",NumberFeatures);
fprintf( out1, "%ld\n",NumberPoints);
fprintf( out1, "0\n");
for ( Index i = 0 ; i < NumberPoints ; i++ ){
		Pnt += DataNonZero[i];
		fprintf( out1, "%.8f\n",DataLabels[i]);
        fprintf( out1, "%ld\n",Pnt );
        }
fclose(out1);

out2 = fopen("ttt_sp.txt","w");
for ( Index i = 0 ; i < NumberPoints ; i++ ){
        for ( Index k = 0 ; k < DataNonZero[i] ; k++ ){
                fprintf( out2, "%ld\n",   DataInd[i][k] );
                fprintf( out2, "%.18f\n", double(DataVal[i][k]) );
                }
        }
fclose(out2);
}
//-----------------------------------------------------------------------------
void SVMRBFreal::WriteDenseColumn()
{
FILE *out;

out = fopen("face_densettt_pnt.txt","w");
for ( Index i = 0 ; i < NumberPoints ; i++ ){
		fprintf( out, "%.8f\n",DataLabels[i]);
        for ( Index k = 0 ; k < DataNonZero[i] ; k++ ){
				fprintf( out, "%d\n",   int(DataInd[i][k]) );
                fprintf( out, "%.18f\n", double(DataVal[i][k]) );
                }
        }
fclose(out);
}

//-----------------------------------------------------------------------------
void SVMRBFreal::WriteFullDense()
{
FILE *out;

out = fopen("fulldense.txt","w");
fprintf( out, "%ld\n",NumberFeatures);
fprintf( out, "%ld\n",NumberPoints);
for ( Index i = 0 ; i < NumberPoints ; i++ ){
		fprintf( out, "%.8f\n",DataLabels[i]);
		for ( Index k = 0 ; k < DataNonZero[i] ; k++ ){
				fprintf( out, "%.18f\n", double(DataVal[i][k]) );
				}
		}
fclose(out);
}
//----------------------------------------------------------------------------
QPErrorMsg	SVMRBFreal::LoadTestData( char *in_file )
{
FILE *in;
FILE *in_aux;
Index NumNonZero;
int c,clast;

in = fopen(in_file,"r");
if ( in == NULL ) {
		return MsgFileError;
		}

TestNumberPoints = 0;

while ( (c = fgetc(in)) != EOF ){
		if ( c == '\n' ) TestNumberPoints++;
		clast = c;
		}
fclose(in);
if ( clast !='\n' ) TestNumberPoints++;

in = fopen(in_file,"r");
in_aux = fopen(in_file,"r");

memalloc( TestDataVal, DataVecR, 1+ TestNumberPoints);
memalloc( TestDataInd, IndexVec, 1+ TestNumberPoints);
memalloc( TestDataNonZero, Index,1+ TestNumberPoints);
memalloc( TestDataLabels, hpReal,1+ TestNumberPoints);

IntNumber aux1=0;
DataTypeR aux2=0;

for ( register Index k = 0 ; k < TestNumberPoints ; k++ ){
		fscanf( in , "%d", &aux1);
		fscanf( in_aux , "%d", &aux1);
		Y0[k] = hpReal ( aux1 );
		TestDataLabels[ k ] = hpReal ( aux1 );

		NumNonZero = 0;
		while ( (c = fgetc(in_aux)) != '\n' && c != EOF ){
				if ( c == ':' ) {
						fscanf(in_aux,"%lf", &aux2);
						if ( abs(aux2)> DataTypeR(0) ) NumNonZero++;        // new feature

						}
				if ( c == '#' ) {
						//fscanf(in_aux,"\n"); // comments...
						while ( (c = fgetc(in_aux)) != '\n' && c != EOF) ; // comments
						}
				}

		memalloc( TestDataVal[k], DataTypeR, NumNonZero );
		memalloc( TestDataInd[k], Index, NumNonZero );
		TestDataNonZero[k] = NumNonZero;
		for ( register Index j = 0 ; j < NumNonZero ; ){
				fscanf(in,"%d:%lf",&aux1,&aux2);
				//NumberFeatures = max( NumberFeatures, Index( aux1 ) );
				if ( abs(aux2)> 0  ){
						TestDataInd[k][j] = aux1;
						TestDataVal[k][j] = aux2;
						j++;
						}
				}
		while ( (c = fgetc(in)) != '\n' && c != EOF) ; // comments
		}


fclose(in);
fclose(in_aux);

memalloc ( CcYcKij , hpReal, 1+ TestNumberPoints);


return MsgOk;
}

//------------------------------------------------------------------------
void    SVMRBFreal::InitTestDataIc()
{
for ( Index j = 0 ; j < TestNumberPoints; j++ ){
	CcYcKij[j] = hpZero;
	for ( Index i = 0 ; i < GetNc() ; i++ )
		CcYcKij[j] += GetCi( GetIc(i) ) * DataLabels[ GetIc(i) ] * Kernel_i_test_j ( GetIc(i), j );
	}
}
//--------------------------------------------------------------------
void 	SVMRBFreal::UpdateTestDataIc( Index i, TypeEvent Event )
{
if ( Event == Ic2Is ) {
	for ( Index j = 0 ; j < TestNumberPoints; j++ )
		CcYcKij[j] -= GetCi( i ) * DataLabels[ i ] * Kernel_i_test_j ( i, j );
	}
else if ( Event == Is2Ic ) {
	for ( Index j = 0 ; j < TestNumberPoints; j++ )
		CcYcKij[j] += GetCi( i ) * DataLabels[ i ] * Kernel_i_test_j ( i, j );
	 }
}
//----------------------------------------------------------------------------
hpReal   SVMRBFreal::ComputingPerformanceOnTestData()
{
hpReal classifier_val;
hpReal performance_val = hpZero;
for ( Index j = 0 ; j < TestNumberPoints; j++ ){
		classifier_val = - Getb();//hpZero;
		for ( Index i = 0 ; i < GetNs() ; i++ )
				classifier_val += GetAlpha( i ) * DataLabels[ GetIs(i) ] * Kernel_i_test_j ( GetIs(i), j );
		classifier_val += GetTheta() * CcYcKij[j];

		//for ( Index i = 0 ; i < GetNc() ; i++ )
		//		classifier_val += GetTheta() * GetCi( GetIc(i) ) * DataLabels[ GetIc(i) ] * Kernel_i_test_j ( GetIc(i), j );

		performance_val += FunctionPerformance( classifier_val, TestDataLabels[j] );
		}
return performance_val;
}
//-----------------------------------------------------------------------
hpReal   SVMRBFreal::FunctionPerformance( hpReal classifier, hpReal Label )
{
switch(PerformanceFunction){
        case 0: // L0 penalty
				if ( classifier * Label > 0 )
                        return 0.0e0;
                else return 1.0e0;
				//break;
		case 1: // L1 penalty
				if ( classifier * Label > 0 )
						return 0.0e0;
				else return fabs( classifier );
				//break;
		default:
				return 0.0e0;
				//break;
        }
return 0.0e0;
}
//-----------------------------------------------------------------------
void SVMRBFreal::UpdateEveryIteration ()
{

#if ( TimeMode )
        TimeTotal();
#endif

#if ( PrintVisual )
if ( GetIteration() % 100 == 0 || GetMinReducedCost() > - GetAbsTolError() )
        UpdateFormLabels ( Getn(), GetNs(), GetNo(), GetNc(), GetDegreeDependence(),
                        GetIteration(), GetInnerIter(), Getf_obj(), Getb(), GetStepSize(),
						GetCurrentAllocQ(), GetNq(), GetCurrentAllocL(),
                        GetNop(), GetNcp(), GetNewIndexToBeAdded(), GetMinReducedCost(), GetRedCostBar(),
                        GetCase0Count(), GetCase1Count(), GetCase2Count(), GetInfiniteDirectionCount(),
						GetNumBreakPoints(), GetNumJumps(), GetTheta(), GetInitParameter() , GetFinalParameter() , GetNumberOfTiesStep2(),
                        Get_f_deriv(), Getb_deriv(), Getr1r1(), Getr1z1() );

   #if ( PrintVisual && TimeMode )
   if ( GetIteration() % 100 == 0 || GetMinReducedCost() > - GetAbsTolError() )
        UpdateFormTimeLabels ( TimeKKTsystem, TimeReduceCost, TimeAddToBase, TimeRemoveFromBase,
                           TimeInit, TimeTotalAll, GetTheta() );
   #endif

#else

if ( GetIteration() % 50 == 0 ){
		printf( "Iteration:%d Inner:%d f=%8lf b=%8lf Ns=%d No=%d Nc=%d \n", GetIteration(), GetInnerIter(), Getf_obj(), Getb(), GetNs(), GetNo(), GetNc());

		#if ( TimeMode )
		if ( TimeTotalAll > 0 ){
				printf( "Total Time: %.1lf (KKT %.2lf, RedCost %.2lf Add %.2lf Remove %.2lf Init %.2lf)\n",
				TimeTotalAll, 100*(TimeKKTsystem)/TimeTotalAll,100*(TimeReduceCost)/TimeTotalAll,100*(TimeAddToBase)/TimeTotalAll,100*(TimeRemoveFromBase)/TimeTotalAll,100*(TimeInit)/TimeTotalAll);
				}
		else    printf( "Total Time too close to zero \n");
		#endif

		}

#endif

}

//----------------------------------------------------------------------------
void SVMRBFreal::UpdateEveryIterationPath ()
{

#if ( TimeMode )
        TimeTotal();
#endif

#if ( PrintVisual )
if ( GetNumBreakPoints() % 1 == 0  )
		UpdateFormLabels ( Getn(), GetNs(), GetNo(), GetNc(), GetDegreeDependence(),
                        GetIteration(), GetInnerIter(), Getf_obj(), Getb(), GetStepSize(),
                        GetCurrentAllocQ(), GetNq(), GetCurrentAllocL(),
                        GetNop(), GetNcp(), GetNewIndexToBeAdded(), GetMinReducedCost(), GetRedCostBar(),
                        GetCase0Count(), GetCase1Count(), GetCase2Count(), GetInfiniteDirectionCount(),
						GetNumBreakPoints(), GetNumJumps(), GetTheta(), GetInitParameter() , GetFinalParameter() , GetNumberOfTies(),
                        Get_f_deriv(), Getb_deriv(), Getr1r1(), Getr1z1() );


  #if ( PrintVisual && TimeMode )
  if ( GetNumBreakPoints() % 1 == 0 )
        UpdateFormTimeLabels ( TimeKKTsystemQP, TimeReduceCostQP, TimeAddToBaseQP, TimeRemoveFromBaseQP,
                           TimeInitQP, TimeTotalAllQP, GetTheta() );
        UpdateFormTimePathLabels ( TimeBreakStep1, TimeBreakStep2, TimeUpdates, TimeTotalAll );

  #endif

#else


if ( GetNumBreakPoints() % 20 == 0 ){
        printf( " BreakPoint = %d, Theta = %f, f_obj = %f, Ns = %d, Nc = %d \n", GetNumBreakPoints(), GetTheta(), Getf_obj(), GetNs(), GetNc() );

        #if ( TimeMode )
        if ( TimeTotalAll > 0 ){
                printf( " Total Time: %.1lf ( TB-Step1 %.2lf, TB-Step2 %.2lf, T-Updates %.2lf) \n",
                TimeTotalAll, 100*(TimeBreakStep1)/TimeTotalAll,100*(TimeBreakStep2)/TimeTotalAll,100*(TimeUpdates)/TimeTotalAll);
				}
        else    printf( " Total Time too close to zero \n");
        #endif

        }




#endif

}

//----------------------------------------------------------------------------



//----------------------------------------------------------------------------
hpReal SVMRBFreal::GetDataQij ( Index i, Index j )     // Compute only 1 element
{

hpReal kij = Kernel_ij ( i, j );

// Need to multiply by the labels in the classificaiton problem
return hpReal (DataLabels[i]*DataLabels[j]*kij );

}

//----------------------------------------------------------------------------
void    SVMRBFreal::GetDataQi ( hpVec Qi, Index i)     // Compute Complete Row
{
for ( register Index j = NumberPoints ; j-- ; )
        Qi[j] = SVMRBFreal::GetDataQij( i, j );
}

//----------------------------------------------------------------------------

hpReal SVMRBFreal::Kernel_ij ( Index i, Index j )
{
register IndexVec Indi = DataInd[i];
register IndexVec Indj = DataInd[j];
register Index NZi = DataNonZero[i];   // const
register Index NZj = DataNonZero[j];   // const
register DataVecR DataVali = DataVal[i];
register DataVecR DataValj = DataVal[j];

register Index ki = 0;
register Index kj = 0;
register DataTypeR t = DataTypeR( 0 );
register hpReal kij = hpReal( 0 );
register DataTypeR tmp;

switch ( Kernel_Type ){

        case 1: // linear...
                for (  ; ki < NZi && kj < NZj ;  ){
                        if      ( *Indi > *Indj ){
                                Indj++;
                                kj++;
                                DataValj++;
                                }
                        else if ( *Indi < *Indj ){
                                Indi++;
                                ki++;
                                DataVali++;
                                }
                        else    { // Indi == Indj
                                //t+= DataVali[ki]*DataValj[kj];
                                t+= (*(DataVali++)) * (*(DataValj++));
                                Indi++;
                                Indj++;
                                ki++;
                                kj++;
                                }
						}
                kij = t;
				break;

        case 2: // RBF...
				for (  ; ki < NZi && kj < NZj ;  ){
						if      ( *Indi > *Indj ){
                                //tmp = DataValj[kj++];
                                tmp = *(DataValj++);
                                kj++;
								t += tmp*tmp;
                                Indj++;
                                }
                        else if ( *Indi < *Indj ){
                                //tmp = DataVali[ki++];
                                tmp = *(DataVali++);
                                ki++;
                                t += tmp*tmp;
                                Indi++;
								}
                        else    { // Indi == Indj
                                //tmp = ( DataVali[ki++] - DataValj[kj++] );
                                tmp =  (*(DataVali++)) - (*(DataValj++)) ;
                                ki++;
                                kj++;
                                t +=  tmp*tmp;
                                Indi++;
                                Indj++;
                                }
                        }

                for ( ; kj < NZj ; ){
                        //tmp = DataValj[kj++];
                        tmp = *(DataValj++);
                        kj++;
                        t += tmp*tmp;
                        }
                for ( ; ki < NZi ; ){
                        //tmp = DataVali[ki++];
                        tmp = *(DataVali++);
						ki++;
                        t += tmp*tmp;
                        }
				kij = exp( - t * (sigmainv) );
                break;

        case 3: // Inhomogeneous Polynomial
                for (  ; ki < NZi && kj < NZj ;  ){
						if      ( *Indi > *Indj ){
                                Indj++;
                                kj++;
                                DataValj++;
                                }
                        else if ( *Indi < *Indj ){
                                Indi++;
                                ki++;
                                DataVali++;
                                }
                        else    { // Indi == Indj
                                //t+= DataVali[ki]*DataValj[kj];
                                t+= (*(DataVali++)) * (*(DataValj++));
                                Indi++;
                                Indj++;
                                ki++;
                                kj++;
                                }
                        }
                kij = pow( poly_c + t,poly_d);
                break;
        case 4: // Additional Kernel...
                // Enter the code for the new kernel...
                //...


                //...
                break;

        default:
				break;

        }
return kij;
}


//----------------------------------------------------------------------------

hpReal SVMRBFreal::Kernel_i_test_j ( Index i, Index j )
{
register IndexVec Indi = DataInd[i];
register IndexVec Indj = TestDataInd[j];
register const Index NZi = DataNonZero[i];
register const Index NZj = TestDataNonZero[j];
register DataVecR DataVali = DataVal[i];
register DataVecR DataValj = TestDataVal[j];

register Index ki = 0;
register Index kj = 0;
register DataTypeR t = DataTypeR( 0 );
register hpReal kij = hpReal( 0 );
register DataTypeR tmp;

switch ( Kernel_Type ){

        case 1: // linear...
                for (  ; ki < NZi && kj < NZj ;  ){
                        if      ( *Indi > *Indj ){
                                Indj++;
                                kj++;
                                DataValj++;
                                }
                        else if ( *Indi < *Indj ){
                                Indi++;
                                ki++;
                                DataVali++;
                                }
						else    { // Indi == Indj
                                //t+= DataVali[ki]*DataValj[kj];
                                t+= (*(DataVali++)) * (*(DataValj++));
                                Indi++;
                                Indj++;
                                ki++;
                                kj++;
                                }
						}
                kij = t;
                break;

        case 2: // RBF...
				for (  ; ki < NZi && kj < NZj ;  ){
                        if      ( *Indi > *Indj ){
								//tmp = DataValj[kj++];
                                tmp = *(DataValj++);
                                kj++;
                                t += tmp*tmp;
                                Indj++;
                                }
                        else if ( *Indi < *Indj ){
                                //tmp = DataVali[ki++];
                                tmp = *(DataVali++);
                                ki++;
                                t += tmp*tmp;
                                Indi++;
                                }
                        else    { // Indi == Indj
                                //tmp = ( DataVali[ki++] - DataValj[kj++] );
                                tmp =  (*(DataVali++)) - (*(DataValj++)) ;
                                ki++;
                                kj++;
                                t +=  tmp*tmp;
                                Indi++;
                                Indj++;
                                }
                        }

                for ( ; kj < NZj ; ){
                        //tmp = DataValj[kj++];
                        tmp = *(DataValj++);
                        kj++;
                        t += tmp*tmp;
                        }
                for ( ; ki < NZi ; ){
						//tmp = DataVali[ki++];
                        tmp = *(DataVali++);
                        ki++;
                        t += tmp*tmp;
                        }
                kij = exp( - t * (sigmainv) );
                break;

        case 3: // Inhomogeneous Polynomial
                for (  ; ki < NZi && kj < NZj ;  ){
						if      ( *Indi > *Indj ){
                                Indj++;
								kj++;
                                DataValj++;
                                }
                        else if ( *Indi < *Indj ){
                                Indi++;
                                ki++;
                                DataVali++;
                                }
                        else    { // Indi == Indj
                                //t+= DataVali[ki]*DataValj[kj];
                                t+= (*(DataVali++)) * (*(DataValj++));
                                Indi++;
                                Indj++;
                                ki++;
                                kj++;
                                }
                        }
                kij = pow( poly_c + t,poly_d);
				break;
        case 4: // Additional Kernel...
                // Enter the code for the new kernel...
                //...


                //...
                break;

        default:
                break;

        }
return kij;
}



//----------------------------------------------------------------------------
hpReal   SVMRBFreal::GetMaxQi( Index i )                // return a bound on max | Q_ij |
{

switch ( Kernel_Type ){

        case 1: // linear...
				return hpInf;
//                break;

		case 2: // RBF...
				return 1.0e0;
//                break;

		case 3: // Inhomogeneous Polynomial
				return hpInf;
//                break;
		case 4: // Additional Kernel...
				// Enter the code for the new kernel...
				//...

				//...
				return hpInf;
//                break;

		default:
				return hpInf;
//            	break;
		}

return hpInf;
}
//----------------------------------------------------------------------------
boolean  SVMRBFreal::PositiveKernel()
{
return TRUE;
}
//----------------------------------------------------------------------------
hpReal SVMRBFreal::ClassifyPoint( IndexVec Indj, Index NZj, DataVecR DataValj )
{
register hpReal val = hpReal( 0 );
register Index length;
length = GetNs();
for ( register Index i = 0 ; i < length ; i++ ){
        val += GetYi( i ) * GetAlpha( i ) * Kernel_i_eval ( GetIs(i), Indj, NZj, DataValj );
        }
length = GetNc();
for ( register Index i = 0 ; i < length ; i++ ){
		val += GetYi( i ) * GetCi( i ) * Kernel_i_eval ( GetIs(i), Indj, NZj, DataValj );
        }
return ( val >= 0 ) ? 1.0e0 : -1.0e0;

}
//----------------------------------------------------------------------------

hpReal SVMRBFreal::Kernel_i_eval ( Index i, IndexVec Indj, Index NZj, DataVecR DataValj )
{
register IndexVec Indi = DataInd[i];
register const Index NZi = DataNonZero[i];
register DataVecR DataVali = DataVal[i];

register Index ki = 0;
register Index kj = 0;
register DataTypeR t = DataTypeR( 0 );
register hpReal kij = hpReal( 0 );
register DataTypeR tmp;

switch ( Kernel_Type ){

        case 1: // linear...
                for (  ; ki < NZi && kj < NZj ;  ){
                        if      ( *Indi > *Indj ){
                                Indj++;
                                kj++;
                                DataValj++;
                                }
                        else if ( *Indi < *Indj ){
                                Indi++;
                                ki++;
                                DataVali++;
                                }
                        else    { // Indi == Indj
                                //t+= DataVali[ki]*DataValj[kj];
                                t+= (*(DataVali++)) * (*(DataValj++));
                                Indi++;
                                Indj++;
                                ki++;
                                kj++;
                                }
                        }
                kij = t;
                break;

        case 2: // RBF...
                for (  ; ki < NZi && kj < NZj ;  ){
                        if      ( *Indi > *Indj ){
								//tmp = DataValj[kj++];
                                tmp = *(DataValj++);
                                kj++;
                                t += tmp*tmp;
                                Indj++;
                                }
                        else if ( *Indi < *Indj ){
                                //tmp = DataVali[ki++];
								tmp = *(DataVali++);
                                ki++;
                                t += tmp*tmp;
                                Indi++;
                                }
                        else    { // Indi == Indj
                                //tmp = ( DataVali[ki++] - DataValj[kj++] );
                                tmp =  (*(DataVali++)) - (*(DataValj++)) ;
                                ki++;
                                kj++;
                                t +=  tmp*tmp;
                                Indi++;
                                Indj++;
                                }
                        }

                for ( ; kj < NZj ; ){
                        //tmp = DataValj[kj++];
                        tmp = *(DataValj++);
                        kj++;
                        t += tmp*tmp;
                        }
                for ( ; ki < NZi ; ){
                        //tmp = DataVali[ki++];
                        tmp = *(DataVali++);
                        ki++;
                        t += tmp*tmp;
                        }
                kij = exp( - t * (sigmainv) );
                break;

        case 3: // Inhomogeneous Polynomial
                for (  ; ki < NZi && kj < NZj ;  ){
                        if      ( *Indi > *Indj ){
                                Indj++;
                                kj++;
                                DataValj++;
                                }
						else if ( *Indi < *Indj ){
                                Indi++;
                                ki++;
                                DataVali++;
                                }
                        else    { // Indi == Indj
                                //t+= DataVali[ki]*DataValj[kj];
                                t+= (*(DataVali++)) * (*(DataValj++));
                                Indi++;
                                Indj++;
                                ki++;
                                kj++;
                                }
                        }
                kij = pow( poly_c + t,poly_d);
                break;
        case 4: // Additional Kernel...
                // Enter the code for the new kernel...
                //...


                //...
                break;

        default:
                break;

        }
return kij;
}

