//---------------------------------------------------------------------------

#ifndef SVMRBFrealH
#define SVMRBFrealH
//---------------------------------------------------------------------------
#include "BasicInit.h"
#include "QP_MemRedCostPath.h"

#if ( PrintVisual )
        #include "Path_SVM_Code.h"
#endif

//----------------------------------------------------------------------------
// Type of Data... integer, float...
typedef hpReal          DataTypeR;                // integer numbers
typedef DataTypeR        *DataVecR;                // vector of integers
typedef DataVecR         *DataMatR;                // matrix of integers

class SVMRBFreal : public TQPMemRedCostPath {

public:
                        SVMRBFreal( char *in_file );
                        ~SVMRBFreal();
        void            SolveSVM ();

        // Write Data on Different Formats...
        void            WriteDataSparse();
        void            WriteDataSparseSVMLight();
        void            WriteDenseColumn();
        void            WriteFullDense();
		inline Index    GetNumberFeatures();
		inline Index    GetNumberPoints();
        inline void     SetSigma ( hpReal val );

        void     SetKernel ( Index Kernel_Type0, hpReal sigma, hpReal poly_c, hpReal poly_d );

        hpReal          ClassifyPoint( IndexVec Indj, Index NZj, DataVecR DataValj );
		QPErrorMsg      SplitDataSet( boolean balance, Index NumTest );
		QPErrorMsg		LoadTestData( char *in_file );
		hpReal          ComputingPerformanceOnTestData();
		inline boolean	Initialized();
protected:
        // Reading Data...
        QPErrorMsg      ReadData      ( char *in_file );
        QPErrorMsg      ReadDataSpam  ( char *in_file );
        QPErrorMsg      ReadDataDense ( char *in_file );
        QPErrorMsg      ReadDataDenseMilind( char *in_file, char *in_file2  );
        QPErrorMsg      ReadDataDenseMilind2( char *in_file, char *in_file2  );

		QPErrorMsg		ReadParameters();

		inline void		SetPerformanceFunction( IntNumber val );
		void    		UpdateTestDataIc( Index i, TypeEvent Event );
		void			InitTestDataIc();
		// Display information...
        void            UpdateEveryIteration ();
        void            UpdateEveryIterationPath ();

		inline	void	SetPrintEveryBreakPoint ( IntNumber val );
		inline	void	SetPrintEveryIteration ( IntNumber val );
		IntNumber		PrintEveryBreakPoint;
		IntNumber		PrintEveryIteration;

        // Final Interface between Data and Q...
        hpReal          GetDataQij( Index i, Index j );     // Compute only 1 element
        void            GetDataQi ( hpVec Qi, Index i);     // Compute Complete Row
        hpReal          GetMaxQi( Index i );                // return a bound on max | Q_ij |
        boolean         PositiveKernel();

        // Data Set...
		Index           NumberPoints;
        Index           NumberFeatures;
        DataMatR        DataVal;
        IndexMat        DataInd;
        IndexVec        DataNonZero;
        hpVec           DataLabels;

        hpVec           Y0;

        // Test Data Set...
		Index           TestNumberPoints;
		DataMatR        TestDataVal;
		IndexMat        TestDataInd;
		IndexVec        TestDataNonZero;
		hpVec           TestDataLabels;
		Index           PerformanceFunction;
		hpVec			CcYcKij;

        hpReal          FunctionPerformance( hpReal classifier, hpReal Label );

        // Kernel Information...
        hpReal          Kernel_ij ( Index i, Index j );
        hpReal          Kernel_i_eval ( Index i, IndexVec Indj, Index NZj, DataVecR DataValj );
        hpReal          Kernel_i_test_j ( Index i, Index j);

		// kernel type
		Index Kernel_Type;

		// kernel parameters
		// RBF...
		hpReal          sigma;
		hpReal          sigmainv;
		// Inhomogeneous Polynomial...
		hpReal poly_c;
		hpReal poly_d;


private:
		boolean         SVMInitialized;

};
//----------------------------------------------------------------------------
inline void     SVMRBFreal::SetSigma( hpReal val )
{
if ( sigma > 0 ){
		sigma = val;
		sigmainv = 1.0e0/(2.0e0*sigma*sigma);
		}
else    {
		sigma = 1.0e0;
		sigmainv = 0.5;
		}
}
//----------------------------------------------------------------------------
inline Index    SVMRBFreal::GetNumberPoints()
{
return NumberPoints;
}
//----------------------------------------------------------------------------
inline Index    SVMRBFreal::GetNumberFeatures()
{
return NumberFeatures;
}
//----------------------------------------------------------------------------
inline boolean	SVMRBFreal::Initialized()
{
	return SVMInitialized;
}
//----------------------------------------------------------------------------
inline 	void SVMRBFreal::SetPerformanceFunction( IntNumber val )
{
PerformanceFunction = val;
}
//----------------------------------------------------------------------------
inline	void	SVMRBFreal::SetPrintEveryBreakPoint ( IntNumber val )
{
PrintEveryBreakPoint = val;
}
//----------------------------------------------------------------------------
inline	void	SVMRBFreal::SetPrintEveryIteration ( IntNumber val )
{
PrintEveryIteration = val;
}


#endif
